WAComponent subclass: #EmailParserRootComponent
	instanceVariableNames: 'rootDirectory selectedAccount accountParser statusText'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EmailParser-WebUI'!
!EmailParserRootComponent commentStamp: 'RF 2/22/2009 14:33' prior: 0!
The email parser code was developed as part of the Collaborative Electronic Records Project (CERP), see http://siarchives.si.edu/cerp/.  This code is distributed as Open Source Software under the Squeak license  (http://www.squeak.org/SqueakLicense/)  and the MIT license  (http://www.opensource.org/licenses/mit-license.php).  10/22/2008

The MIT license follows:

Copyright (c) 2008 Rockefeller Archive Center
Copyright (c) 2008 Smithsonian Institution Archives

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.




This models the outer pane of the Squeak Seaside WebUI for running the Email Parser.

It assumes (for now at least) that all Accounts accessible for parsing from this UI live in one root directory (default name C:\digpres\email).

The main functionality is to choose an Account folder/directory, start parsing with a button press, and then, when desired, view the progress of the parse.

rootDirectory -- the directory that holds all accounts to be parsed.  Default 'C:\digipres\email'
accounts -- a SortedCollection (alphabetically) of subdirectory names (strings) in the rootDirectory
selectedAccount -- the name (String) of whichever account, if any, has been selected for parsing
accountParser -- the instance of Account to be used for parsing
forkedParser -- the instance of Process that has been forked off to do the parse

The Seaside-generated web functionality is (at present) set to be available on port 9091.  To start seaside
execute the following:
		WAKom startOn: 9091
To stop it, execute:"
		WAKom stop
Unless the user stops Seaside, it is unaffected by saving/quiting Squeak.  As soon as Squeak is restarted,
it restores the Seaside access to port 9091.  However, if the image is moved to a different machine, that may not be true.  So, the first time it is run on a new machine it would be best to stop, then restart Seaside.

The web page's background image is served, at this time, by a Comanche (Squeak's web server code)
instance running on port 9090.  To start Comanche, use the following code
		| oldService ma |
		oldService := TcpService serviceOnPort: 9090 ifAbsent: [nil].
		oldService notNil ifTrue: [oldService stop].
		ma := ModuleAssembly core.
		ma serverRoot: FileDirectory default fullName.	"change this if you want a different root directory"
		ma documentRoot: FileDirectory default fullName.	"determines where it looks for docs"
		ma directoryIndex: 'index.html index.htm'.			"determines the default page to be served"
		ma serveFiles.
		(HttpService startOn: 9090 named: 'httpd') plug: ma rootModule
		
Be sure that the background image, named: MailAccountBkgrd.gif is in a directory named 'images' located in whatever root directory has been designated.  The above code sets the root directory as the same directory in
which the Squeak.exe and Squeak image live.

Note, there seems to be some sort of race/priority problem with serving status updates to the Web interface.  It
probably has to do with the fact that the parser is running and eating up lots of compute resources at the same time as both the Comanche and the Seaside servers.  The symptom is that in long-running parses, if you press the status update button on the Web page, it may update, or may not.  Once it fails to update, it seem frozen thereafter until the parse finishes.  I have tried playing with process priorities to no avail.  There is little documentation on the Web about this issue although others have commented on it.  For now I just don't rely on the status update.  For long parses I prefer to run them from within Squeak rather than using the Web interface.
(SLB, 9/2/08)!


!EmailParserRootComponent methodsFor: 'parsing' stamp: 'SLB 10/4/2008 16:04'!
prepareForParse
	"<^ (true | false)>"
	"Check to ensure that the prior parse, if any, is complete.  Ensure that any results of
	 any prior parse are removed or parse is aborted.  If all's well, return true, else false."

	| fileDirectory |
	(self selectedAccount = 'NO ACCOUNTS'
		or: [self selectedAccount = 'ERROR: Missing Email_Accounts Directory'])
			ifTrue: [^false].		"if no legal account directory chosen, abort"
	(self accountParser notNil and: [self accountParser isParsing])
		ifTrue: [^false].		"if parse still active, abort"
	fileDirectory := FileDirectory on:
			self rootDirectory, FileDirectory pathNameDelimiter asString, self selectedAccount.
	(fileDirectory fileExists: selectedAccount, '.xml')
		ifTrue:
			[(self confirm: 'Already Parsed.  Reparse?')
				ifTrue:
					[Transcript cr; show: 'Prior parse results exist...erasing old results'; flush.
					 fileDirectory deleteFileNamed: selectedAccount, '.xml'.
					 Account cleanupOldParseResultsIn: fileDirectory.
					 self accountParser: (Account forRootFolder: fileDirectory)]
				ifFalse:
					[self inform: 'Parsing aborted'.
					 Transcript cr; show: 'Prior parse results exist...aborting reparse'; flush.
					 ^false]]
		ifFalse:
			[self accountParser: (Account forRootFolder: fileDirectory)].
	^true! !

!EmailParserRootComponent methodsFor: 'parsing' stamp: 'SLB 5/6/2008 22:03'!
refreshStatus
	"Ask the account for it's latest status text."

	self accountParser notNil
		ifTrue: [self statusText: self accountParser parseStatusStream contents].! !

!EmailParserRootComponent methodsFor: 'parsing' stamp: 'SLB 5/12/2008 22:11'!
startParse
	"If all's well, start the parse"

	self accountParser notNil
		ifTrue:
			[[self accountParser orchestrateParsing]
					forkAt: Processor userBackgroundPriority - 1]! !


!EmailParserRootComponent methodsFor: 'rendering' stamp: 'SLB 10/6/2008 20:50'!
renderContentOn: html
	"<html: hOf WAHtmlCanvas>"
	"Render the outer window stuff"

	html div id: 'setup'; with:
		[html heading: 'E-Mail Account Parsing'; break.
		 html paragraph: [html text: 'All Account directories to be parsed -- and no subdirectories other than properly formed Account directories -- should be located in a directory named "Email_Accounts" which itself must be located in the same directory as the Parser software.
Each Account directory must contain all folder subdirectories that you wish included in the parse.  Examples might include InBox and/or Sent folders.
Any folder  may itself contain subdirectories representing sub-folders.
Within any folder or subfolder, the file containing email messages to be parsed (one mbox file per folder subdirectory) must be in "mbox" format, with extension ".mbox".'.
		html break; break; strong; text: 'Choose the account directory you wish to parse from the following drop-down list of available candidate accounts that appear to be well-formed.'.
		html break; break; strong; text: 'Once you have chosen the desired target account, press the "Proceed with parsing" button. If the chosen account has already been parsed, you will be asked whether or not you wish to reparse it.'].
		html anchor
			url: 'http://localhost:9090/index.html';
			with: 'See help pages for more detail'.
		html break.
		html break.
		html form:
			[self renderParseInitiationTableOn: html.
			 self renderParseStatusTableOn: html] ].

! !

!EmailParserRootComponent methodsFor: 'rendering' stamp: 'SLB 10/3/2008 21:58'!
renderParseInitiationTableOn: html
	"<html: hOf WAHtmlCanvas>"
	"Render the table for picking the account and starting the parse."

	html table: [
		html tableRow: [
			html tableData: 'Choose Account: '.
			html tableData: [	
				html select
					beSubmitOnChange;
					list: self accounts;
					selected: selectedAccount;
					callback: [ :value | self selectedAccount: value ] ] ].
		html tableRow: [
			html tableData: ' '.
			html tableData: [
				html submitButton text: 'Proceed with parsing';
								callback:
									[self prepareForParse
										ifTrue: [self startParse]
										ifFalse: [self inform: 'Cannot begin new parse'] ] ] ] ].

! !

!EmailParserRootComponent methodsFor: 'rendering' stamp: 'SLB 5/8/2008 14:52'!
renderParseStatusTableOn: html
	"<html: hOf WAHtmlCanvas>"
	"Render the table for viewing the parse."

	html table: [
			html tableRow: [html tableData: [html break; break] ].
			html tableRow: [
				html tableData: 'Current Parse Status          '.
				html tableData: [
					html submitButton callback:
						[self refreshStatus]; text: 'Refresh Status' ] ].
			html tableRow: [
				html tableData: ' '.
				html tableData: [
					html html: self statusTextAsHtml ] ] ].

! !

!EmailParserRootComponent methodsFor: 'rendering' stamp: 'SLB 5/18/2008 21:35'!
style
	"Return css style information for the page"

	^ '
h1 {text-align: center;
	font-size: 48;
	line-height: 0%;
	margin-top: .2cm;
	margin-bottom: .5cm;}
body {background-color: #DDDDFF; margin: 5px;
       background-image: url(http://localhost:9090/images/MailAccountBkgrd.gif); }
p {font-family: Times, serif;
	font-size: 14px;
	color: #000000;}
#setup {background-color: #FDFDFD; 
	position:absolute;
		top: 1.9cm;
		right: 10%;
		left: 18%;
	padding: 15px .25cm;
	border: medium single rgb(0,0,50);
	border-style: solid; }'! !


!EmailParserRootComponent methodsFor: 'queries' stamp: 'SLB 10/4/2008 16:00'!
accounts
	"Getter for the collection of subdirectories of the rootDirectory, if any"

	| legalAccounts |
	self checkRootDirectoryPath
		ifTrue:
			[legalAccounts := self getLegalAccounts.
			 legalAccounts isEmpty
				ifTrue: [legalAccounts add: 'NO ACCOUNTS']]
		ifFalse:
			[legalAccounts := OrderedCollection with: 'ERROR: Missing Email_Accounts Directory'].
	^legalAccounts! !

!EmailParserRootComponent methodsFor: 'queries' stamp: 'SLB 10/3/2008 22:01'!
checkRootDirectoryPath
	"Answer true if the designated Email_Accounts directory exists.
	 NOTE: This doesn't check
	 that the directory is a legal root account tree."

	^(FileDirectory default directoryExists: 'Email_Accounts')! !

!EmailParserRootComponent methodsFor: 'queries' stamp: 'SLB 10/4/2008 16:08'!
getLegalAccounts
	"<^ hOf SortedCollection {of: iOf String}>"
	"collect and vet the subdirectories of the rootDirectory, if any"

	| candidates |
	candidates := (FileDirectory on: self rootDirectory) directoryNames asSortedCollection.
	^candidates select: [ :candidate | self isLegalAccount: candidate ]! !

!EmailParserRootComponent methodsFor: 'queries' stamp: 'SLB 6/19/2008 22:26'!
isLegalAccount: candidatePath
	"<candidate: iOf String, ^(true | false)>"
	"Answer true iff candidate is a directory tree at least 2 deep containing
	 at least one subdirectory at the second level or below that contains
	 esactly one file with the extension .mbox."

	| candidateAccount |
	candidateAccount := FileDirectory on: self rootDirectory,
						FileDirectory pathNameDelimiter asString,
						candidatePath.
	candidateAccount withAllSubdirectoriesCollect: [ :subDirectory |
		(subDirectory fileNamesMatching: '*.mbox') size = 1
			ifTrue: [^true]].
	^false! !


!EmailParserRootComponent methodsFor: 'convenience' stamp: 'SLB 5/12/2008 12:55'!
statusTextAsHtml
	"Answer statusText converted to html and with tabs replaced by 3 spaces."

	| convertedString |
	self statusText size > 1
		ifFalse:
			[^ 'No parse status available']
		ifTrue:
			[convertedString := self statusText copyReplaceAll: '	' with: '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;'.
			convertedString := convertedString copyReplaceAll: '
' with: '<BR>'].
	^convertedString! !


!EmailParserRootComponent methodsFor: 'accessing' stamp: 'SLB 4/13/2008 20:10'!
accountParser
	"Getter for the Account instance that is parsing the email data"

	^accountParser! !

!EmailParserRootComponent methodsFor: 'accessing' stamp: 'SLB 4/13/2008 20:10'!
accountParser: anAccount
	"Setter for the Account instance that is parsing the email data"

	accountParser := anAccount! !

!EmailParserRootComponent methodsFor: 'accessing' stamp: 'SLB 4/3/2008 20:02'!
children
	"This method should return a collection of all subcomponents of the current component that will be rendered in #renderContentOn:. Components that are displayed using #call: are *not* children.

	If the contents returned by this method change over time make sure to return them in #states as well, otherwise the back button will fail."

	^ #()! !

!EmailParserRootComponent methodsFor: 'accessing' stamp: 'SLB 10/3/2008 22:14'!
rootDirectory
	"Getter for a string representation of the root directory,default '.\Email_Accounts' "

	rootDirectory isNil
		ifTrue:
			[rootDirectory := FileDirectory default pathName, 
			 FileDirectory default pathNameDelimiter asString , 'Email_Accounts'].
	^rootDirectory! !

!EmailParserRootComponent methodsFor: 'accessing' stamp: 'SLB 4/11/2008 22:49'!
selectedAccount
	"Getter for the selected subdirectories of the rootDirectory, if any"

	^selectedAccount! !

!EmailParserRootComponent methodsFor: 'accessing' stamp: 'SLB 4/11/2008 22:49'!
selectedAccount: aString
	"Setter for the selected subdirectories of the rootDirectory, if any"

	selectedAccount := aString! !

!EmailParserRootComponent methodsFor: 'accessing' stamp: 'SLB 5/15/2008 19:39'!
statusText
	"Getter for the contents of the account's parseStatusStream"

	| folderStatus |
	self accountParser isNil
		ifTrue:
			[statusText := ' ']
		ifFalse:
			[statusText := self accountParser parseStatusStream contents.
			 self accountParser parsingStatus = #Parsing
				ifTrue:
					[folderStatus := self accountParser activeFolder lastMessageParsed printString.
					statusText := statusText, ' ... ', folderStatus, ' messages parsed']].
	^statusText! !

!EmailParserRootComponent methodsFor: 'accessing' stamp: 'SLB 5/6/2008 21:57'!
statusText: aString
	"Setter for the contents of the account's parseStatusStream"

	statusText := self accountParser parseStatusStream contents! !


!EmailParserRootComponent methodsFor: 'initialize-release' stamp: 'SLB 10/3/2008 21:51'!
initialize
	"Set defaults if any"

	super initialize.! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

EmailParserRootComponent class
	instanceVariableNames: ''!

!EmailParserRootComponent class methodsFor: 'testing' stamp: 'SLB 4/28/2008 22:24'!
canBeRoot
	"When returning true, the component can be registered as a standalone application from the config interface."

	^ true! !
