ProtoObject subclass: #SHA1Plugin
	instanceVariableNames: ''
	classVariableNames: 'K'
	poolDictionaries: ''
	category: 'SHA1-Plugin'!
!SHA1Plugin commentStamp: 'RF 2/22/2009 14:34' prior: 0!
The email parser code was developed as part of the Collaborative Electronic Records Project (CERP), see http://siarchives.si.edu/cerp/.  This code is distributed as Open Source Software under the Squeak license  (http://www.squeak.org/SqueakLicense/)  and the MIT license  (http://www.opensource.org/licenses/mit-license.php).  10/22/2008

The MIT license follows:

Copyright (c) 2008 Rockefeller Archive Center
Copyright (c) 2008 Smithsonian Institution Archives

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.




Note, for this code to be modified or used to alter the plugin, it must be a subclass of InterpreterPlugin which is seldom loaded into a normal image.  Lacking that, the SHA1Plugin will appear to be a subclass of ProtoObject.

A SHA1Plugin contains the code for a primitive version of the SHAOne hash function.
!


!SHA1Plugin methodsFor: 'system primitives' stamp: 'SLB 12/24/2007 09:41'!
primTransform
	"Slang version of the SHA1>>transform method.  The stack has no args, only the receiver.
	 instVar: byteBuffer (a ByteArray of length 64) is at offset 1 
	 instVar: state (a WordArray of length 5) is at offset 3 
	 instVar: savedState (a WordArray of length 5) is at offset 4 
	 instVar: workBuffer (a WordArray of length 80) is at offset 6"

	| rcvrOop state savedState byteBuffer workBuffer j k t |
	self var: #byteBuffer declareC: 'unsigned char *byteBuffer'.
	self var: #state declareC: 'unsigned int *state'.
	self var: #savedState declareC: 'unsigned int *savedState'.
	self var: #workBuffer declareC: 'unsigned int *workBuffer'.
	self var: #t declareC: 'unsigned int t'.
	self var: #k declareC: 'unsigned int k'.
	self export: true.
	rcvrOop := interpreterProxy stackObjectValue: 0.
	byteBuffer := interpreterProxy fetchArray: 1 ofObject: rcvrOop.
	state := interpreterProxy fetchArray: 3 ofObject: rcvrOop.
	savedState := interpreterProxy fetchArray: 4 ofObject: rcvrOop.
	workBuffer := interpreterProxy fetchArray: 6 ofObject: rcvrOop.

	0 to: 15 do: [ :i |   "Do bufferConvert.  Move 512bit 'chunk' in byteBuffer to workBuffer."
		j := i * 4.
		workBuffer at: i put: ((((byteBuffer at: (3 + j))
						bitOr: (byteBuffer at: 2 + j) << 8)
						bitOr: (byteBuffer at: 1 + j) << 16)
						bitOr: (byteBuffer at: j)     << 24)].

	16 to: 79 do: [ :i |	"Extend sixteen words to 80 words"
		t := ((((workBuffer at: i - 3)
					bitXor: (workBuffer at: i - 8))
					bitXor: (workBuffer at: i - 14))
					bitXor: (workBuffer at: i - 16)).
		workBuffer at: i put:
			(((t << 1) bitAnd: 16rFFFFFFFF) bitOr: (t >> 31))].    "this line is a logical left rotate by 1"

	0 to: 4 do: [ :i | savedState at: i put: (state at: i)].		"save old hash value"

	"THE MAIN LOOP"
	0 to: 79 do: [ :i |
		i < 20 ifTrue:
			[k := 16r5A827999.
			"NOTE: the bitXor: 16rFFFFFFFF below is equivalent to bitInvert32"
			t := ((state at: 1) bitAnd: (state at: 2))
					bitOr: (((state at: 1) bitXor: 16rFFFFFFFF) bitAnd: (state at: 3))].

		(i >= 20 and: [i <= 39]) ifTrue:
			[k := 16r6ED9EBA1.
			 t := ((state at: 1) bitXor: (state at: 2)) bitXor: (state at: 3)].

		(i >= 40 and: [i <= 59]) ifTrue:
			[k := 16r8F1BBCDC.
			 t := (((state at: 1) bitAnd: (state at: 2)) bitOr: ((state at: 1) bitAnd: (state at: 3))) 
					bitOr: ((state at: 2) bitAnd: (state at: 3))].
		i >= 60 ifTrue:
			[k := 16rCA62C1D6.
			 t := ((( state at: 1) bitXor: (state at: 2)) bitXor: (state at: 3))].

		t := (t + ((((state at: 0) << 5) bitAnd: 16rFFFFFFFF) bitOr: ((state at: 0) >> 27)) "leftRotate 5"
					 + ((state at: 4) + k + (workBuffer at: i)))
					 bitAnd: 16rFFFFFFFF.
		state at: 4 put: (state at: 3).
		state at: 3 put: (state at: 2).
		state at: 2 put: ((((state at: 1) << 30) bitAnd: 16rFFFFFFFF) bitOr: ((state at: 1) >> 2)).
		state at: 1 put: ( state at: 0 ).
		state at: 0 put: t].

	0 to: 4 do: [ :i |		"add prior hash into this chunk's hash"
		state at: i put: (((state at: i) + (savedState at: i)) bitAnd: 16rFFFFFFFF)].! !


SHA1 subclass: #SHAOne
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SHA1-Plugin'!

!SHAOne methodsFor: 'private' stamp: 'SLB 12/23/2007 22:03'!
transform
	"Do the SHA1 transform method in a primitive if possible"

	<primitive: 'primTransform' module: 'SHA1Plugin'>
	^SHA1Plugin doPrimitive: 'primTransform'! !


SHA1Test subclass: #SHAOneTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SHA1-Plugin'!

!SHAOneTest methodsFor: 'accessing' stamp: 'SLB 12/17/2007 20:49'!
digest
	"getter for test purposes"

	^digest! !
