﻿Object subclass: #Account
	instanceVariableNames: 'globalId rootDirectory folders xmlOutputFileStream parseStatusStream parsingStatus activeFolder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EmailParser-Model'!
!Account commentStamp: 'RAF 8/5/2009 14:45' prior: 0!
The email parser code was developed as part of the Collaborative Electronic Records Project (CERP), see http://siarchives.si.edu/cerp/.  This code is distributed as Open Source Software under the Squeak license  (http://www.squeak.org/SqueakLicense/)  and the MIT license  (http://www.opensource.org/licenses/mit-license.php).  10/22/2008 

The MIT license follows:

Copyright (c) 2008 Rockefeller Archive Center
Copyright (c) 2008 Smithsonian Institution Archives

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
R. Ferrante 8/1/2009

(See note about implementation at the end of the descriptive text.)

This class models an email Account, a container possibly with multiple folders each containing multiple email messages.  An Account maps to one person and one email client.  There may be many folders, each with subfolders in a person's email Account.

The parser is "Model=Driven" rather than syntax-driven.  By that I mean that parsing co-evolves with the creation of the Document Object Model (DOM) that is the result of the parse.  A root Account object is created that populates itself, creates Folder objects that populate themselves which, in turn, create Message objects (RFC2822MIMEMessage objects) which, in turn create and populate Headers, MessageBodies, etc.  Each object in the eventual DOM is handed the input FileStream, parses out what it needs and passes it on to the next object.  So parsing "smarts" is distributed amongst the objects. Then, at appropriate junctures, e.g., when each Message has been parsed, the DOM outputs the XML parse results according to the mail-account.xsd schema.  Thus, each object in the DOM tree "knows" both how to parse itself and how to render itself in XML.  This strategy for parsing is especially appropriate given that email "syntax" has evolved to be very complex and is interpreted rather differently by different developers of email clients.  A model-driven parser can distribute the many rule exceptions into the appropriate DOM objects where they "make sense" (at least somewhat) in the context of each object.  However, those whose experience with parsing has been with code that traverses some syntax tree may find this code confusing at first.

The assumptions about the Account input are that a set of mbox files representing the messages in those folders are in a directory tree under the Account directory (which is specified at the start of parsing).  That directory is the "root" of the Account.  The XML output file resulting from the parse is placed in this root Account directory and named 'foldername.xml' where foldername is the name of the top level Account directory.  The directory tree maps to the email user's folder tree.  Top level folders, e.g., Sent and Inbox, are represented by directories of the same name in the Account directory.  The raw RFC2822 mbox format messages for a given folder (i.e., in a given directory with that folder's name) are in a file within that directory named 'xxx.mbox'.  Note that xxx.mbox was by convention 'messages.mbox' but that has been relaxed in later parser versions so that 'messages' can reflect some other purpose. Note however that the name of the mbox file appears nowhere in the parser output so the purpose of naming the mbox something other than messages.mbox would presumably be something to do with the logistics of preparing the AIP or making backups with more descriptive file names.

If an account happens to contain only one large undifferentiated mbox file, the archivist will have to concoct a name for the folder it is presumed to have lived within and create a directory with that folder name in which to place the xxx.mbox file.  If, in scanning the messages, it seems that the Account holder is the sender of them all, it should be named 'Sent'.  If the Account holder is an addressee of all the messages, it should ce called 'InBox'.  If the majority of the messages' Subject lines mention some common topic, create a name from the topic.  Otherwise perhaps the archivist will have to fall back on a name like 'Miscellaneous Messages'

When the parser processes the messages in a given messages.mbox file, it stores large body parts, typically base64-encoded attachments, in the same directory where the "xxx.mbox' file resides.  The names of the attachment files are of the form: 'attachnnnnnn.xml' where nnnnnn is a 16 digit random number that is referenced in the main XML file. These random number are a temporary replacement for unique handle IDs from a true handle server which we do not yet have.  The filename tells you nothing about what message it was attached to nor the filename of the attached file. Attachment filenames cannot be relied upon as unique IDs because a different version of the same file (or even the same version) with the same name may be attached to another message in the folder.

By convention and usage in most email clients, an mbox file contains email from only one folder. Outlook, Groupwise and Lotus Notes do not follow that convention (nor do they store mail in mbox format).  Those systems use other file types, such as the .pst used in Outlook or the .nsf used in Notes that may contain mail for multiple folders.  However those formats are proprietary, sometimes mysterious, poorly documented (if at all) and subject to change as new versions of the mail software are shipped.  For example, Microsoft Outlook 2007 reputedly has a storage format dramatically different from the .pst format and the .pst format itself has changed over the years.  Yet software is available to convert these proprietary formats into standard mbox format. So, at present, the assumption made for this parser is that mail is in mbox format and that Folders (i.e., directories) and mbox files have a one-to-one relationship.

A SLIGHTLY NON-STANDARD METHOD IN CHARACTER
The version of Squeak in which this code was originally developed (v3.2) did not have a method in the class Character for testing for space/tab.  So I added one in the 'testing' protocol of Character:

	isWhitespace
		"Answer whether the receiver is space or tab. SLB 4/24/2007 10:30"

		value = 32 ifTrue: [^true].	"space"
		value = 9 ifTrue: [^true].	"tab"
		^false

Two other basic methods are in v3.2 but is not in v3.9.  Depending on the Squeak
version you are dealing with, you may have to add them too.
In the 'testing' protocol of Stream

	isExternalStream
		^false

And in the 'testing' protocol of FileStream

	isExternalStream
		^true

Given that oddity, there may be more methods this code relies upon that are missing in other versions (Why? who knows).

A ChangeSet named EmailParser-Misc contains the above methods and two other methods:
	Character>>isEmailDateTimeSeparator  and
	PositionableStream>>skipEmailDateTimeSeparators
for parsing email date headers.

AN IMPLEMENTATION NOTE ABOUT CROSS-PLATFORM EOL CONVENTIONS:
This parser is *intended* to function equally well on Windows, Linux, and Macintoshes.  That brings into play the different conventions on those three platforms for marking the end of line.  Windows uses CrLf (a two character end of line marker), *nix uses just Lf, and Macs use just Cr.Note that it was developed in Windows.  Some testing has been done on Linux.  No testing on Mac yet.  Given the differences in EOL and in file/directory naming conventions, it is likely that there are a few bugs to be found in using it in either Linux or Macs.

When writing the XML output file, the parser uses an instance of CrLfFileStream which ensures that the XML file uses whatever EOL is native to the machine it runs on.  So the output will be most easily used on the same operating system as the one used to parse the email.  That way, all text processing tools (editors, XML parsers, etc.) on the host OS can work with the XML output

Internally, whatever the host OS, Squeak's streams use #cr for end-of-line because Squeak is a descendent of Apple's version of Smalltalk-80.  When reading email text, the parsing methods use 'Stream nextLine' and the code depends upon that marker being cr.  When writing the resulting XML it adds a cr at the end of each line but then the CrLfFileStream converts that to the native convention.

A NOTE ON HASHING:
The Mail-Account.xsd schema allows for storing a hash value for each method and does not specify a means for storing a hash on larger groups, e.g., an mbox file or even an account.  This is somewhat at variance with common hashing practices, which are typically on a per-file basis.  The reasons are at least two fold. First, the North Carolina Archives usecase involves the transfer of individual messages from the State employees to the archive, so it is necessary to be able to handle hash checksums per message.  Second, when and if provenance is required, it is the message, not the mailbox, that is the unit of interest.

IMPLICATIONS OF THE EOL CONVENTION ON HASHING:
for purposes of computing per-method hash values the hash must be computed over the exact character stream of the input mbox file, i.e., prior to any EOL conversion.  Because the EOL convention of the original file (as presented to the parser, not necessarily to the email client that created the email) can differ from that of the host parsing OS, the EOL convention must be detected and documented in the XML output so that hashes may be compared accurately.

A NOTE ABOUT IMPLEMENTATION HISTORY -- some disclaimers about implementation style:
This email parsing application was built as a prototype. One consequence is that it grew organically.  Email has many ideosyncracies and complexities, both theoretical (as specified by various specs such as in RFC822 and RFC2822) and actual (as generated by a wide variety of email clients that take liberties with and interpret differently the RFC2822 specs).  In addition, it co-evolved not only with a growing understanding of the complexities of actual email, but also along with the XML schema into which it renders the parsed email. David Minor, of the North Carolina State Archives, built a first pass at such a schema in early 2007 (called SEmail.xsd).  Some of the complexities of actual email as discovered during development of this parser (most notably recursive multi-part-bodies) required changes to the schema. As incompatabilities accumulated, David was led to do a substantial rewrite.  The second generation schema (called Mail-Account.xsd) captured much more detail about headers, added the Account and Folder tags, and handled more elegantly the recursion of multi-part-bodies.  Those changes, in turn, required substantial changes to the parser, some of which uncovered other weaknesses in the schema.  Almost a dozen versions of the schema (11 as of this writing, SLB 11/17/2007 17:19) were made in the process of revising the parser and the schema so that they were compatible and could handle the many complexities discovered in real email.  What tended to suffer during this several months-long iterative process was code quality/clenliness and the factoring of the parser class hierarchy.

The Email-Parser class hierarchy was built initially to match closely the first schema (SEmail). A substantial body of email (something like 100Mb of mail from various kinds of email clients) was tested with that parser. So when it came to revisions to output the new schema (Mail-Account) I did not want to change the parsing code which worked pretty well.  Rather than refactoring the class hierarchy, which would necessitate rewriting much of the parsing code, I changed just the XML output code.  That turned out to be somewhat awkward as the iterative changes accumulated, but the timing of the CERP project didn't allow a complete refactoring and/or rewrite.  For several reasons, not least because I did not understand the full sweep of email specs and variations in interpretation of those specs until well into the project, the Class Hierarch is very flat, i.e., lacking much inheritance.  There could perhaps be more use of inheritance now, but the CERP project needs to move on and it isn't a good use of my time to do a refactoring for cleanliness.

Method complexity/size also suffered. In part because of the deficiences in the Class Hierarchy and in part because of the organic growth of the parser and schema, there are many more overlarge methods than there should be.  I am especially embarassed about this state of affairs because I wrote a detailed paper (published in the IBM Systems Journal, 1996, http://www.findarticles.com/p/articles/mi_m0ISJ/is_n2_v35/ai_18503982) about the evils of overly complex methods.  So I can't claim ignorance.  All I can say is that, given the opportunity, I'd like to revisit this code and clean it up.  But the odds aren't good that I will get the chance.  SLB 11/17/2007 17:35!


!Account methodsFor: 'parsing' stamp: 'SLB 6/19/2008 15:45'!
orchestrateParsing
	"Initiate XML output, traverse the tree of folders and parse their mbox files, then finish
	 and close the XML output file.  ParsingStatus must be #New at the start, #Parsing during
	 the parse, and left as #Complete on exit."

	self openXMLOutputFileStream.
	self printXMLPreambleOn: self xmlOutputFileStream.
	self parseStatusStream nextPutAll: 'Parsing initiated ', Date today printString, ' for ';
						   nextPutAll: self rootDirectory fullName.
	self parsingStatus: #Parsing.
	self folders isEmpty
		ifTrue: [EmailXMLOutputException signal: 'No folders in this account']
		ifFalse: [self folders do: [ :aFolder |
			self activeFolder: aFolder.
			aFolder parseAndPrintXMLOn: self xmlOutputFileStream enclosingIndentLevel: 1]].
	self printXMLCloseOn:  self xmlOutputFileStream.
	self xmlOutputFileStream close.
	self parsingStatus: #Complete.
	self saveParseStatusToFile
	! !


!Account methodsFor: 'printing' stamp: 'SLB 2/10/2008 18:33'!
printXMLCloseOn: aStream
	"Append the portion of myself that goes after the Account contents."

	aStream cr; nextPutAll: '</Account>'.! !

!Account methodsFor: 'printing' stamp: 'SLB 12/10/2008 11:04'!
printXMLPreambleOn: aStream
	"Append the XML description of the portion of myself that precedes the email messages.
	 It is assumed that Account is the top-level parent class. We will include only one
	 Account in each output file."

	aStream nextPutAll: '<?xml version="1.0" encoding="UTF-8"?>'.
	aStream cr; nextPutAll: '<Account xmlns="http://www.archives.ncdcr.gov/mail-account"'.
	aStream cr; nextPutAll: '  xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"'.
	aStream cr; nextPutAll: '  xsi:schemaLocation="http://www.archives.ncdcr.gov/mail-account.xsd">'.
	aStream crtab; nextPutAll: '<GlobalId> ', self globalId, ' </GlobalId>'.! !


!Account methodsFor: 'file management' stamp: 'SLB 8/20/2008 21:23'!
cleanupLooseEnds
	"Close files.  This is intended to be used manually from a
	 Workspace or Transcript when a parse has crashed and
	 may have left some open files.  It does not (yet) check
	 for open temp files or attachment files."

	self xmlOutputFileStream notNil
		ifTrue: [self xmlOutputFileStream close].
	self folders do: [ :aFolder | aFolder summaryFileStream close].
	MboxFileReader allInstances do: [ :aFileReader |
		aFileReader mboxFileStream close]! !

!Account methodsFor: 'file management' stamp: 'SLB 12/13/2008 12:16'!
openXMLOutputFileStream
	"Open and return the output file stream."

	| newFileName delimiter |
	delimiter := FileDirectory pathNameDelimiter.
	newFileName := self rootDirectory pathName, delimiter asString,
					(self rootDirectory pathName copyAfterLast: delimiter), '.xml'.
	(FileStream isAFileNamed: newFileName)
		ifTrue: [(PopUpMenu confirm: 'file already exists, OK to overwrite?')
			ifTrue:
				[Account cleanupOldParseResultsIn: self rootDirectory.
				 self xmlOutputFileStream: (CrLfFileStream forceNewFileNamed: newFileName)]
			ifFalse:
				[EmailXMLOutputException signal: 'Output file name already in use']]
		ifFalse: [self xmlOutputFileStream: (CrLfFileStream newFileNamed: newFileName)].
	self xmlOutputFileStream converter: Latin1TextConverter new.
	^self xmlOutputFileStream! !

!Account methodsFor: 'file management' stamp: 'SLB 5/15/2008 21:13'!
saveParseStatusToFile
	"Open new file and write the parseStatusStream to the file."

	| statusFileName statusFileStream delimiter |
	delimiter := FileDirectory pathNameDelimiter.
	statusFileName := self rootDirectory pathName, delimiter asString, 'parseStatus.txt'.
	statusFileStream := (CrLfFileStream forceNewFileNamed: statusFileName).
	statusFileStream nextPutAll: self parseStatusStream contents.
	statusFileStream close.
! !


!Account methodsFor: 'queries' stamp: 'SLB 5/11/2008 22:11'!
isParsing
	"Answer whether a parse is in progress"

	^self parsingStatus = #Parsing! !


!Account methodsFor: 'accessing' stamp: 'SLB 5/15/2008 17:18'!
activeFolder
	"<^ hOf Folder>"
	"Getter"

	^activeFolder! !

!Account methodsFor: 'accessing' stamp: 'SLB 5/15/2008 17:18'!
activeFolder: aFolder
	"Setter"

	activeFolder := aFolder! !

!Account methodsFor: 'accessing' stamp: 'SLB 5/13/2007 16:56'!
folders
	"<^ hOf OrderedCollection {of: hOf Folder}>"
	"laissez faire getter for folders"

	folders isNil
		ifTrue: [folders := OrderedCollection new].
	^folders! !

!Account methodsFor: 'accessing' stamp: 'SLB 8/11/2007 15:16'!
globalId
	"<^ iOf String>"
	"Lassez faire getter for the Global Id.  At present it returns a string that mimics a handle.
	 When we make it more official, it will return a legal handle URI, e.g.,
	 'hdl://nnnnn.Account.CERPHandleServer@CERP.org'"
	"Account new globalId inspect"

	globalId isNil
		ifTrue: [globalId := 999999999 atRandom printString , '.Account.fake.CERPHandleServer@CERP.org'].
	^globalId! !

!Account methodsFor: 'accessing' stamp: 'SLB 8/10/2007 15:47'!
globalId: aString
	"Setter for the Global Id.  At present it uses a string.  When
	 we make it more official, it will use a legal handle URI, e.g., 'hdl://nnnnn.Account.CERPHandleServer@CERP.org'"

	globalId := aString! !

!Account methodsFor: 'accessing' stamp: 'SLB 5/5/2008 11:44'!
parseStatusStream
	"<^ hOf ReadWriteStream>"
	"Laissez faire getter"

	parseStatusStream isNil
		ifTrue: [parseStatusStream := ReadWriteStream with: String new].
	^parseStatusStream! !

!Account methodsFor: 'accessing' stamp: 'SLB 5/5/2008 11:41'!
parseStatusStream: aReadWriteStream
	"setter"

	parseStatusStream := aReadWriteStream! !

!Account methodsFor: 'accessing' stamp: 'SLB 5/12/2008 12:40'!
parsingStatus
	"<^ hOf Symbol>"
	"Laissez Faire getter sets default parse status"
	
	parsingStatus isNil
		ifTrue: [parsingStatus := #New].
	^parsingStatus! !

!Account methodsFor: 'accessing' stamp: 'SLB 6/19/2008 17:49'!
parsingStatus: aSymbol
	"Setter.  The arg must be one of #New, #Parsing, or #Complete"

	(aSymbol = #New or: [aSymbol = #Parsing] or: [aSymbol = #Complete])
		ifTrue: [parsingStatus := aSymbol]! !

!Account methodsFor: 'accessing' stamp: 'SLB 8/20/2007 22:05'!
rootDirectory
	"<^ hOf FileDirectory>"
	"Getter for the directory of the root Account"

	^rootDirectory! !

!Account methodsFor: 'accessing' stamp: 'SLB 8/20/2007 22:05'!
rootDirectory: aFileDirectory
	"<aFileDirectory: hOf FileDirectory, ^self>"
	"Setter for the directory of the root Account"

	rootDirectory := aFileDirectory! !

!Account methodsFor: 'accessing' stamp: 'SLB 8/22/2007 14:34'!
setGlobalId
	"<^ iOf String>"
	"When real handles are enabled, this will use the handle server to set the ID.
	 For now we just use the fact that the globalId getter sets it."
	"Account new setGlobalId inspect"

	^self globalId! !

!Account methodsFor: 'accessing' stamp: 'SLB 5/17/2007 08:36'!
xmlOutputFileStream
	"<^ hOf FileStream>"
	"getter"

	^xmlOutputFileStream! !

!Account methodsFor: 'accessing' stamp: 'SLB 5/17/2007 08:37'!
xmlOutputFileStream: aFileStream
	"setter"

	xmlOutputFileStream := aFileStream! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Account class
	instanceVariableNames: ''!

!Account class methodsFor: 'instance creation' stamp: 'SLB 5/5/2008 16:51'!
forRootFolder: aFileDirectory
	"Answer a new Account for an mbox email hierarchy rooted at the given directory."

	| me subdirectory |
	me := Account new.
	me rootDirectory: aFileDirectory.
	aFileDirectory directoryNames asSortedCollection do: [ :aDirectoryName |	
		subdirectory := FileDirectory on: aFileDirectory pathName,
										 FileDirectory pathNameDelimiter asString,
										 aDirectoryName.
		me folders add:
			(Folder folderName: aDirectoryName
					folderDirectory: subdirectory
					forAccount: aFileDirectory
					parseStatusStream: me parseStatusStream)].
	me setGlobalId.
	^me! !

!Account class methodsFor: 'instance creation' stamp: 'SLB 9/4/2008 15:55'!
selectAndParseAccount
	"Pick the directory that is the root of an Account, Create an Account for it and,
	 recursively parse the 'messages.mbox' files in (sub)directories that represent
	 a folder hierarchy of the Account.  This is primarily meant to be used
	 manually from a Workspace or Transcript."
	"Account selectAndParseAccount"

	| me |
	me := Account forRootFolder: PluggableFileList getFolder.
	^me orchestrateParsing! !

!Account class methodsFor: 'instance creation' stamp: 'SLB 8/21/2008 11:33'!
selectAndParseAccountWithSpyOn
	"Pick the directory that is the root of an Account, Create an Account for it and,
	 recursively parse the 'messages.mbox' files in (sub)directories that represent
	 a folder hierarchy of the Account.  When complete write the profiling results
	 to a file."
	"Account selectAndParseAccount"

	| me |
	me := Account forRootFolder: PluggableFileList getFolder.
	^MessageTally spyOn: [me orchestrateParsing] toFileNamed: 'SpyResults.txt'! !


!Account class methodsFor: 'file-directory management' stamp: 'SLB 5/6/2008 20:29'!
cleanupOldParseResultsIn: aFileDirectory
	"Remove summary files, attachment files, badMessage files, and the MessageSummary file.
	 Then recursively do the same for all subdirectories.  This is intended to be used before
	 reparsing the account so that the directory is not cluttered with multiple copies of these
	 files."

	aFileDirectory fileNames do: [ :aFileName |
		((aFileName beginsWith: 'attach')
			or: [aFileName beginsWith: 'BadMess']
			or: [aFileName = 'MessageSummary.csv'])
				ifTrue: [aFileDirectory deleteFileNamed: aFileName]].
	aFileDirectory directoryNames do: [ :aDirectoryName |	
		self cleanupOldParseResultsIn:
			(FileDirectory
				on: (aFileDirectory pathName,
						aFileDirectory pathNameDelimiter asString,
						aDirectoryName))]! !


Object subclass: #BodyPartType
	instanceVariableNames: 'localID contentID contentType contentTransferEncoding contentDescription contentDisposition nativeFileRelPath childMessage recursiveMultiPartBody otherHeaders phantomBody content isEmptyBodyPart'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EmailParser-Model'!
!BodyPartType commentStamp: 'SLB 12/19/2008 17:33' prior: 0!
The BodypartType models David Minor's first SEmail schema BodyPartType definition.  It does not correspond well to any one type in the later Email-Account schema.  In making the transition, I bastardized this to output XML in the Email-Account schema.  If time permits, later (doubtfull) I will refactor it into multiple classes to manage recursive multiPart bodies, childMessages, etc.  SLB 10/20/2007 15:53

Local-ID (Long), 
Content-ID (String, 0-1), 
Content-Type (ContentTypeHeader, min 1), 
Content-Transfer-Encoding (ContentTransferEncodingHeader, exactly 1), 
Content-Disposition (ContentDispositionHeader, 0-1),
{choice Native-File-Rel-Path (anyURI, min 1), or ref=ChildMessage}, OtherHeaders (HeadersType, 0-1), 
PhantomBody (String, 0-1), Used for Content-Type = message/external-body.  The parser does not yed implement this.  See  RFC1341, section 7.3.3 -- The Message/External-Body subtype

Unlike a SimpleBody, a BodyPart may start with a blank line and may contain no headers.
Only headers that start with 'Content-' have any meaning to a BodyPart.

Content-Types
1) Five of the seven initial media type values refer to discrete bodies. The content of these types must be handled by non-MIME mechanisms; they are opaque to MIME processors.
2) The five discrete types are: text, image, audio, video, and application.
3) The two non-dicrete types are multipart and message.
4) Unrecognized subtypes of "text" should be treated as subtype "plain" as long as the MIME implementation knows how to handle the charset.
5) Unrecognized subtypes of text which also specify an unrecognized charset should be treated as "application/octet-stream".
!


!BodyPartType methodsFor: 'printing' stamp: 'SLB 5/23/2007 20:36'!
printOn: aStream
	"print myself"

	aStream nextPutAll: 'BODY PART, localID: '.
	localID printOn: aStream.
	self contentID notNil
		ifTrue: [aStream nextPutAll: ', ContentID: '.
				self contentID printOn: aStream].
	aStream nextPutAll: ', Content-Type: '.
	self contentType printOn: aStream.
	aStream nextPutAll: ', contentTransferEncoding: '.
	contentTransferEncoding printOn: aStream.
	self contentDisposition notNil
		ifTrue: [aStream nextPutAll: ', ContentDisposition: '.
				self contentDisposition printOn: aStream].
	aStream nextPutAll: ', nativeFileRelPath: '.
	nativeFileRelPath printOn: aStream.
	aStream nextPutAll: ', childMessage: '.
	childMessage printOn: aStream.
	aStream nextPutAll: ', otherHeaders: '.
	otherHeaders printOn: aStream.
	aStream nextPutAll: ', phantomBody: '.
	phantomBody printOn: aStream.
	aStream cr; nextPutAll: 'content: '.
	content printOn: aStream.    ! !

!BodyPartType methodsFor: 'printing' stamp: 'SLB 12/13/2008 14:33'!
printXMLForChildMessageOn: aStream enclosingIndentLevel: tabs
	"Print my contents in the mail-account.xsd ChildMessage format at the indicated indent level."

	aStream crtab: tabs; basicNextPutAll: '<SingleBody>'.
	self contentType isNil
		ifFalse: [self contentType printXMLOn: aStream enclosingIndentLevel: tabs + 1].
	self contentTransferEncoding isNil
		ifTrue: [ContentTransferEncodingHeader default printXMLOn: aStream enclosingIndentLevel: tabs + 1]
		ifFalse: [self contentTransferEncoding printXMLOn: aStream enclosingIndentLevel: tabs + 1].
	self contentID notNil
		ifTrue: [aStream crtab: tabs + 1; basicNextPutAll: '<ContentId>'.
				self contentID printHeaderValueOn: aStream.
				aStream basicNextPutAll: '</ContentId>'].
	self contentDisposition notNil
		ifTrue: [aStream crtab: tabs + 1; basicNextPutAll: '<Disposition>'.
				self contentDisposition printHeaderValueOn: aStream.
				aStream basicNextPutAll: '</Disposition>'].
	self otherHeaders isEmpty
		ifFalse: [otherHeaders do: [ :aHeader |
			aHeader printXMLOn: aStream enclosingIndentLevel: tabs + 1]].
	aStream crtab: tabs + 1; basicNextPutAll: '<ChildMessage>'.
	self childMessage printXMLOn: aStream enclosingIndentLevel: tabs + 1.
	aStream  crtab: tabs + 1; basicNextPutAll: '</ChildMessage>'.
	aStream crtab: tabs; basicNextPutAll: '</SingleBody>'! !

!BodyPartType methodsFor: 'printing' stamp: 'SLB 12/13/2008 14:33'!
printXMLForRecursiveMultipartOn: aStream enclosingIndentLevel: tabs
	"Print my contents in the mail-account.xsd format for a MultiPart at the indicated indent level."
	"Note: although the mail-account schema assumes that this body is a standard multipart body,
	 the parser has not been refactored with that in mind (yet).  So this form of 'multi-body' does
	 not handle preamble/epilog, which don't appear to be present in recursive multibodies
	 anyway. It also doesn't (yet) deal with ContentDescription headers...just cause we didn't know
	 about them.SLB 11/14/2007 16:24."

	aStream crtab: tabs; basicNextPutAll: '<MultiBody>'.
	self contentType printXMLOn: aStream enclosingIndentLevel: tabs + 1.
	self contentTransferEncoding notNil
		ifTrue: [self contentTransferEncoding printXMLOn: aStream enclosingIndentLevel: tabs + 1].
	self contentID notNil
		ifTrue: [aStream crtab: tabs + 1; basicNextPutAll: '<ContentId>'.
				self contentID printHeaderValueOn: aStream.
				aStream basicNextPutAll: '</ContentId>'].
	self contentDisposition notNil
		ifTrue: [aStream crtab: tabs + 1; basicNextPutAll: '<Disposition>'.
				self contentDisposition printHeaderValueOn: aStream.
				aStream basicNextPutAll: '</Disposition>'].
	self otherHeaders isEmpty
		ifFalse: [otherHeaders do: [ :aHeader |
			aHeader printXMLOn: aStream enclosingIndentLevel: tabs + 1]].
	self recursiveMultiPartBody printRecursiveMultiBodyXMLOn: aStream enclosingIndentLevel: tabs + 1.
	aStream crtab: tabs; basicNextPutAll: '</MultiBody>'.! !

!BodyPartType methodsFor: 'printing' stamp: 'SLB 12/13/2008 14:33'!
printXMLForSingleBodyOn: aStream enclosingIndentLevel: tabs
	"Print my contents in the mail-account.xsd SingleBody format at the indicated indent level."

	aStream crtab: tabs; basicNextPutAll: '<SingleBody>'.
	self contentType isNil
		ifTrue: [ContentTypeHeader default printXMLOn: aStream enclosingIndentLevel: tabs + 1]
		ifFalse: [self contentType printXMLOn: aStream enclosingIndentLevel: tabs + 1].
	self contentTransferEncoding isNil
		ifTrue: [ContentTransferEncodingHeader default printXMLOn: aStream enclosingIndentLevel: tabs + 1]
		ifFalse: [self contentTransferEncoding printXMLOn: aStream enclosingIndentLevel: tabs + 1].
	self contentID notNil
		ifTrue: [self contentID printXMLOn: aStream enclosingIndentLevel: tabs + 1].
	self contentDescription notNil
		ifTrue: [self contentDescription printXMLOn: aStream enclosingIndentLevel: tabs + 1].
	self contentDisposition notNil
		ifTrue: [self contentDisposition printXMLOn: aStream enclosingIndentLevel: tabs + 1].
	self otherHeaders isEmpty
		ifFalse: [otherHeaders do: [ :aHeader |
			(aHeader isMIME and: [aHeader isMalformed not])
				ifTrue: [aHeader printXMLOn: aStream enclosingIndentLevel: tabs + 1]]].
	self nativeFileRelPath notNil
		ifTrue:		"It is an external-body-content-type"
			[aStream crtab: tabs + 1; basicNextPutAll: '<ExtBodyContent>'.
			 aStream crtab: tabs + 2; basicNextPutAll: '<RelPath>', self nativeFileRelPath, '</RelPath>'.
			 aStream crtab: tabs + 2; basicNextPutAll: '<LocalId>', self localID printString, '</LocalId>'.
			 aStream crtab: tabs + 2; basicNextPutAll: '<XMLWrapped>true</XMLWrapped>'.
			 aStream crtab: tabs + 1; basicNextPutAll: '</ExtBodyContent>']
		ifFalse:		"It is an int-body-content-type"
			[aStream crtab: tabs + 1; basicNextPutAll: '<BodyContent>';
					  crtab: tabs + 2; basicNextPutAll: '<Content>'.
			 self content notNil
				ifTrue:
					[((self content indexOf: $<) > 0 or: [(self content indexOf: $&) > 0])
						ifTrue: [aStream basicNextPutAll: '<!![CDATA['; basicNextPutAll: self safeCDATAContent; basicNextPutAll: ']]>']
						ifFalse: [aStream basicNextPutAll: self content]].
			aStream  crtab: tabs + 2; basicNextPutAll: '</Content>';	"end int-body-content-type tags"
						  crtab: tabs + 1; basicNextPutAll: '</BodyContent>'].
	aStream crtab: tabs; basicNextPutAll: '</SingleBody>'! !

!BodyPartType methodsFor: 'printing' stamp: 'SLB 12/13/2008 13:42'!
printXMLOn: aStream enclosingIndentLevel: tabs
	"Print my contents in the mail-account.xsd format at the indicated indent level.
	Note: this method is way too big and kludgie.  Gotta refactor it when I get time."

	self isEmptyBodyPart
		ifTrue: [^self].		"Its an empty body part, don't print anything"
	(self contentType isNil or: [self contentType isSimple])
		ifTrue: [self printXMLForSingleBodyOn: aStream enclosingIndentLevel: tabs]
		ifFalse: [self recursiveMultiPartBody notNil
			ifTrue: [self printXMLForRecursiveMultipartOn: aStream enclosingIndentLevel: tabs]
			ifFalse: [self childMessage notNil
				ifTrue: [self printXMLForChildMessageOn: aStream enclosingIndentLevel: tabs]
				ifFalse: [^self	"Its an empty body part, don't print anything"]]].
	self phantomBody notNil
		ifTrue: [aStream crtab: tabs + 1; basicNextPutAll: '<PhantomBody>'.
				aStream basicNextPutAll: self phantomBody.
				aStream basicNextPutAll: '</PhantomBody>'].! !

!BodyPartType methodsFor: 'printing' stamp: 'SLB 11/15/2007 15:45'!
resetHeaders
	"Primarily for debugging.  Reset the 'hasBeenPrinted value of all headers
	 so next attempt to print them will work."

	self otherHeaders do: [ :aHeader |
		aHeader hasBeenPrinted: false].! !


!BodyPartType methodsFor: 'queries' stamp: 'SLB 8/20/2008 12:15'!
isSingleBodyPart
	"True if Content-Type is nil (implicit text/plain), or text/xxx, or application, or any
	 kind of in-line attachment, e.g., image, video, audio, etc."

	| type |
	type := self contentType.
	^type isNil
		or: [type isText
		or: [type isApplication
		or: [type isImage
		or: [type isAudio]
		or: [type isVideo]]]]! !

!BodyPartType methodsFor: 'queries' stamp: 'SLB 7/27/2008 16:46'!
isSingleTextBodyPart
	"True if Content-Type is nil (implicit text/plain), or text/xxx."

	| type |
	type := self contentType.
	^type isNil or: [type isText]! !


!BodyPartType methodsFor: 'parsing' stamp: 'SLB 12/13/2008 14:10'!
appendRestOfLargeTextBodyFrom: aStream upTo: multipartBoundary
	"<aStream: hOf ReadWriteStream, multipartBoundary: iOf String, ^iOf String>"
	"Collect and append the the remainder of this body part up to the
	 multipartBoundary string to my content string.
	 Return the last line of the bodyPart."

	| nextLine contentStream |
	contentStream := WriteStream with: self content.
	[aStream atEnd or: [(nextLine := aStream nextLine) includesSubString: multipartBoundary]]
		whileFalse:
			[contentStream basicNextPutAll: String cr, nextLine].
	self content: contentStream contents.
	^nextLine! !

!BodyPartType methodsFor: 'parsing' stamp: 'SLB 8/21/2008 11:34'!
createOutputFileStreamFor: inputStream
	"<inputStream: hOf ReadWriteStream, ^hOf StandardFileStream>"
	"Create and return a FileStream in the same directory as inputStream (which must
	 be a FileStream itself) with a name unique to this BodyPart."

	| directoryName fileName |
	directoryName := inputStream directory pathName.
	fileName := 'attach', self localID asString, '.xml'.
	self nativeFileRelPath: './', fileName.
	^StandardFileStream forceNewFileNamed: directoryName,
		FileDirectory pathNameDelimiter asString, fileName! !

!BodyPartType methodsFor: 'parsing' stamp: 'SLB 8/20/2008 21:22'!
getBodyFrom: aStream multipartBoundary: multipartBoundary
	"<aStream: hOf ReadWriteStream, multipartBoundary: iOf String, ^iOf String>"
	"Collect the the remainder of this body part up to the
	 multipartBoundary string.  Determine what type it is, e.g., SimpleBody,
	 in which case it is stored in the content instVar, a recursive MultiPartBody,
	 or a ChildMessage.  Return the last line of whatever is parsed."
	"Note: this is essentially a long ugly case statement.  In a perfect world, I,
	 should refactor classes and then write it cleanly.  Sigh."

	| nextLine newPart |
	self isSingleBodyPart
		ifTrue:   "It's a SinglePartBody"
			[nextLine := self getContentFrom: aStream upTo: multipartBoundary]
		ifFalse:
			[self contentType isMultipart 
				ifTrue:   "It's a recursively embedded MultiPartBody. Delve into it."
					[newPart := (MultiPartBody multipartBoundary: self contentType multiPartBoundary).
					 self recursiveMultiPartBody: newPart.
					 newPart parentMultipartBoundary: multipartBoundary.
					 nextLine :=  newPart parseFromStream: aStream]
				ifFalse:
					[self contentType isMessage
						ifTrue:
							[newPart := (ChildMessage new).
							 self childMessage: newPart.
							 newPart parentMultipartBoundary: multipartBoundary.
							 nextLine :=  newPart parseFromStream: aStream]
						ifFalse:
							[EmailUnrecognizedTypeException signal: 'Unhandled Content-Type: ', self contentType value]]].  "Well, what kind of body is it?"
	^nextLine! !

!BodyPartType methodsFor: 'parsing' stamp: 'SLB 8/2/2008 15:06'!
getContentFrom: aStream upTo: multipartBoundary
	"<aStream: hOf ReadWriteStream, multipartBoundary: iOf String, ^iOf String>"
	"Collect the the remainder of this body part up to the multipartBoundary string.
	 If content is not text and exceeds the Max length of internal attachments,
	 put it out in an external attachment file. Return the last line of whatever is parsed."

	| nextLine cr maxLength bodyStream |
	bodyStream := WriteStream on: (String new: 500).
	cr := Character cr.
	maxLength := RFC2822MIMEMessage MaxBodyPartLength.
	aStream atEnd
		ifTrue: [EmailParserException signal: 'Stream atEnd prior to parsing body part: ', self contentType class name].
	[((nextLine := aStream nextLine) includesSubString: multipartBoundary)
		or: [bodyStream size > maxLength]]
		whileFalse:
			[bodyStream nextPut: cr; nextPutAll: nextLine].
	self content: bodyStream contents.
	self content size > maxLength
		ifTrue:					  "if oversized, must deal with rest of body part"
			[self isSingleTextBodyPart
				ifTrue: [^self appendRestOfLargeTextBodyFrom: aStream upTo: multipartBoundary]
				ifFalse: [^self writeBodyPartContentsToFileFrom: aStream upTo: multipartBoundary]]
		ifFalse:
			[^nextLine]! !

!BodyPartType methodsFor: 'parsing' stamp: 'SLB 8/2/2008 10:47'!
getHeadersFrom: aStream multipartBoundary: multipartBoundary
	"<aStream: hOf ReadWriteStream, multipartBoundary: iOf String, ^iOf String>"
	"Parse all the headers.  Note that a legal body part need not have any headers and it
	 may have ONLY headers.  If there are both headers and a body, the end of the headers
	 is marked by an empty line.  Store header values of those with explicit instVars.
	 The others go in otherHeaders. Return the last line of whatever is parsed."

	| nextLine nextHeader headerName boundary |
	boundary := '--', multipartBoundary.
	[aStream atEnd
		or: [(nextLine := aStream nextLine) size = 0]  "empty line marks end of headers"
		or: [nextLine beginsWith: boundary]]   "marks end of bodyPart"
		whileFalse:
			[self isEmptyBodyPart: false.			"found at least one header"
			 nextHeader := (EmailHeader headerSubclassFor: nextLine)
							   parseFromStream: aStream withFirstLine: nextLine.	
			 (headerName := nextHeader name asLowercase) = 'content-type:'
				ifTrue: [self contentType: nextHeader]
				ifFalse: [headerName = 'content-id:'
					ifTrue: [self contentID: nextHeader]
					ifFalse: [headerName = 'content-transfer-encoding:'
						ifTrue: [self contentTransferEncoding: nextHeader]
						ifFalse: [headerName = 'content-description:'
							ifTrue: [self contentDescription: nextHeader]
							ifFalse: [headerName = 'content-disposition:'
								ifTrue: [self contentDisposition: nextHeader]
								ifFalse: [self otherHeaders add: nextHeader]]]]]].
	^nextLine! !

!BodyPartType methodsFor: 'parsing' stamp: 'SLB 11/24/2007 13:42'!
parseFromStream: aStream multipartBoundary: multipartBoundary
	"<aStream: hOf ReadWriteStream, multipartBoundary: iOf String, ^iOf String>"
	"from current stream position to end of this body part.  Return the last
	 line which, by definition, contains the multipart boundary that ends
	 the body part.  Returns the last line read while getting the body."

	| nextLine |
	self setLocalID.
	nextLine := self getHeadersFrom: aStream multipartBoundary: multipartBoundary.
	(nextLine beginsWith: '--', multipartBoundary)
		ifTrue: [^nextLine].
	self isEmptyBodyPart: false.  "we have found at least one line of body material"
	^self getBodyFrom: aStream multipartBoundary: multipartBoundary.! !

!BodyPartType methodsFor: 'parsing' stamp: 'SLB 5/10/2007 20:30'!
setLocalID
	"Set the localID with a random integer"

	self localID: RFC2822MIMEMessage randomID! !

!BodyPartType methodsFor: 'parsing' stamp: 'SLB 12/13/2008 14:11'!
writeBodyPartContentsToFileFrom: aStream upTo: multipartBoundary
	"<aStream: hOf ReadWriteStream, multipartBoundary: iOf String, ^iOf String>"
	"Collect the the remainder of this body part up to the
	 multipartBoundary string.  Return the last line of the bodyPart..
	 Write contents out to file, save the file rel path, and add the rest of the contents,
	 up to the multipartBoundary to the file."

	| nextLine fileStream fileName |
	fileStream := self createOutputFileStreamFor: aStream.
	self writeBodyPartHeaderTo: fileStream.
	fileName := fileStream fullName.
	fileStream basicNextPutAll: self content.
	fileStream close.
	self content: nil.
	fileStream := (StandardFileStream oldFileNamed: fileName) setToEnd.
	 "Resume collecting the contents into the fileStream"
	[(nextLine := aStream nextLine) includesSubString: multipartBoundary]
		whileFalse:
			[fileStream basicNextPutAll: String cr, nextLine].
	fileStream cr; basicNextPutAll: '</Content>'.
	fileStream cr; basicNextPutAll: '</ExternalBodyPart>'.
	fileStream close.
	^nextLine! !

!BodyPartType methodsFor: 'parsing' stamp: 'SLB 12/13/2008 14:11'!
writeBodyPartHeaderTo: aStream
	"<aStream: hOf ReadWriteStream, ^self>"
	"Write XML header info to identify this external body part."

	aStream basicNextPutAll: '<ExternalBodyPart>'.
	aStream cr; basicNextPutAll: '<LocalUniqueID>', self localID printString, '</LocalUniqueID>'.
	aStream cr; basicNextPutAll: '<ContentType>', self contentType value printString, '</ContentType>'.
	self contentID notNil
		ifTrue:
			[self contentID printXMLOn: aStream enclosingIndentLevel: 0].
	self contentDisposition notNil
		ifTrue:
			[self contentDisposition printXMLOn: aStream enclosingIndentLevel: 0].
	self contentTransferEncoding notNil
		ifTrue:
			[aStream cr; basicNextPutAll: '<ContentTransferEncoding>', self contentTransferEncoding value,
									'</ContentTransferEncoding>'].
	aStream cr; basicNextPutAll: '<Content>'.
	aStream cr.
! !


!BodyPartType methodsFor: 'accessing' stamp: 'SLB 5/7/2007 21:18'!
childMessage
	"<^ hOf ChildMessage>"
	"getter"

	^childMessage! !

!BodyPartType methodsFor: 'accessing' stamp: 'SLB 5/7/2007 21:20'!
childMessage: aChildMessage
	"setter"

	childMessage := aChildMessage! !

!BodyPartType methodsFor: 'accessing' stamp: 'SLB 5/7/2007 22:19'!
content
	"<^ hOf String>"
	^content! !

!BodyPartType methodsFor: 'accessing' stamp: 'SLB 5/7/2007 22:19'!
content: aString
	"<aString iOf String>"

	content := aString! !

!BodyPartType methodsFor: 'accessing' stamp: 'SLB 11/26/2007 13:19'!
contentDescription
	"<^ hOf MIMEHeader>"
	"Get the body part's Content-Description.  Note: all are case insensitive."

	^contentDescription! !

!BodyPartType methodsFor: 'accessing' stamp: 'SLB 11/26/2007 13:20'!
contentDescription: aHeader
	"set the body part's Content-Description."

	contentDescription := aHeader! !

!BodyPartType methodsFor: 'accessing' stamp: 'SLB 10/6/2007 17:05'!
contentDisposition
	"<^ hOf MIMEHeader>"
	"Get the body part's Content-Disposition.  Note: all are case insensitive."

	^contentDisposition! !

!BodyPartType methodsFor: 'accessing' stamp: 'SLB 10/6/2007 17:00'!
contentDisposition: aHeader
	"set the body part's Content-Disposition."

	contentDisposition := aHeader! !

!BodyPartType methodsFor: 'accessing' stamp: 'SLB 10/6/2007 17:04'!
contentID
	"<^ hOf MIMEHeader>"
	^contentID! !

!BodyPartType methodsFor: 'accessing' stamp: 'SLB 10/6/2007 17:01'!
contentID: aHeader
	"<aString hOf MIMEHeader>"

	contentID := aHeader! !

!BodyPartType methodsFor: 'accessing' stamp: 'SLB 10/6/2007 17:07'!
contentTransferEncoding
	"<^ hOf MIMEHeader>"
	"Get the body part's Content-Type, one of: '7bit' '8bit' 'binary' 'quoted-printable' 'base64' ietf-token' or 'x-token'.  Note: all are these parameters case insensitive."

	^contentTransferEncoding! !

!BodyPartType methodsFor: 'accessing' stamp: 'SLB 10/6/2007 16:59'!
contentTransferEncoding: aHeader
	"set the body part's Content-Type header, e.g., '7bit' or 'base64'.  Case insensitive."

	contentTransferEncoding := aHeader! !

!BodyPartType methodsFor: 'accessing' stamp: 'SLB 10/6/2007 17:07'!
contentType
	"<^ hOf MIMEHeader>"
	"getter"

	^contentType! !

!BodyPartType methodsFor: 'accessing' stamp: 'SLB 10/6/2007 17:07'!
contentType: aHeader
	"<^ hOf MIMEHeader>"
	"set the value of the part's ContentTypeHeader."

	contentType := aHeader! !

!BodyPartType methodsFor: 'accessing' stamp: 'SLB 11/24/2007 13:30'!
isEmptyBodyPart
	"<^ (true | false)>"

	isEmptyBodyPart isNil
		ifTrue: [isEmptyBodyPart := true].
	^isEmptyBodyPart! !

!BodyPartType methodsFor: 'accessing' stamp: 'SLB 11/24/2007 13:31'!
isEmptyBodyPart: aBoolean
	"set to false if any header or content found."

	isEmptyBodyPart := aBoolean! !

!BodyPartType methodsFor: 'accessing' stamp: 'SLB 5/7/2007 14:30'!
localID
	"<^ hOf Integer>"
	^localID! !

!BodyPartType methodsFor: 'accessing' stamp: 'SLB 5/7/2007 14:31'!
localID: anInteger
	"setter"

	localID _ anInteger! !

!BodyPartType methodsFor: 'accessing' stamp: 'SLB 5/7/2007 17:20'!
nativeFileRelPath
	"<^ iOf String>"
	"Getter -- returns a string that is a legal URL"

	^nativeFileRelPath! !

!BodyPartType methodsFor: 'accessing' stamp: 'SLB 11/21/2007 15:26'!
nativeFileRelPath: aString
	"Setter -- aString must be a legal URL"

	nativeFileRelPath := aString! !

!BodyPartType methodsFor: 'accessing' stamp: 'SLB 5/7/2007 21:22'!
otherHeaders
	"<^ hOf OrderedCollection {of: hOf MsgHeader}>"
	"Laissez faire getter"

	otherHeaders isNil
		ifTrue: [otherHeaders := OrderedCollection new].
	^otherHeaders! !

!BodyPartType methodsFor: 'accessing' stamp: 'SLB 5/7/2007 21:23'!
phantomBody
	"<^ iOf String>"
	"getter"

	^phantomBody! !

!BodyPartType methodsFor: 'accessing' stamp: 'SLB 5/7/2007 21:23'!
phantomBody: aString
	"setter"

	phantomBody := aString! !

!BodyPartType methodsFor: 'accessing' stamp: 'SLB 5/23/2007 08:11'!
recursiveMultiPartBody
	"<^ hOf MultiPartBody>"
	"Get the body part's recursiveMultiPartBody. "

	^recursiveMultiPartBody! !

!BodyPartType methodsFor: 'accessing' stamp: 'SLB 5/23/2007 08:12'!
recursiveMultiPartBody: aMultiPartBody
	"setter"

	recursiveMultiPartBody := aMultiPartBody! !

!BodyPartType methodsFor: 'accessing' stamp: 'SLB 12/5/2008 21:11'!
safeCDATAContent
	"<^ (iOf String | nil)>"
	"Answer a string that can be safely inserted into a CDATA body.
	 That is, a string that contains no occurrence of ']]'  If that string is,
	 present, replace it with escaped hex values of characters &#093;&#093; "

	^self content copyReplaceAll: ']]' with: '&#093;&#093;' ! !


Object subclass: #ChildMessage
	instanceVariableNames: 'localID messageID messageHeaders mimeHeaders body parentMultipartBoundary childStream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EmailParser-Model'!
!ChildMessage commentStamp: 'SLB 12/19/2008 17:34' prior: 0!
A ChildMessage is an e-mail message forwarded by or sent as an attachment to another e-mail message,  It generates a body-part of type message/rfc822.

The "message/rfc822" subtype has no terminating condition other than hitting its terminating boundary parameter string or running out of data (EOF)..

Child messages may themselves contain attachments.  We parse this message, with its attachments, and include it within the xml encoding of its parent.

Structure: localID messageID headers body
 localID:			initially a Long integer (in the C world), Should be a handle
 messageID:		String
 headers: 		collection of MsgHeaders
 body:			SimpleBody, SingleBody or MultiPartBody

   A media type of "message/rfc822" indicates that the body contains an
   encapsulated message, with the syntax of an RFC 822 message.
   However, unlike top-level RFC 822 messages, the restriction that each
   "message/rfc822" body must include a "From", "Date", and at least one
   destination header is removed and replaced with the requirement that
   at least one of "From", "Subject", or "Date" must be present.

   It should be noted that, despite the use of the numbers "822", a
   "message/rfc822" entity isn't restricted to material in strict
   conformance to RFC822, nor are the semantics of "message/rfc822"
   objects restricted to the semantics defined in RFC822. More
   specifically, a "message/rfc822" message could well be a News article
   or a MIME message, neither of which existed in the RFC822 spec.

   No encoding other than "7bit", "8bit", or "binary" is permitted for
   the body of a "message/rfc822" entity.  The message header fields are
   always US-ASCII in any case, and data within the body can still be
   encoded, in which case the Content-Transfer-Encoding header field in
   the encapsulated message will reflect this.  Non-US-ASCII text in the
   headers of an encapsulated message can be specified using the
   mechanisms described in RFC 2047.
!


!ChildMessage methodsFor: 'accessing' stamp: 'SLB 5/29/2007 20:58'!
body
	"<^ (hOf SimpleBody | hOf MultiPartBody)."

	^body! !

!ChildMessage methodsFor: 'accessing' stamp: 'SLB 5/29/2007 20:58'!
body: simpleOrMultipartBody
	"setter"

	body _ simpleOrMultipartBody! !

!ChildMessage methodsFor: 'accessing' stamp: 'SLB 7/4/2007 19:34'!
childStream
	"<^ hOf ReadWriteStream>"
	"Getter"

	^childStream! !

!ChildMessage methodsFor: 'accessing' stamp: 'SLB 7/4/2007 19:34'!
childStream: aReadWriteStream
	"<aReadWriteStream: hOf ReadWriteStream, ^self>"
	"Setter"

	childStream := aReadWriteStream! !

!ChildMessage methodsFor: 'accessing' stamp: 'SLB 12/3/2007 10:28'!
contentType
	"<^hOf ContentTypeType>"
	"Answer the ContentTypeHeader if any, else nil"

	^self mimeHeaders at: 'content-type:' ifAbsent: [nil]! !

!ChildMessage methodsFor: 'accessing' stamp: 'SLB 5/29/2007 20:52'!
localID
	"<^ hOf Integer>"
	^localID! !

!ChildMessage methodsFor: 'accessing' stamp: 'SLB 5/29/2007 20:53'!
localID: anInteger
	"setter"

	localID _ anInteger! !

!ChildMessage methodsFor: 'accessing' stamp: 'SLB 10/20/2007 16:25'!
messageHeaders
	"<^ hOf OrderedCollection of: {hOf MsgHeader}>"
	"Laissez faire getter"

	messageHeaders isNil
		ifTrue: [messageHeaders := OrderedCollection new].
	^messageHeaders! !

!ChildMessage methodsFor: 'accessing' stamp: 'SLB 5/29/2007 22:13'!
messageID
	"<^ iOf String>"
	"getter"

	^messageID! !

!ChildMessage methodsFor: 'accessing' stamp: 'SLB 5/29/2007 22:13'!
messageID: aString
	"setter"

	messageID _ aString! !

!ChildMessage methodsFor: 'accessing' stamp: 'SLB 11/16/2007 17:32'!
mimeHeaders
	"<^ hOf OrderedCollection of: {hOf MIMIHeader}>"
	"Laissez faire getter"

	mimeHeaders isNil
		ifTrue: [mimeHeaders := Dictionary new].
	^mimeHeaders! !

!ChildMessage methodsFor: 'accessing' stamp: 'SLB 5/29/2007 20:50'!
parentMultipartBoundary
	"<^ iOf String>"
	"Getter"

	^parentMultipartBoundary! !

!ChildMessage methodsFor: 'accessing' stamp: 'SLB 5/29/2007 20:52'!
parentMultipartBoundary: aString
	"Setter"

	parentMultipartBoundary := aString! !


!ChildMessage methodsFor: 'parsing' stamp: 'SLB 7/4/2007 19:37'!
createChildFileStreamFor: inputStream
	"<inputStream: hOf ReadWriteStream, ^hOf StandardFileStream>"
	"Create and return a FileStream in the same directory as inputStream (which must
	 be a FileStream itself) with a name unique to this ChildMessage."

	| directory fileName |
	directory := inputStream directory pathName.
	fileName := 'childTemp', self localID asString, '.txt'.
	self childStream: (StandardFileStream forceNewFileNamed: directory,
								FileDirectory pathNameDelimiter asString, fileName)! !

!ChildMessage methodsFor: 'parsing' stamp: 'SLB 12/8/2008 10:10'!
getBodyFrom: aStream
	"Parse the body which is the remainder of the email."

	| contentType |
	self setDefaultHeadersIfNeeded.
	contentType := self mimeHeaders at: 'content-type:'.
	contentType isText
		ifTrue:
			[self body: (SingleBody mimeHeaders: self mimeHeaders).
			 self body parseFromStream: aStream]
		ifFalse: [contentType isMultipart
			ifTrue:
				[self body: (MultiPartBody multipartBoundary: self contentType multiPartBoundary).
				 self body mimeHeaders: self mimeHeaders.
				 self body parseFromStream: aStream]
			ifFalse:
				[EmailUnrecognizedTypeException signal: 'ChildMessage Unhandled Content-Type: ',
					contentType value]]  "Well, what kind of body is it?"! !

!ChildMessage methodsFor: 'parsing' stamp: 'SLB 8/21/2008 21:45'!
getChildStreamFrom: aStream
	"<aStream: hOf ReadWriteStream, ^iOf String>"
	"Pull out the child message text into a new childStream to be parsed.
	 if aStream is an external (file) stream, create a new FIleStream for the child.
	 Note: if the child message is terminated by some 'outer' multipart boundary rather
	 than the expected boundary (which is a questionable practice that some email clients
	 use), the child message will grab all the rest of the enclosing message and the
	 parser will fail. There are ways to handle such a scenario, but for now it simply
	 isn't handled.
	 Returns the nextLine to be parsed. SLB 7/4/2007 18:07."

	| nextLine firstPass boundaryLine |
	boundaryLine := '--', self parentMultipartBoundary.
	[(nextLine := aStream nextLine) isEmpty]
		whileTrue: ["skip leading empty lines"].
	firstPass := true.
	aStream isExternalStream
		ifTrue: [self createChildFileStreamFor: aStream]
		ifFalse: [self childStream: (ReadWriteStream on: String new)].
	[nextLine beginsWith: boundaryLine]
		whileFalse:
				[firstPass
					ifTrue: [firstPass := false]
					ifFalse: [childStream cr].
				 self childStream nextPutAll: nextLine.
				 nextLine := aStream nextLine].
	self childStream reset.
	^nextLine! !

!ChildMessage methodsFor: 'parsing' stamp: 'SLB 11/23/2007 16:28'!
getHeadersFrom: messageStream
	"Parse all the headers.  The end of the headers is marked by an empty line.  Note that a legal message must have at least one header so we can grab the first one without checking for empty line since the mbox imput has been exported from an email system and hence is legal enough to have survived prior parsing by email servers and clients."
"NOTE this is copied wholesale from RFC2822MIMEMessage...may need some fixing."

	| nextLine nextHeader |
	[messageStream atEnd or: [(nextLine := messageStream nextLine) size = 0]]
		whileFalse:
			[nextHeader := (EmailHeader headerSubclassFor: nextLine)
								 parseFromStream: messageStream withFirstLine: nextLine.
			 (nextHeader name asLowercase = 'message-id:')
				ifTrue: [self messageID: nextHeader value]
				ifFalse: [nextHeader isMIME
					ifTrue: [self mimeHeaders at: nextHeader name put: nextHeader]
					ifFalse: [self messageHeaders add: nextHeader]]]! !

!ChildMessage methodsFor: 'parsing' stamp: 'SLB 8/21/2008 21:46'!
parseFromStream: aReadWriteStream
	"<aReadWriteStream: hOf ReadWriteStream, ^iOf string>"
	"The stream must contain only the text for one message.  If the original email was
	 in an mbox format, the sender of this message is expected to have undone any '>From '
	 quoting (see www.qmail.org/man/man5/mbox.html).  Returns the last line encountered
	 in parsing, presumably the multipart boundary string that terminates the message.
	 NOTE: Web commentary asserts that child messages are more likely to not be well formed.
	 If the assertion is true, reliably finding its end during normal parsing is at risk because
	 any line might contain the boundary string, e.g., within a header line.  At present
	 few of the model objects bother to look for that string."

	| nextLine |
	self setLocalID.
	nextLine := self getChildStreamFrom: aReadWriteStream.
	self getHeadersFrom: childStream.
	self getBodyFrom: childStream.
	self setMessageID.
	childStream isExternalStream
		ifTrue: [childStream close.
				childStream directory deleteFileNamed: childStream name].
	^nextLine! !

!ChildMessage methodsFor: 'parsing' stamp: 'SLB 12/8/2008 10:18'!
setDefaultHeadersIfNeeded
	"This is a Single/Simple body email.  Set Content-type/charset and Content-Transfer-Encoding
	 headers if absent."

	self mimeHeaders at: 'content-type:' 
		ifAbsent:	   "set to text/plain, us-ascii"
			[self mimeHeaders at: 'content-type:'
							 put: ContentTypeHeader default].
	self mimeHeaders at: 'content-transfer-encoding:'
		ifAbsent:
			[self mimeHeaders at: 'content-transfer-encoding:'
							 put: ContentTransferEncodingHeader default]! !

!ChildMessage methodsFor: 'parsing' stamp: 'SLB 5/29/2007 21:13'!
setLocalID
	"Fill in local-ID with random number.  This is for prototype only.
	 Something like a handle server should provide these IDs."

	self localID: RFC2822MIMEMessage randomID! !

!ChildMessage methodsFor: 'parsing' stamp: 'SLB 10/22/2007 19:31'!
setMessageID
	"Create a supplied message-Id if necessary.
	 Note, a child message may not have a Message ID, either because it isn't well
	 formed, or because the exchange was within a small email group where none
	 was set."

	self messageID isNil
		ifTrue: [self messageID: MsgHeader defaultMissingMessageIDHeader value]! !


!ChildMessage methodsFor: 'printing' stamp: 'SLB 11/9/2007 20:50'!
printXMLForHeadersOn: aStream enclosingIndentLevel: tabs
	"Print my headers in the mail-account.xsd format at the indicated indent level.  Some use
	 individualized tags.  The rest just use the general <Header> tag.
	Note: For now (SLB 11/9/2007 20:47) only one header, date:, is printed both in a special and in a
	 generic form.  David Minor and I have agreed that special headers where the value is the same
	 do not need to be printed twice.  The Date: header is not the same.  It's a W3C DateTime in the
	 special version and a RFC822 DateTime in the generic version."

	| tags fields |
	tags := #('MimeVersion>' 'OrigDate>' 'From>' 'Sender>' 'To>' 'Cc>' 'Bcc>' 'InReplyTo>'
			'References>' 'Subject>' 'Comments>' 'Keywords>').
	fields := #('mime-version:' 'date:' 'from:' 'sender:' 'to:' 'cc:' 'bcc:' 'reply-to:' 
			'references:' 'subject:' 'comments:' 'keywords:').
	1 to: tags size do: [ :i |
		self messageHeaders do: [ :aHeader |
			(aHeader hasBeenPrinted not and: [aHeader name asLowercase = (fields at: i)])
				ifTrue:
					[aHeader hasBeenPrinted: true. "So special headers (except date:) won't be printed twice."
					 self printXMLForSpecialHeader: aHeader on: aStream withTag: (tags at: i)
							  enclosingIndentLevel: tabs]]].
	self messageHeaders do: [ :aHeader |		  "print remaining unprinted message headers"
		aHeader hasBeenPrinted
			ifFalse:
				[aHeader printXMLOn: aStream enclosingIndentLevel: tabs.
				 aHeader hasBeenPrinted: true]].! !

!ChildMessage methodsFor: 'printing' stamp: 'SLB 12/15/2008 19:07'!
printXMLForSpecialHeader: aHeader on: aStream withTag: tagString enclosingIndentLevel: tabs
	"Print the 'special' header in the mail-account.xsd format at the indicated indent level."

	| w3cDate |
	aStream crtab: tabs; nextPut: $<; basicNextPutAll: tagString.
	tagString = 'OrigDate>'
		ifTrue:  "Convert it to a W3C DateTime string before printing"
			[w3cDate := W3CDateTime new.
			[w3cDate parseFromRFC822String: aHeader value]
				on: EmailMalformedDateException
				do: [ :ex | self recordParseErrorLocation: 'printing W3CDate'
								type: ex description.
					self w3cDate: (self w3cDate guessFromRFC2822String: aHeader value)].
			w3cDate printW3CDateTimeFormatOn: aStream.
			aHeader hasBeenPrinted: false "force the generic Date header to also be printed" ]
		ifFalse:
			[aHeader printHeaderValueOn: aStream].
	aStream basicNextPutAll: '</'; nextPutAll: tagString! !

!ChildMessage methodsFor: 'printing' stamp: 'SLB 12/13/2008 13:48'!
printXMLOn: aStream enclosingIndentLevel: tabs
	"Print my contents in the mail-account.xsd format at the indicated indent level."

	aStream crtab: tabs + 1; basicNextPutAll: '<LocalId>'.
		self localID printOn: aStream.
		aStream basicNextPutAll: '</LocalId>'.
	aStream crtab: tabs + 1; basicNextPutAll: '<MessageId>'.
	(self messageID includes: $<)
		ifTrue: [aStream basicNextPutAll: '<!![CDATA['; basicNextPutAll: self messageID; basicNextPutAll: ']]>']
		ifFalse: [aStream basicNextPutAll: self messageID].
		aStream basicNextPutAll: '</MessageId>'.
	self printXMLForHeadersOn: aStream enclosingIndentLevel: tabs + 1.
	self body printXMLOn: aStream enclosingIndentLevel: tabs + 1.! !

!ChildMessage methodsFor: 'printing' stamp: 'SLB 11/16/2007 17:41'!
resetHeaders
	"Primarily for debugging.  Reset the 'hasBeenPrinted value of all headers
	 so next attempt to print them will work."

	self messageHeaders do: [ :aHeader |
		aHeader hasBeenPrinted: false].
	self mimeHeaders valuesDo: [ :aHeader |
		aHeader hasBeenPrinted: false]! !


!ChildMessage methodsFor: 'queries' stamp: 'SLB 11/16/2007 17:26'!
contentTransferEncoding
	"return the Content-Transfer-Encoding header if present, Else nil"

	^self mimeHeaders at: 'content-transfer-encoding' ifAbsent: [nil]! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

ChildMessage class
	instanceVariableNames: ''!

!ChildMessage class methodsFor: 'instance creation' stamp: 'SLB 5/31/2007 16:20'!
multipartBoundary: aString
	"Answer a new instance with the given multipartBoundary string"

	| me |
	me := self new.
	me multipartBoundary: aString.
	^me! !


Object subclass: #EmailHeader
	instanceVariableNames: 'origString name value comments hasBeenPrinted'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EmailParser-Model'!
!EmailHeader commentStamp: 'SLB 11/21/2007 09:29' prior: 0!
This is an abstract superclas for both MsgHeader and MIMEHeader.  MsgHeaders are for the variety of headers in main messages, e.g., To: From: and all sorts of X-headers.  MIMEHeaders are more rigidly structured headers, whose names begin with 'Content-' that define MIME multi-part bodies and contents.


Structure shared by EmailHeader and all of its subclasses:
 origString 		iOf String -- the entire original string for the header
 name 			iOf String -- the header name
 value			iOf String -- the header value
 comments		hOf OrderedCollection {of: iOf String} -- whatever parens-enclosed 
				comments are in the header.  Note that at least one X-header (X-UIDL) may
				have open parens in the ID string, hence X-headers comments are not extracted.
 hasBeenPrinted Boolean -- a way to keep track of which headers have been output as XML so
				that when the remaining headers are printed we avoid duplicates if desired.!


!EmailHeader methodsFor: 'accessing' stamp: 'SLB 9/28/2007 15:23'!
comments
	"<^hOf OrderedCollection {:of iOf: String}>"
	"Laissez faire getter for comments collection."

	comments isNil
		ifTrue: [comments := OrderedCollection new].
	^comments! !

!EmailHeader methodsFor: 'accessing' stamp: 'SLB 10/18/2007 22:35'!
hasBeenPrinted
	"<^ (true | false)>"
	"Laissez faire getter"
	
	hasBeenPrinted isNil ifTrue: [hasBeenPrinted := false].
	^hasBeenPrinted! !

!EmailHeader methodsFor: 'accessing' stamp: 'SLB 10/18/2007 22:35'!
hasBeenPrinted: aBoolean
	"setter.  Since this variable defaults to false, aBoolean will usually be 'true' "

	hasBeenPrinted := aBoolean! !

!EmailHeader methodsFor: 'accessing' stamp: 'SLB 9/24/2007 22:19'!
name
	"<^ iOf String>"
	"getter"
	
	^name! !

!EmailHeader methodsFor: 'accessing' stamp: 'SLB 9/24/2007 22:20'!
name: aString
	"setter"

	name := aString! !

!EmailHeader methodsFor: 'accessing' stamp: 'SLB 9/24/2007 22:18'!
origString
	"Getter.  Returns the original mail-file string for this header, folded if the original was folded"

	^origString! !

!EmailHeader methodsFor: 'accessing' stamp: 'SLB 9/24/2007 22:18'!
origString: aString
	"Setter for instVar that holds the entire (perhaps folded) string for this header, not including the ending crlf"

	origString := aString! !

!EmailHeader methodsFor: 'accessing' stamp: 'SLB 9/24/2007 22:20'!
value
	"<^ iOf String>"
	"getter"
	
	^value! !

!EmailHeader methodsFor: 'accessing' stamp: 'SLB 9/24/2007 22:20'!
value: aString
	"setter"

	value := aString! !


!EmailHeader methodsFor: 'queries' stamp: 'SLB 9/25/2007 14:35'!
isFolded
	"Answer true if the header extends beyond one line."

	| testStream |
	testStream := (ReadStream on: self origString).
	testStream nextLine.
	^testStream atEnd not and:
		[testStream peek isWhitespace]! !

!EmailHeader methodsFor: 'queries' stamp: 'SLB 10/6/2007 11:59'!
isMIME
	"Default answer false."

	^false! !

!EmailHeader methodsFor: 'queries' stamp: 'SLB 11/24/2007 16:05'!
isMalformed
	"No I am not"

	^false! !


!EmailHeader methodsFor: 'parsing' stamp: 'SLB 2/20/2008 10:15'!
collectCommentFrom: aStream
	"aStream is positioned just after the open parens of one comment.  Collect the comment
	 text and add it to my comments collection. Record warning if parens are unbalanced"

	| parensCount commentStream |
	parensCount := 1.
	commentStream := WriteStream with: '('.
	[parensCount = 0 or: [aStream atEnd]]
		whileFalse:
			[aStream peek = $(
				ifTrue: [parensCount := parensCount + 1].
			 aStream peek = $)
				ifTrue: [parensCount := parensCount - 1].
			 commentStream nextPut: aStream next].
	self comments add: commentStream contents.
	(aStream atEnd and: [parensCount > 0])
		ifTrue:
			[EmailMalformedHeaderException signal:
				'Warning - unbalanced comment parentheses in ', self class name, ', contents: ', commentStream contents]! !

!EmailHeader methodsFor: 'parsing' stamp: 'SLB 2/9/2008 15:46'!
extractCommentsFrom: aStream
	"Pull out comment if any and return a comment-free stream.  Note, there can be multiple
	 comments and parentheses may be nested.  The most common case is that there are no
	 comments.  Handle that case efficiently.  Do not get fooled by an open parens within a
	 quoted parameter value.  It is not the start of a comment."

	| commentFreeStream inQuotedParam nextChar |
	aStream contents indexOfSubCollection: '(' startingAt: 1
		ifAbsent: [^aStream].	"header has no comments"
	aStream reset.
	commentFreeStream := ReadWriteStream on: String new.
	inQuotedParam := false.
	[aStream atEnd]
		whileFalse:
			[(nextChar := aStream next) = $"
				ifTrue: [inQuotedParam := inQuotedParam not].
			 (inQuotedParam not and: [nextChar = $( ])
				ifTrue: [self collectCommentFrom: aStream]
				ifFalse: [commentFreeStream nextPut: nextChar]].
	^commentFreeStream reset! !

!EmailHeader methodsFor: 'parsing' stamp: 'SLB 9/25/2007 14:40'!
getOrigStringFrom: aReadWriteStream
	"get the original string which may be folded onto multiple lines.
	 if folded, the first string of subsequent lines will be whitespace."

	self origString: aReadWriteStream nextLine.
	[aReadWriteStream atEnd not and:
		[aReadWriteStream peek isWhitespace]]
			whileTrue:
				[self origString: self origString, String cr, aReadWriteStream nextLine].! !

!EmailHeader methodsFor: 'parsing' stamp: 'SLB 9/25/2007 14:41'!
getOrigStringFrom: messageStream withFirstLine: firstLine
	"get the original string which may be folded onto multiple lines.
	 if folded, the first string of subsequent lines will be whitespace."

	self origString: firstLine.
	[messageStream atEnd not and:
		[messageStream peek isWhitespace]]
			whileTrue:
				[self origString: self origString, String cr, messageStream nextLine].! !

!EmailHeader methodsFor: 'parsing' stamp: 'SLB 9/28/2007 15:24'!
saveComment: aString
	"Add the string to my comments collection"

	self comments add: aString! !

!EmailHeader methodsFor: 'parsing' stamp: 'SLB 9/29/2007 15:21'!
unfoldedStream
	"<^hOf ReadWriteStream>"
	"Answer a Stream on the original string, unfolded if necessary.
	 Note: uses primitive indesOfAscii:inString:startingAt: for speed"

	| unfoldedStream start foundCr  |
	unfoldedStream := ReadWriteStream on: String new.
	start := 1.
	foundCr := 1.
	[foundCr > 0] whileTrue:	
		[foundCr := String indexOfAscii: 13 inString: self origString startingAt: start.
		 foundCr > 0
			ifTrue:
				[unfoldedStream nextPutAll: (self origString copyFrom: start to: foundCr - 1).
				 start := foundCr + 1]].
	unfoldedStream nextPutAll: (self origString copyFrom: start to: origString size).
	^unfoldedStream reset! !


!EmailHeader methodsFor: 'printing' stamp: 'SLB 12/15/2008 17:21'!
printHeaderNameOn: aStream
	"Print my header name escaping < and/or & chars with CDATA encoding if necessary.
	 Note, header names may legally have ANY printable ASCII character except : 
	 It is usually an error, though, for exotic chars in header names."

	((self name includes: $<) or: [(self name includes: $&)])
		ifTrue: [aStream basicNextPutAll: '<!![CDATA['; basicNextPutAll: self name; basicNextPutAll: ']]>']
		ifFalse: [aStream basicNextPutAll: self name].! !

!EmailHeader methodsFor: 'printing' stamp: 'SLB 12/15/2008 17:22'!
printHeaderValueOn: aStream
	"Print my header value escaping < and/or & chars with CDATA encoding if necessary."

	((self value includes: $<) or: [(self value includes: $&)])
		ifTrue: [aStream basicNextPutAll: '<!![CDATA['; basicNextPutAll: self value; basicNextPutAll: ']]>']
		ifFalse: [aStream basicNextPutAll: self value].! !

!EmailHeader methodsFor: 'printing' stamp: 'SLB 9/25/2007 14:58'!
printOn: aStream
	"Pring basic instVars of a header."

	self origString notNil
		ifTrue: [aStream nextPutAll: self origString].
	aStream cr; nextPutAll: 'name: '.
	self name printOn: aStream.
	aStream nextPutAll: ' value: '.
	self value printOn: aStream! !

!EmailHeader methodsFor: 'printing' stamp: 'SLB 1/18/2008 15:47'!
quotedValueOf: aString
	"Answer the quoted value of the string, i.e., escape < and/or & chars
	 with CDATA encoding if necessary."

	^((aString includes: $<) or: [(aString includes: $&)])
		ifTrue: ['<!![CDATA[', aString, ']]>']
		ifFalse: [aString].! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

EmailHeader class
	instanceVariableNames: ''!

!EmailHeader class methodsFor: 'testing' stamp: 'SLB 2/9/2008 15:50'!
testCommentExtraction1
	"for this one, the only ( is inside double quotes and is not a comment.
	 So the comments instvar should be nil.
	 Note: for test purposes it leaves the comment-free string in the header's value instvar."
	"EmailHeader testCommentExtraction1 inspect"

	| headerStream me |
	me := EmailHeader new.
	headerStream := ReadWriteStream with:
	'Content-Disposition: attachment;
	filename="ArchibaldPressQ&Amay24 (v.2.doc"'.
	me value: (me extractCommentsFrom: headerStream) contents.
	^me! !

!EmailHeader class methodsFor: 'testing' stamp: 'SLB 2/9/2008 15:50'!
testCommentExtraction2
	"This one has real comments.  After the test, they should be in the comments instvar.
	 Note: for test purposes it leaves the comment-free string in the header's
	 value instvar, so check to see if right thing is in there."
	"EmailHeader testCommentExtraction2 inspect"

	| headerStream me |
	me := EmailHeader new.
	headerStream := ReadWriteStream with:
	'Content-Disposition: attachment; (this comment) (and (this nested one))
	filename="ArchibaldPressQ&Amay24 (v.2.doc"'.
	me value: (me extractCommentsFrom: headerStream) contents.
	^me! !


!EmailHeader class methodsFor: 'instance creation' stamp: 'SLB 11/23/2007 22:17'!
headerSubclassFor: headerString
	"<headerString: iOf String, ^hOf EmailHeader>"
	"Answer an instance of the appropriate header subclass."

	| typeDiscriminator headerStream |
	headerStream := ReadStream on: headerString.
	(headerString indexOf: $:) > 0
		ifFalse: [^MalformedHeader new].
	typeDiscriminator := (headerStream upTo: $:) asLowercase.
	(typeDiscriminator beginsWith: 'x')
		ifTrue: [^XHeader new].
	(typeDiscriminator beginsWith: 'content-')
		ifTrue:
			[typeDiscriminator = 'content-type'
				ifTrue: [^ContentTypeHeader new].
			 typeDiscriminator = 'content-transfer-encoding'
				ifTrue: [^ContentTransferEncodingHeader new].
			 typeDiscriminator = 'content-disposition'
				ifTrue: [^ContentDispositionHeader new].
			 typeDiscriminator = 'content-description'
				ifTrue: [^ContentDescriptionHeader new].
			 typeDiscriminator = 'content-id'
				ifTrue: [^ContentIdHeader new].
			 ^OtherMIMEHeader new]
		ifFalse:
			[(typeDiscriminator = 'message-id')
				ifTrue: [^MsgIdHeader new]
				ifFalse: [^MsgHeader new]]! !


Error subclass: #EmailParserException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EmailParser-Model'!
!EmailParserException commentStamp: 'SLB 8/29/2007 08:50' prior: 0!
This exception, and its subclesses, is signalled when some parsing method can't complete.  Althoug it does not as yet (SLB 8/29/2007 08:47) do so, it should add a message to the 'incomplete' collection in the Message instance that give some indication about where the error occurred and why.!



EmailParserException subclass: #EmailMalformedDateException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EmailParser-Model'!


EmailParserException subclass: #EmailMalformedHeaderException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EmailParser-Model'!
!EmailMalformedHeaderException commentStamp: 'SLB 10/10/2007 19:40' prior: 0!
An exception generated when a parsing error occurs within a header.  In the new Email-Account schema, this will generate an 'Incomplete' tag with explanatory text.  At present SLB 10/10/2007 19:39 is doesn't!



EmailParserException subclass: #EmailMalformedMailException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EmailParser-Model'!
!EmailMalformedMailException commentStamp: 'SLB 10/10/2007 19:41' prior: 0!
Generated by an error that can't be pinned down to any particular syntactic part of the email..  In the new Email-Account schema, this will generate an 'Incomplete' tag with explanatory text.  At present SLB 10/10/2007 19:39 is doesn't!



EmailParserException subclass: #EmailUnrecognizedTypeException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EmailParser-Model'!
!EmailUnrecognizedTypeException commentStamp: 'SLB 10/10/2007 19:43' prior: 0!
Adds some more detail to the 'incomplete' annotation when some Content-Type cannot be recognized.  What was the unrecognized type?!



EmailParserException subclass: #EmailXMLOutputException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EmailParser-Model'!
!EmailXMLOutputException commentStamp: 'SLB 10/10/2007 19:43' prior: 0!
Adds some more detail to the 'incomplete' annotation, e.g., what was the problem? file not found, not writable, disk full, etc.!



Object subclass: #Folder
	instanceVariableNames: 'folderName folderDirectory subFolders mboxFileReader accountRootDirectory summaryFileStream badFileName parseStatusStream lastMessageParsed'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EmailParser-Model'!
!Folder commentStamp: 'SLB 12/19/2008 17:37' prior: 0!
This represents one account "folder", i.e., subdirectory.  The email in that folder is assumed to live in one xxx.mbox file in the subdirectory.  By convention, the file is named messages.mbox but the basename (xxx) can be any legal file name.  Note however that the name of the file does not appear in any of the parser output.

Note that one cannot make assumptions about the order of messages in a Folder/mbox.  It is natural to assume that they would be in order by date/time received.  That seems often to be the case.  But we have encountered an Account in which some folders were in order by date/tme and other folders by sender.  Presumably the reason is that the messages were written-out in the sort-order in use by the email client viewer at the time the mbox was generated.!


!Folder methodsFor: 'printing' stamp: 'SLB 2/10/2008 18:35'!
printOn: aStream
	"Print my contents on aStream."

	aStream cr; nextPutAll: 'FolderName = '.
	self folderName printOn: aStream.
	aStream nextPutAll: ', FolderDirectory = '.
	self folderDirectory printOn: aStream.
	aStream nextPutAll: ', AccountRootDirectory = '.
	self accountRootDirectory printOn: aStream.! !

!Folder methodsFor: 'printing' stamp: 'SLB 12/15/2008 19:09'!
printXMLCloseOn: aStream enclosingIndentLevel: tabs
	"Append the XML schema postscript.  All messages and sub folders have been parsed
	 and printed before this method is called."

	| eolString |
	self mboxFileReader notNil
		ifTrue:
			[aStream crtab: tabs + 1; basicNextPutAll: '<Mbox>'.
			aStream crtab: tabs + 2; basicNextPutAll: '<RelPath>'.
				aStream basicNextPutAll: self folderPath.
				aStream basicNextPutAll: '</RelPath>'.
			aStream crtab: tabs + 2; basicNextPutAll: '<Eol>'.
			eolString := #('CR' 'LF' 'CRLF')
						at: (#(#cr #lf #crlf) indexOf: self mboxFileReader eolConvention).
			aStream basicNextPutAll: eolString.
			aStream basicNextPutAll: '</Eol>'.
			aStream crtab: tabs + 1; basicNextPutAll: '</Mbox>'].
	aStream crtab: tabs; basicNextPutAll: '</Folder>'.! !

!Folder methodsFor: 'printing' stamp: 'SLB 12/15/2008 19:09'!
printXMLPreambleOn: aStream enclosingIndentLevel: tabs
	"Append the XML description of the portion of myself that preceeds the email messages."

	aStream crtab: tabs; basicNextPutAll: '<Folder>'.
	aStream crtab: tabs + 1; basicNextPutAll: '<Name> '.
		aStream basicNextPutAll: self folderName.
		aStream basicNextPutAll: '</Name>'.! !


!Folder methodsFor: 'parsing' stamp: 'SLB 6/20/2008 21:50'!
parseAndPrintXMLOn: xmlOutputFileStream enclosingIndentLevel: tabs
	"Parse the messages represented by the mboxFileReader and output the XML
	 This is a file-to-file parse.  One at a time pull out a message, undo any 'From ' quoting,
	 When done with all the mail files, output closing XML for the folder."

	self printXMLPreambleOn: xmlOutputFileStream enclosingIndentLevel: tabs.
	self parseMessagesTo: xmlOutputFileStream enclosingIndentLevel: tabs.
	self parseSubfoldersTo: xmlOutputFileStream enclosingIndentLevel: tabs.
	self printXMLCloseOn: xmlOutputFileStream enclosingIndentLevel: tabs.
	self parseStatusStream cr; nextPutAll: ' Done Parsing: ', self folderName, ' ', Time now printString.
	Transcript cr; show: ' Done Parsing: ', self folderName, ' at ', Time now printString.! !

!Folder methodsFor: 'parsing' stamp: 'SLB 11/30/2008 09:02'!
parseMessagesTo: xmlOutputFileStream enclosingIndentLevel: tabs
	"Read and parse each message in the folder and put xml results on the output stream.
	 NOTE: printCommaDelimitedSummaryOn: should not be called until after the parse and
	 XML printing is complete since the summary prints the w3c date, and error messages.
	 Errors can occur during printing and the w3c date isn't available until after the date
	 has been printed."

	| messageCount fromLine messageStream parsedMessage |
	messageCount := 0.
	self parseStatusStream cr;
		nextPutAll: ' Begin Parsing: ', self folderName, ' at ', Time now printString.
	Transcript cr; show: Time now printString, ', Parsing: ', self folderName.
	[self mboxFileReader isNil or: [self mboxFileReader atEnd]]
		whileFalse: [
			messageCount = 0
				ifTrue: [fromLine := self mboxFileReader firstFromLine]
				ifFalse: [fromLine := self mboxFileReader nextFromLine].
			messageCount := messageCount + 1.
			self lastMessageParsed: messageCount.
			Processor yield.
			(messageCount  \\ 100) = 0
				ifTrue:	   "print info on every hundredth message to track progress"
					[Transcript show: ' +', messageCount printString].
			self resetBadFileName.
			messageStream := self mboxFileReader nextMessage.
			parsedMessage := (RFC2822MIMEMessage folder: self relativePath: self folderPath).
			parsedMessage myHash: self mboxFileReader myHash finalize.
			[parsedMessage parseFromStream: messageStream.
			 parsedMessage printXMLOn: xmlOutputFileStream enclosingIndentLevel: tabs + 1.
			 parsedMessage printCommaDelimitedSummaryOn: self summaryFileStream]
				on: Error
				do: [ :ex | 
					self badFileName: (self saveProblematicMessageStream: messageStream).
					parsedMessage logFatalError: ex
								   on: self summaryFileStream
								   fileName: badFileName.
					ex return].
			messageStream isExternalStream ifTrue: [self cleanUpTempFileStream: messageStream]].
	messageCount > 0
		ifTrue:
			[self parseStatusStream crtab; nextPutAll: messageCount printString, ' messages parsed'.
			Transcript crtab; show: 'Message: ', messageCount printString, ' ',
						fromLine.  "print last line"
			 self mboxFileReader close.
			 self summaryFileStream close]
		ifFalse:
			[self parseStatusStream crtab; nextPutAll: ' Folder contains no mbox file'.
			 Transcript crtab; show: 'No mbox file'].! !

!Folder methodsFor: 'parsing' stamp: 'SLB 8/26/2007 16:28'!
parseSubfoldersTo: xmlOutputFileStream enclosingIndentLevel: tabs
	"Recursivly enter any subfolders and parse their messages"

	self getSubFolders.
	self subFolders do: [ :aSubFolder |
		aSubFolder parseAndPrintXMLOn: xmlOutputFileStream enclosingIndentLevel: tabs + 1].! !


!Folder methodsFor: 'accessing' stamp: 'SLB 8/26/2007 17:57'!
accountRootDirectory
	"<^ hOf FileDirectory>"
	"Getter. This instVar holds the directory corresponding to the top-level account."

	^accountRootDirectory! !

!Folder methodsFor: 'accessing' stamp: 'SLB 8/26/2007 17:58'!
accountRootDirectory: aFileDirectory
	"Setter. This instVar holds the directory corresponding to the top-level account."

	accountRootDirectory := aFileDirectory! !

!Folder methodsFor: 'accessing' stamp: 'SLB 3/4/2008 19:28'!
badFileName
	"<^iOf String>"
	"Getter"

	^badFileName! !

!Folder methodsFor: 'accessing' stamp: 'SLB 3/4/2008 19:28'!
badFileName: aString
	"<aString: iOf String, ^self>"
	"Setter"

	badFileName := aString! !

!Folder methodsFor: 'accessing' stamp: 'SLB 8/21/2007 21:34'!
folderDirectory
	"<^ hOf FileDirectory>"
	"Getter. This instVar holds the directory corresponding to this folder."

	^folderDirectory! !

!Folder methodsFor: 'accessing' stamp: 'SLB 8/21/2007 21:12'!
folderDirectory: aFileDirectory
	"Setter. This instVar holds the directory corresponding to this folder."

	folderDirectory := aFileDirectory! !

!Folder methodsFor: 'accessing' stamp: 'SLB 7/6/2007 22:04'!
folderName
	"<^iOf String>"
	"Getter"

	^folderName! !

!Folder methodsFor: 'accessing' stamp: 'SLB 7/6/2007 22:04'!
folderName: aString
	"<aString: iOf String, ^self>"
	"Setter"

	folderName := aString! !

!Folder methodsFor: 'accessing' stamp: 'SLB 8/22/2008 15:00'!
folderPath
	"<^iOf String>"
	"Answer the path, beginning at the topLevel, i.e., the Account directory.  Hence
	 top level folders will have a path of './account directory/toplevelfoldername',
	 The next level will have './account directory/toplevelfoldername/secondlevelfoldername'
	 and so forth."

	| accountName fullPath accountPathLength |
	accountPathLength := self accountRootDirectory pathName size.
	fullPath := self folderDirectory pathName.
	accountName := self accountRootDirectory pathName copyAfterLast: FileDirectory pathNameDelimiter.
	^('./', accountName, (fullPath last:  fullPath size - accountPathLength)) replaceAll: $\ with: $/! !

!Folder methodsFor: 'accessing' stamp: 'SLB 5/5/2008 16:43'!
getSubFolders
	"Find and create Folder objects for any subfolders."

	| subdirectory |
	self folderDirectory directoryNames asSortedCollection do: [ :aDirectoryName |	
		subdirectory := FileDirectory on: self folderDirectory pathName,
										 FileDirectory pathNameDelimiter asString,
										 aDirectoryName.
		self subFolders add:
			(Folder  folderName: aDirectoryName
					folderDirectory: subdirectory
					forAccount: self accountRootDirectory
					parseStatusStream: self parseStatusStream)].! !

!Folder methodsFor: 'accessing' stamp: 'SLB 5/13/2008 11:26'!
lastMessageParsed
	"<^hOf Integer>"
	"Getter"
	
	lastMessageParsed isNil
		ifTrue: [lastMessageParsed := 0].
	^lastMessageParsed! !

!Folder methodsFor: 'accessing' stamp: 'SLB 5/13/2008 11:26'!
lastMessageParsed: anInteger
	"Setter"

	lastMessageParsed := anInteger! !

!Folder methodsFor: 'accessing' stamp: 'SLB 2/15/2008 15:16'!
mboxFileReader
	"<^ (hOf MboxFileReader | nil)>"
	"getter for the mail box file manager.  Note: a folder may have sub-folders yet
	 have no messages.mbox file of its own.  In which case, the instVar will be nil."

	^mboxFileReader! !

!Folder methodsFor: 'accessing' stamp: 'SLB 8/21/2007 12:21'!
mboxFileReader: anMboxFileReader
	"setter for the mail box file manager"

	mboxFileReader := anMboxFileReader! !

!Folder methodsFor: 'accessing' stamp: 'SLB 5/5/2008 16:38'!
parseStatusStream
	"<^ hOf ReadWriteStream>"
	"getter"

	^parseStatusStream! !

!Folder methodsFor: 'accessing' stamp: 'SLB 5/5/2008 16:38'!
parseStatusStream: aReadWriteStream
	"setter"

	parseStatusStream := aReadWriteStream! !

!Folder methodsFor: 'accessing' stamp: 'SLB 3/4/2008 19:29'!
resetBadFileName
	"reset to nil for next message"

	badFileName := nil! !

!Folder methodsFor: 'accessing' stamp: 'SLB 8/24/2007 18:56'!
subFolders
	"Laisez faire getter.  Answer the collection of subfolders"

	subFolders isNil
		ifTrue: [subFolders := OrderedCollection new].
	^subFolders! !

!Folder methodsFor: 'accessing' stamp: 'SLB 2/27/2008 21:48'!
summaryFileStream
	"<^hOf FileStream>"
	"Laissez faire getter for the .csv (comma or tab separated vlaues) file that logs one line
	 for each parsed message."

	summaryFileStream notNil
		ifTrue: [^summaryFileStream].
	"else create new stream and write out the column titles"
	summaryFileStream := CrLfFileStream forceNewFileNamed: self folderDirectory pathName, FileDirectory pathNameDelimiter asString, 'MessageSummary.csv'.
	summaryFileStream nextPutAll: 'From,To,Date,Subject,MessagID,Hash,Errors,First Error Msg'.
	^summaryFileStream! !


!Folder methodsFor: 'file management' stamp: 'SLB 8/22/2007 13:06'!
cleanUpTempFileStream: externalMessageStream
	"<externalMessageStream: hOf FileStream, ^self>"
	"close and delete external stream."

	externalMessageStream close.
	externalMessageStream directory deleteFileNamed: externalMessageStream name.! !

!Folder methodsFor: 'file management' stamp: 'SLB 8/22/2007 13:07'!
directory
	"<^hOf FileDirectory>"
	"fetch the file directory in which the input, temp files, and attachment files live."

	^self mboxFileReader directory! !

!Folder methodsFor: 'file management' stamp: 'SLB 8/22/2007 13:08'!
mboxFileStream
	"Get from my mboxFileReader"

	^self mboxFileReader mboxFileStream! !

!Folder methodsFor: 'file management' stamp: 'SLB 3/4/2008 19:56'!
saveProblematicMessageStream: messageStream
	"<messageStream: hOf MessageStream, ^iOf String>"
	"Create (or rename) file in folder directory and save the malformed message.
	 Answer the badMessage file name.  If a badMessage file already exists, don't
	 create another one"

	| badMessageFile |
	self badFileName isNil
		ifTrue:
			[self badFileName: 'BadMessage', 999999 atRandom printString, '.eml'.
			badMessageFile := CrLfFileStream forceNewFileNamed: self folderDirectory pathName, 										FileDirectory pathNameDelimiter asString, badFileName.
			messageStream isExternalStream
				ifTrue:
					[badMessageFile nextPutAll: messageStream contentsOfEntireFile]
				ifFalse:
					[badMessageFile nextPutAll: messageStream contents].
			badMessageFile close].
	^self badFileName! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Folder class
	instanceVariableNames: ''!

!Folder class methodsFor: 'instance creation' stamp: 'SLB 6/20/2008 21:45'!
folderName: aString folderDirectory: aFileDirectory forAccount: accountDirectory parseStatusStream: aStatusStream
	"Answer an new instance of folder with the approp instvars installed."

	| folder mboxFiles |
	folder := self new.
	folder folderName: aString.
	folder folderDirectory: aFileDirectory.
	folder accountRootDirectory: accountDirectory.
	(mboxFiles := aFileDirectory fileNamesMatching: '*.mbox') size = 1
		ifTrue:
			[folder mboxFileReader: (MboxFileReader onFullPath: aFileDirectory pathName,
						FileDirectory pathNameDelimiter asString, mboxFiles first)].
	folder parseStatusStream: aStatusStream.
	^folder! !


Object subclass: #Hash
	instanceVariableNames: 'messageDigest isFinal value function eolType'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EmailParser-Model'!
!Hash commentStamp: 'SLB 8/22/2008 15:29' prior: 0!
This class stores and prints information about the hashing of an email message.  At this time, it uses the SHA1 hash algorithm.  Note that MD5 (a popular alternative) is completely compromised and should never be used.  The more secure variants of the SHA algorithms, e.g., SHA256, are considerably slower.  Without using a primitive to compute the SHA1, hashing takes about 85% of the entire email parsing time.  With the primitive, hashing still uses perhaps 20-30% of the time.  So switching to SHA256 would more than double the total parse time (perhaps way more than double).

Each message is hashed from the first character of the first line of the message (excluding the 'From ' line) through the eol character(s) at the end of the last line (excluding any subsequent empty lines between messages.  Whatever eol convention is used in the message text is hashed unaltered (see eolType below).  Thus a message obtained from a Windows server may be identical to the same message from a Linux server except for the EOLs, but the two versions WILL NOT hash the same.  To be sure the messages are the same, one must convert on or the other to the other's EOL convention and then hash them.

Structure:
 messageDigest	hOf MessageDigest -- The instance of, e.g., SHA1, that accumulates the info
				during the streaming over the message text (in MboxFileReader?).  The info
				is accumulated by a series of 'update' messages.  It is finalized exactly once
				by sending it the 'digest' message which returns the hash.

 isFinal			Boolean -- true if the messageDigest has been finalized.  When it is finalized
				it places the resulting ByteArray into the value instVar.

 function		iOf String -- one of a list defined in the mail-account XML schema.  That list
				is: MD5, WHIRLPOOL, SHA1, SHA224, SHA256, SHA384, SHA512, RIPEMD160.
				Note that MD5 is considered insecure and should not be used.  Experts consider
				SHA1 (a 160 bit hash) to be at risk in the next couple of years.

 value			iOf ByteArray -- The results of the hash.  Note that for printing, it will	
				be converted into a hexadecimal string representing the binary value of the
				hash, e..g., 40 hex characters for SHA1, 128 chars for SHA512 .

 eolType		iOf String -- one of a list defined in the mail-account XML schema.  That list
				is: CR, LF, CRLF.  CRLF is the Windows convention, CR is the Macintosh convention
				and LF is the Unix/Linux convention.  RFC2822 mandates CRLF, but many mbox
				files use instead the convention on their host machine.

The purpose of recording the eol convention used in the message is to make it easier to determine
if a hash missmatch between two emails that purportedly are the same might be due to no more than
different eol conventions.  That is, if the hash values are different when the eol conventions are
the same, there is some substantive difference in the email's text.  Note, however that one must
also ensure that the purported copy of an archived email message has no empty lines at its end.
The RFC2822 spec mandates one blank line between the end of the message and the next 'From ' line
however the message itself does not include that blank line.  Many mbox files have multiple blank
lines between emails and the number varies depending on what software has processed the messages
and/or the mbox file.  Since one extra (or different) character changes the hash, we must take care to remove all trailing blank lines before computing the hash.!


!Hash methodsFor: 'finalizing' stamp: 'SLB 10/15/2007 21:10'!
finalize
	"<^self>"
	"Get the digest and save in the value instVar.  Set isFinal to true so that finalization
	 can only be done once."

	self isFinal
		ifFalse:
			[self isFinal: true.
			self value: self digest]! !

!Hash methodsFor: 'finalizing' stamp: 'SLB 9/15/2007 11:46'!
isFinal
	"<^(true | false)>"
	"Default false.  Finalize sets it to true."

	isFinal isNil
		ifTrue: [isFinal := false].
	^isFinal! !


!Hash methodsFor: 'printing' stamp: 'SLB 4/3/2008 11:04'!
asHex
	"Convert the byteArray hash results to a hex string.  This really should be in ByteArray
	 but for code management reasons, I'll implement it here.
	 Note: Squeak 3.9's Integer>>radix: outputs true hex rather than the old 16rnn format."
	"Transcript show: (Hash new value: #(16rCD 16rC7 16r6E 16rFB 16r92 16r81 16rA1 16r48 16rA4 16r97 16r20 16r0E 16r04) asByteArray) asHex.  Transcript flush." 

	| hexStream |
	hexStream := WriteStream on: (String new: self value size).
	self value do: [ :aByte |
		aByte < 16
			ifTrue: [hexStream nextPut: $0].		   "handle leading 0"
		hexStream nextPutAll: (aByte radix: 16)].
	^hexStream contents! !

!Hash methodsFor: 'printing' stamp: 'SLB 12/13/2008 13:47'!
printXMLOn: aStream enclosingIndentLevel: tabs
	"Append the XML description of myself."

	aStream crtab: tabs; basicNextPutAll: '<Eol>'.
		aStream basicNextPutAll: self eolType asString asUppercase.
		aStream basicNextPutAll: '</Eol>'.
	aStream crtab: tabs; basicNextPutAll: '<Hash>'.
	aStream crtab: tabs + 1; basicNextPutAll: '<Value>'.
		aStream basicNextPutAll: self asHex.
		aStream basicNextPutAll: '</Value>'.
	aStream crtab: tabs + 1; basicNextPutAll: '<Function>'.
		aStream basicNextPutAll: self function.
		aStream basicNextPutAll: '</Function>'.
	aStream crtab: tabs; basicNextPutAll: '</Hash>'.! !


!Hash methodsFor: 'accessing' stamp: 'SLB 9/14/2007 22:25'!
eolType
	"<iOf Symbol>"
	"Getter for the eol convention used in the raw email text.  One of: #cr, #lf, #crlf."

	^eolType! !

!Hash methodsFor: 'accessing' stamp: 'SLB 9/14/2007 22:24'!
eolType: aSymbol
	"Setter for the eol convention used in the raw email text.  One of: #cr, #lf, #crlf."

	eolType := aSymbol! !

!Hash methodsFor: 'accessing' stamp: 'SLB 9/13/2007 11:53'!
function
	"<iOf String>"
	"Getter for the hash function used.  One of: MD5, WHIRLPOOL, SHA1, SHA224, SHA256,
	 SHA384, SHA512, or RIPEMD160.  Default is SHA1 at present."

	^function! !

!Hash methodsFor: 'accessing' stamp: 'SLB 9/13/2007 11:52'!
function: aString
	"<iOf String>"
	"Getter for the hash function used.  One of: MD5, WHIRLPOOL, SHA1, SHA224, SHA256,
	 SHA384, SHA512, or RIPEMD160"

	function := aString! !

!Hash methodsFor: 'accessing' stamp: 'SLB 10/15/2007 21:10'!
isFinal: aBoolean
	"Setter for isFinal."

	isFinal := aBoolean! !

!Hash methodsFor: 'accessing' stamp: 'SLB 9/15/2007 11:27'!
messageDigest
	"<^hOf MessageDigest>"
	"Answer the hashing instance that has been updated during reading of a message.  
	 NOTE: the messageDigest can be updated (by sending update: aByteArray) as many
	 times as you like, but it can return the resulting hash (by sending 'digest') only once.
	 The 'digest' causes an non-reversible change to the hash value.  The instance must
	 be sent the 'reinitialize' message before it can be used again.  Note also that as of this
	 writing (SLB 9/6/2007 14:24), SHA1 offers sufficient security.  However in future years
	 it would be best to use SHA512 or, perhaps, a new secure hash function yet to be
	 recommended by NSA/NIST.  I'm not doing it now, at this prototype stage, because
	 SHA512 is substantially slower."

	^messageDigest! !

!Hash methodsFor: 'accessing' stamp: 'SLB 9/15/2007 11:30'!
messageDigest: aMessageDigest
	"Set the hashing instance updated with the text for the message with original EOLs.  
	 NOTE: the messageDigest can be updated (by sending update: aByteArray) as many
	 times as you like, but it can return the resulting hash (by sending 'digest') only once.
	 The 'digest' causes an non-reversible change to the hash value.  The instance must
	 be sent the 'reinitialize' message before it can be used again.  Note also that as of this
	 writing (SLB 9/6/2007 14:24), SHA1 offers sufficient security.  However in future years
	 it would be best to use SHA512 or, perhaps, a new secure hash function yet to be
	 recommended by NSA/NIST.  I'm not doing it now, at this prototype stage, because
	 SHA512 is substantially slower."

	messageDigest := aMessageDigest.! !

!Hash methodsFor: 'accessing' stamp: 'SLB 9/15/2007 11:43'!
value
	"<iOf ByteArray>"
	"Answer the integer byteArray value of the hash.  For printing it is converted to hex.
	 If value is nil, first finalize the hashing."

	self isFinal
		ifFalse: [self finalize].
	^value! !

!Hash methodsFor: 'accessing' stamp: 'SLB 9/14/2007 21:33'!
value: aByteArray
	"Setter for the integer byteArray value of the hash.  For printing it is converted to hex."

	value := aByteArray! !


!Hash methodsFor: 'hashing' stamp: 'SLB 9/16/2007 13:04'!
digest
	"<^ iOf ByteArray>"
	"Delegate fetching the hash (i.e., the hash result) to my messageDigest"

	^self messageDigest digest! !

!Hash methodsFor: 'hashing' stamp: 'SLB 9/16/2007 13:01'!
update: aString
	"Delegate updating to my messageDigest"

	self messageDigest update: aString asByteArray! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Hash class
	instanceVariableNames: ''!

!Hash class methodsFor: 'instance creation' stamp: 'SLB 12/4/2008 15:15'!
messageDigest: aMessageDigest eolType: aSymbol
	"Return a new instance of me with has class and eolType installed.
	 Note: the special-case code for SHA1 is because the class that implements
	 it with a primitive is called SHAOne although the algortithm is SHA1.
	 If other hash functions are eventually implemented by primitives, they must
	 also be handled as special cases."

	| me functionName |
	me := self new.
	me messageDigest: aMessageDigest;
		eolType: aSymbol.
	
	functionName := aMessageDigest class name asString.
	functionName = 'SHAOne'
		ifTrue: [functionName := 'SHA1'].
	me function: functionName.
	^me! !


!Hash class methodsFor: 'testing' stamp: 'SLB 4/3/2008 16:10'!
exampleEolAndHash
	"Create example object for XML printing test."
	"Hash exampleEolAndHash printXMLPreambleOn: Transcript enclosingIndentLevel: 1. Transcript flush"
	"Hash exampleEolAndHash asHex should return: 'CDC76E5C9914FB9281A1C7E284D73E67F1809A0E' "

	| me |
	me := Hash new.
	me eolType: #crlf.
	me function: 'SHA1'.
	me value: #(16rCD 16rC7 16r6E 16r5C 16r99 16r14 16rFB 16r92 16r81 16rA1 16rC7 16rE2 16r84 16rD7 16r3E 16r67 16rF1 16r80 16r9A 16r0E) asByteArray.
	me isFinal: true.
	^me! !


Object subclass: #HeaderParameter
	instanceVariableNames: 'name value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EmailParser-Model'!
!HeaderParameter commentStamp: 'SLB 8/22/2008 16:48' prior: 0!
A header parameter parses, prints, and manages the contents of parameters in headers, expecially
MIME headers.  Note that in the normla case, i.e., with legal email, the parameter name should never contain a < or &, hence should not need to be "quoted" as XML.  The parameter value might have such characters but they would be rare.  However, in practice, improperly constructed emails have been encountered that contain html (because they were sent in the early days of adding html to email and the email clients were buggy).  One example was a header parameter name that contained nothing but '<br>'.  So both name and value must be checked for those characters and quoted if necessary. SLB 2/25/2008 20:10

For an example header with a parameter and a comment, execute
 "MIMEHeader testContentType4 inspect"

Structure:
 name			iOf String -- the parameter name, e.g., charset
 value			iOf String -- the value of that parameter, e.g., us-ascii
 comment		iOf String -- A comment is a set of ASCII characters, which is  enclosed  in
				matching parentheses and which is not within a quoted-string.  Comments
				nest, so that if an unquoted left parenthesis occurs in a comment string,
				there  must  also be a matching right parenthesis.  When a comment acts
				as the delimiter between a sequence of two lexical symbols, such as two
				atoms, it is lexically equivalent with a single SPACE.!


!HeaderParameter methodsFor: 'accessing' stamp: 'SLB 5/4/2007 21:58'!
name
	"<^iOf String>"
	"getter"

	^name! !

!HeaderParameter methodsFor: 'accessing' stamp: 'SLB 5/4/2007 21:58'!
name: aString
	"setter"

	name := aString! !

!HeaderParameter methodsFor: 'accessing' stamp: 'SLB 5/4/2007 21:59'!
value
	"<^iOf String>"
	"getter"

	^value! !

!HeaderParameter methodsFor: 'accessing' stamp: 'SLB 5/4/2007 21:59'!
value: aString
	"setter"

	value := aString! !


!HeaderParameter methodsFor: 'parsing' stamp: 'SLB 1/17/2008 19:58'!
parseFromStream: aStream
	"Parse the parameter.  The stream should be positioned just after the semicolon.
	 However, in some as yet unknown conditions, it is positioned at the semicolon.
	 Convert name *but not value* to lowercase
	 Remove any double-quotes around the parameter value."

	aStream peekFor: $;.   "Get rid of leading semicolon if necessary"
	self name: (aStream upTo: $=) asLowercase.
	(aStream peekFor: $")		  "deal with quoted values that may have embedded blanks"
		ifTrue:
			[self value: (aStream upTo: $")]
		ifFalse:
			[self value: (aStream upTo: $;)].! !


!HeaderParameter methodsFor: 'printing' stamp: 'SLB 11/18/2007 12:18'!
printOn: aStream
	"print myself"

	aStream nextPutAll: 'ParameterType -- name: '.
	self name printOn: aStream.
	aStream nextPutAll: ' value: '.
	self value printOn: aStream.! !

!HeaderParameter methodsFor: 'printing' stamp: 'SLB 12/13/2008 13:47'!
printXMLOn: aStream enclosingIndentLevel: tabs 
	"Print myself in XML on aStream indenting starting with tabs."

	aStream crtab: tabs + 1; basicNextPutAll: '<Name>', (self quoteValue: self name), '</Name>'.
	aStream crtab: tabs + 1; basicNextPutAll: '<Value>', (self quoteValue: self value), '</Value>'.! !

!HeaderParameter methodsFor: 'printing' stamp: 'SLB 11/26/2007 14:53'!
quoteValue: aString
	"Escape < and/or & chars with CDATA encoding if necessary."

	^((aString includes: $<) or: [(aString includes: $&)])
		ifTrue: ['<!![CDATA[', aString, ']]>']
		ifFalse: [aString].! !


!HeaderParameter methodsFor: 'queries' stamp: 'SLB 5/5/2007 22:39'!
isBoundary
	"Is this a boundary parameter?"

	^self name = 'boundary'! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

HeaderParameter class
	instanceVariableNames: ''!

!HeaderParameter class methodsFor: 'examples' stamp: 'SLB 9/5/2008 19:55'!
simpleMultipartExample1
	"Example parameter"
	"HeaderParameter new parseFromStream: HeaderParameter simpleMultipartExample1"

	^(ReadWriteStream with:
		'boundary="simple boundary" ') reset! !


!HeaderParameter class methodsFor: 'instance creation' stamp: 'SLB 9/27/2007 22:36'!
name: nameString value: valueString
	"Answer new one of me with the given instVar values"

	| me |
	me := self new.
	me name: nameString.
	me value: valueString.
	^me! !


EmailHeader subclass: #MIMEHeader
	instanceVariableNames: 'parameters'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EmailParser-Model'!
!MIMEHeader commentStamp: 'SLB 10/10/2007 19:45' prior: 0!
Manages issues specific to MIMEHeaders such as parsing and printing parameters.


Structure:
 parameters		hOf OrderedCollection {of: ParameterType} -- the collection of MIME header parameters.
 !


!MIMEHeader methodsFor: 'parsing' stamp: 'SLB 10/2/2007 20:32'!
parseFromCleanedHeaderStream: aReadWriteStream
	"Parse a mime header such as the Content-Type header of the form
	 Content-type: image/x-myfancyimagetype; depth=2; alpha=ignore.
	 Headers have a primary value terminated with a colon, one or more parameters separated
	 by semicolons, and one or more comments in parentheses.  A cleaned header is unfolded,
	 comments removed, and all whitespace removed except within quoted strings."

	self name: (aReadWriteStream upTo: $:) asLowercase, ':'.
	self value: (aReadWriteStream upTo: $;).
	[aReadWriteStream atEnd]
		whileFalse:
			[self parameters add: (HeaderParameter new parseFromStream: aReadWriteStream)]! !

!MIMEHeader methodsFor: 'parsing' stamp: 'SLB 10/3/2007 16:35'!
parseFromStream: aReadWriteStream withFirstLine: aString
    "Warning: The stream may contain only the text for one header or may be much larger."

    self getOrigStringFrom: aReadWriteStream withFirstLine: aString.
    ^self parseFromCleanedHeaderStream: (self removeWhitespaceSeparatorsFrom:
                (self extractCommentsFrom: self unfoldedStream))! !

!MIMEHeader methodsFor: 'parsing' stamp: 'SLB 9/26/2007 19:27'!
parseFromUnfoldedString: aString
	"Parse a mime header such as the Content-Type header of the form
	 Content-type: image/x-myfancyimagetype;depth=2;alpha=ignore (a comment)
	 Headers have a primary value terminated with a colon, one or more parameters
	 separated by semicolons, and one or more comments in parentheses."

	| inputStream |
	inputStream := ReadStream on: aString.
	self name: (inputStream upTo: $:), ':'.
	inputStream skipSeparators.
	self value: (inputStream upTo: $;).
	[inputStream atEnd]
		whileFalse:
			[self parameters add: (HeaderParameter new parseFromStream: inputStream)]! !

!MIMEHeader methodsFor: 'parsing' stamp: 'SLB 1/17/2008 19:44'!
removeWhitespaceSeparatorsFrom: aStream
	"Remove any whitespace characters (space, tab) except within double-quoted strings.
	 The input stream should be unfolded and have comments removed."

	| tokenizableStream inQuote nextChar |
	inQuote := false.
	tokenizableStream := ReadWriteStream on: String new.
	[aStream atEnd]
		whileFalse:
			[((nextChar := aStream peek) isWhitespace and: [inQuote not])
				ifTrue:
					[aStream skip: 1]
				ifFalse:
					[tokenizableStream nextPut: aStream next.
					 nextChar = $" ifTrue: [inQuote := inQuote not]]].
	^tokenizableStream reset! !


!MIMEHeader methodsFor: 'accessing' stamp: 'SLB 10/3/2007 19:37'!
parameterNamed: aString
	"Answer the named parameter ifPresent, else nil."

	self parameters do: [ :aParameter |
		aParameter name = aString
			ifTrue: [^aParameter]].
	^nil! !

!MIMEHeader methodsFor: 'accessing' stamp: 'SLB 9/27/2007 22:01'!
parameters
	"<^hOf OrderedCollection {:of hOf: ParameterType}>"
	"Laissez faire getter for parameter collection."

	parameters isNil
		ifTrue: [parameters := OrderedCollection new].
	^parameters! !


!MIMEHeader methodsFor: 'printing' stamp: 'SLB 9/25/2007 14:55'!
printOn: aStream
	"Basic printing of header."

	super printOn: aStream.
	aStream cr.
	self parameters printOn: aStream! !

!MIMEHeader methodsFor: 'printing' stamp: 'SLB 9/6/2008 08:07'!
quoteValue: aString
	"Escape < and/or & chars with CDATA encoding if necessary."

	^((aString includes: $<) or: [(aString includes: $&)])
		ifTrue: ['<!![CDATA[', aString, ']]>']
		ifFalse: [aString].! !


!MIMEHeader methodsFor: 'queries' stamp: 'SLB 10/6/2007 12:00'!
isMIME
	"Yes I am."

	^true! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

MIMEHeader class
	instanceVariableNames: ''!

!MIMEHeader class methodsFor: 'test examples' stamp: 'SLB 9/6/2008 07:57'!
testContentDescription1
	"Answer parsed simple example"
	"MIMEHeader testContentDescription1 inspect"

	^(self new parseFromUnfoldedString: 'Content-Description: a bunch of text' )! !

!MIMEHeader class methodsFor: 'test examples' stamp: 'SLB 9/6/2008 07:56'!
testContentDisposition1
	"Answer a parsed example with various comments"
	"MIMEHeader testContentDisposition1 inspect"

	| newMimeHeader |
	newMimeHeader := self new origString:
		'Content-Disposition: attachment;filename=genome.jpeg(a (nested)
 		comment);modification-date(second comment)="Wed, 12 Feb 1997 16:29:51 -0500" (third comment)'.
	^newMimeHeader parseFromCleanedHeaderStream:
				(newMimeHeader removeWhitespaceSeparatorsFrom:
					(newMimeHeader extractCommentsFrom: newMimeHeader unfoldedStream))! !

!MIMEHeader class methodsFor: 'test examples' stamp: 'SLB 9/6/2008 07:58'!
testContentId1
	"Answer parsed example that requires CDATA"
	"MIMEHeader testContentId1 inspect"

	^(self new parseFromUnfoldedString: 'Content-ID: <2866.609442237.eudoraMail.root@abc02>' )! !

!MIMEHeader class methodsFor: 'test examples' stamp: 'SLB 9/26/2007 17:09'!
testContentTransferEncoding1
	"Answer parsed example"
	"MIMEHeader testContentTransferEncoding1 inspect"

	^(self new parseFromUnfoldedString: 'Content-Transfer-Encoding: base64 (typical encoding)' )! !

!MIMEHeader class methodsFor: 'test examples' stamp: 'SLB 9/26/2007 17:21'!
testContentType1
	"Answer parsed example"
	"MIMEHeader testContentType1 inspect"

	^(self new parseFromUnfoldedString:  'Content-type: application/octet-stream; name="smime.p7s"')! !

!MIMEHeader class methodsFor: 'test examples' stamp: 'SLB 9/6/2008 08:11'!
testContentType2
	"Answer parsed example where an apparent comment is in the value.
	 Note: the value of a content type is opaque text where apparent comments are
	 not to be extracted."
	"MIMEHeader testContentType2 inspect"

	^(self new parseFromUnfoldedString: 'Content-type: image/x-myfancyimagetype;depth=2;alpha=ignore (a comment)' )! !

!MIMEHeader class methodsFor: 'test examples' stamp: 'SLB 9/26/2007 17:20'!
testContentType3
	"Answer parsed example"
	"MIMEHeader testContentType3 inspect"

	^(self new parseFromUnfoldedString:  'Content-type: text/plain; charset=us-ascii')
	! !

!MIMEHeader class methodsFor: 'test examples' stamp: 'SLB 9/8/2008 21:08'!
testContentType4
	"Answer parsed example"
	"MIMEHeader testContentType4 inspect"

	^(self new parseFromUnfoldedString: 'Content-type: text/plain; charset=us-ascii' )! !


!MIMEHeader class methodsFor: 'default headers' stamp: 'SLB 9/25/2007 13:44'!
defaultContentTransferEncodingHeader
	"To be used when the message does not contain a Content-Transfer-Encoding header."

	| me |
	me := self new.
	me name: 'Content-Transfer-Encoding:'.
	me value: '7bit'.
	me origString: ''.
	^me! !


MIMEHeader subclass: #ContentDescriptionHeader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EmailParser-Model'!
!ContentDescriptionHeader commentStamp: 'SLB 9/26/2007 22:12' prior: 0!
The ability to associate some descriptive information with a given body is often desirable. For example, it may be useful to mark an "image" body as "a picture of the Space Shuttle Endeavor." Such text may be placed in the Content-Description header field. This header field is always optional.

     description := "Content-Description" ":" *text!


!ContentDescriptionHeader methodsFor: 'printing' stamp: 'SLB 12/13/2008 13:48'!
printXMLOn: aStream enclosingIndentLevel: tabs
	"Print my contents in the mail-account.xsd format at the indicated indent level."

	aStream crtab: tabs; basicNextPutAll: '<Description>'. 
	self printHeaderValueOn: aStream. 
	aStream basicNextPutAll: '</Description>'.
	self comments isEmpty
		ifFalse:
			[aStream crtab: tabs; basicNextPutAll: '<DescriptionComments>'.
			 self comments do: [:aComment | 
				aStream crtab: tabs + 1; basicNextPutAll: aComment].
			 aStream crtab: tabs; basicNextPutAll: '</DescriptionComments>']! !


MIMEHeader subclass: #ContentDispositionHeader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EmailParser-Model'!
!ContentDispositionHeader commentStamp: 'SLB 9/26/2007 22:14' prior: 0!
disposition := "Content-Disposition" ":"
                    disposition-type
                    *(";" disposition-parm)

     disposition-type := "inline"
                       / "attachment"
                       / extension-token
                       ; values are not case-sensitive

     disposition-parm := filename-parm
                       / creation-date-parm
                       / modification-date-parm
                       / read-date-parm
                       / size-parm
                       / parameter

     filename-parm := "filename" "=" value

     creation-date-parm := "creation-date" "=" quoted-date-time

     modification-date-parm := "modification-date" "=" quoted-date-time

     read-date-parm := "read-date" "=" quoted-date-time

     size-parm := "size" "=" 1*DIGIT

     quoted-date-time := quoted-string
                      ; contents MUST be an RFC 822 `date-time'
                      ; numeric timezones (+HHMM or -HHMM) MUST be used!


!ContentDispositionHeader methodsFor: 'printing' stamp: 'SLB 12/13/2008 13:48'!
printXMLOn: aStream enclosingIndentLevel: tabs
	"Print my contents in the mail-account.xsd format at the indicated indent level."
"Disposition, DispositionFileName, DispositionComments, DispositionParams(HeaderParameter)"

	aStream crtab: tabs; basicNextPutAll: '<Disposition>'. 
	self printHeaderValueOn: aStream. 
	aStream basicNextPutAll: '</Disposition>'.
	self parameters isEmpty
		ifFalse:
			[self parameters do: [:aParam |
				aParam name asLowercase = 'filename'
					ifTrue: [aStream crtab: tabs; basicNextPutAll: '<DispositionFileName>',
					 				(self quoteValue: aParam value), '</DispositionFileName>']
					ifFalse: [aStream crtab: tabs; basicNextPutAll: '<DispositionParams>'.
					 	aParam printXMLOn: aStream enclosingIndentLevel: tabs + 1.
						aStream crtab: tabs; basicNextPutAll: '</DispositionParams>']]].
	self comments isEmpty
		ifFalse:
			[aStream crtab: tabs; basicNextPutAll: '<DispositionComments>'.
			 self comments do: [:aComment | 
				aStream crtab: tabs + 1; basicNextPutAll: aComment].
			 aStream crtab: tabs; basicNextPutAll: '</DispositionComments>']! !


MIMEHeader subclass: #ContentIdHeader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EmailParser-Model'!
!ContentIdHeader commentStamp: 'SLB 9/26/2007 22:11' prior: 0!
In constructing a high-level user agent, it may be desirable to allow one body to make reference to another. Accordingly, bodies may be labelled using the "Content-ID" header field, which is syntactically identical to the "Message-ID" header field:

     id := "Content-ID" ":" msg-id


Like the Message-ID values, Content-ID values must be generated to be world-unique.

The Content-ID value may be used for uniquely identifying MIME entities in several contexts, particularly for caching data referenced by the message/external-body mechanism. Although the Content-ID header is generally optional, its use is MANDATORY in implementations which generate data of the optional MIME media type "message/external-body". That is, each message/external-body entity must have a Content-ID field to permit caching of such data.

It is also worth noting that the Content-ID value has special semantics in the case of the multipart/alternative media type. This is explained in the section of RFC 2046 dealing with multipart/alternative. !


!ContentIdHeader methodsFor: 'printing' stamp: 'SLB 12/13/2008 13:48'!
printXMLOn: aStream enclosingIndentLevel: tabs
	"Print my contents in the mail-account.xsd format at the indicated indent level."

	aStream crtab: tabs; basicNextPutAll: '<ContentId>'.
	self printHeaderValueOn: aStream.
	aStream basicNextPutAll: '</ContentId>'.! !


!ContentIdHeader methodsFor: 'parsing' stamp: 'SLB 11/21/2007 16:15'!
parseFromCleanedHeaderStream: aReadWriteStream
	"The value of a Content-id header is an opaque string.  Don't parse comments of parameters."

	self name: (aReadWriteStream upTo: $:) asLowercase, ':'.
	self value: aReadWriteStream upToEnd.! !

!ContentIdHeader methodsFor: 'parsing' stamp: 'SLB 11/21/2007 11:55'!
parseFromStream: aReadWriteStream withFirstLine: aString
    "Warning: The stream may contain only the text for one header or may be much larger.
	Note: ContentID Headers are opaque strings so don't parse comments or parameters."

    self getOrigStringFrom: aReadWriteStream withFirstLine: aString.
    ^self parseFromCleanedHeaderStream: self unfoldedStream! !


MIMEHeader subclass: #ContentTransferEncodingHeader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EmailParser-Model'!
!ContentTransferEncodingHeader commentStamp: 'SLB 9/26/2007 22:08' prior: 0!
encoding := "Content-Transfer-Encoding" ":" mechanism

     mechanism := "7bit" / "8bit" / "binary" /
                  "quoted-printable" / "base64" /
                  ietf-token / x-token

	binary implies not line oriented!


!ContentTransferEncodingHeader methodsFor: 'printing' stamp: 'SLB 12/13/2008 13:48'!
printXMLOn: aStream enclosingIndentLevel: tabs
	"Print my contents in the mail-account.xsd format at the indicated indent level."

	aStream crtab: tabs; basicNextPutAll: '<TransferEncoding>'.
	self printHeaderValueOn: aStream.
	aStream basicNextPutAll: '</TransferEncoding>'.
	self comments isEmpty
		ifFalse:
			[aStream crtab: tabs; basicNextPutAll: '<TransferEncodingComments>'.
			 self comments do: [:aComment | 
				aStream crtab: tabs + 1; basicNextPutAll: aComment].
			 aStream crtab: tabs; basicNextPutAll: '</TransferEncodingComments>']! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

ContentTransferEncodingHeader class
	instanceVariableNames: ''!

!ContentTransferEncodingHeader class methodsFor: 'default headers' stamp: 'SLB 10/3/2007 20:25'!
default
	"Return the default instance, text/plain"
	"ContentTransferEncodingHeader default"

	| me |
	me := self new.
	me name: 'content-transfer-encoding:'.
	me value: '7bit'.
	me comments add: '(GENERATED DEFAULT)'.
	^me! !


MIMEHeader subclass: #ContentTypeHeader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EmailParser-Model'!
!ContentTypeHeader commentStamp: 'SLB 9/26/2007 22:12' prior: 0!
content := "Content-Type" ":" type "/" subtype
                *(";" parameter)
                ; Matching of media type and subtype
                ; is ALWAYS case-insensitive.

     type := discrete-type / composite-type

     discrete-type := "text" / "image" / "audio" / "video" /
                      "application" / extension-token

     composite-type := "message" / "multipart" / extension-token

     extension-token := ietf-token / x-token

     ietf-token := <An extension token defined by a
                    standards-track RFC and registered
                    with IANA.>

     x-token := <The two characters "X-" or "x-" followed, with
                 no intervening white space, by any token>

     subtype := extension-token / iana-token

     iana-token := <A publicly-defined extension token. Tokens
                    of this form must be registered with IANA
                    as specified in RFC 2048.>

     parameter := attribute "=" value

     attribute := token
                  ; Matching of attributes
                  ; is ALWAYS case-insensitive.

     value := token / quoted-string!


!ContentTypeHeader methodsFor: 'printing' stamp: 'SLB 12/13/2008 13:48'!
printXMLOn: aStream enclosingIndentLevel: tabs
	"Print my contents in the mail-account.xsd format at the indicated indent level."
" ContentTypeParams(HeaderParameter)"

	| param |
	aStream crtab: tabs; basicNextPutAll: '<ContentType>'.
	self printHeaderValueOn: aStream.
	aStream basicNextPutAll: '</ContentType>'.
	(param := self parameterNamed: 'charset') isNil
		ifFalse:
			[aStream crtab: tabs; basicNextPutAll: '<Charset>';
			 	basicNextPutAll:  param value;
			 	basicNextPutAll: '</Charset>'].
	(param := self parameterNamed: 'content-name') isNil
		ifFalse:
			[aStream crtab: tabs; basicNextPutAll: '<ContentName>';
			 	basicNextPutAll:  param value;
				basicNextPutAll: '</ContentName>'].
	(param := self parameterNamed: 'boundary') isNil
		ifFalse:
			[aStream crtab: tabs; basicNextPutAll: '<BoundaryString>';
			 	basicNextPutAll:  param value;
			 	basicNextPutAll: '</BoundaryString>'].
	self comments isEmpty
		ifFalse:
			[aStream crtab: tabs; basicNextPutAll: '<ContentTypeComments>'.
			 self comments do: [:aComment | 
				aStream crtab: tabs + 1; basicNextPutAll: (self quotedValueOf: aComment)].
			 aStream crtab: tabs; basicNextPutAll: '</ContentTypeComments>']! !


!ContentTypeHeader methodsFor: 'queries' stamp: 'SLB 9/27/2007 20:45'!
isAppledouble
	"self explanatory"

	^self value asLowercase endsWith: 'appledouble'! !

!ContentTypeHeader methodsFor: 'queries' stamp: 'SLB 9/27/2007 20:46'!
isApplication
	"self explanatory"

	^self value asLowercase beginsWith: 'application'! !

!ContentTypeHeader methodsFor: 'queries' stamp: 'SLB 2/8/2008 22:34'!
isAudio
	"self explanatory"

	^self value asLowercase beginsWith: 'audio'! !

!ContentTypeHeader methodsFor: 'queries' stamp: 'SLB 9/27/2007 20:47'!
isImage
	"self explanatory"

	^self value asLowercase beginsWith: 'image'! !

!ContentTypeHeader methodsFor: 'queries' stamp: 'SLB 2/20/2008 10:17'!
isMessage
	"Answer whether or not the content type is a message/xxx.  message/rfc822, has no required parameters in the Content-Type field. Less common subtypes, 'partial' and 'External-body', do have required parameters.  At this writing, SLB 10/20/2007 17:17, the parser only handles message/rfc822.  If the header is a message header and not message/rfc822, signal error."

	| messageType |
	messageType := self value asLowercase.
	(messageType beginsWith: 'message')
		ifTrue:
			[messageType = 'message/rfc822'
				ifTrue: [^true]
				ifFalse: [EmailParserException signal: 'unhandled content-type: ', messageType]].
	^false! !

!ContentTypeHeader methodsFor: 'queries' stamp: 'SLB 9/27/2007 20:48'!
isMultipart
	"self explanatory"

	^self value asLowercase beginsWith: 'multipart'! !

!ContentTypeHeader methodsFor: 'queries' stamp: 'SLB 9/27/2007 20:48'!
isSimple
	"Is it a simple type, i.e., not a multipart or child.  Simple types include
	 text, image, attachment, etc."

	| myType |
	myType := self value asLowercase.
	^ (myType beginsWith: 'text')
		or: [myType beginsWith: 'image']
		or: [myType beginsWith: 'application']
		or: [myType beginsWith: 'audio']
		or: [myType beginsWith: 'video']! !

!ContentTypeHeader methodsFor: 'queries' stamp: 'SLB 9/27/2007 20:49'!
isText
	"self explanatory"

	^self value asLowercase beginsWith: 'text'! !

!ContentTypeHeader methodsFor: 'queries' stamp: 'SLB 8/20/2008 10:47'!
isVideo
	"self explanatory"

	^self value asLowercase beginsWith: 'video'! !

!ContentTypeHeader methodsFor: 'queries' stamp: 'SLB 2/20/2008 10:53'!
multiPartBoundary
	"Return the boundary string from the multipart parameter"

	self parameters do: [ :aParameter |
		aParameter isBoundary
			ifTrue: [^aParameter value]].
	EmailParserException signal: 'Missing boundary parameter Content-Type header'! !


!ContentTypeHeader methodsFor: 'parsing' stamp: 'SLB 1/16/2008 10:01'!
parseFromStream: aReadWriteStream withFirstLine: aString
    "Warning: The stream may contain only the text for one header or may be much larger.
	NOTE, this overridees the same method in MIMEHeader because some boundary Strings have
	open and closed parenses in them that are otherwise removed by the superclass method.
		   SLB 1/15/2008 22:25"

    self getOrigStringFrom: aReadWriteStream withFirstLine: aString.
    ^self parseFromCleanedHeaderStream:
			(self removeWhitespaceSeparatorsFrom: self unfoldedStream)! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

ContentTypeHeader class
	instanceVariableNames: ''!

!ContentTypeHeader class methodsFor: 'test examples' stamp: 'SLB 1/17/2008 19:25'!
exampleWithBoundaryParameter1
	"Example Content-Type header"
	"ContentTypeHeader new parseFromStream: ContentTypeHeader exampleWithBoundaryParameter1 withFirstLine: 'Content-Type: multipart/related;'"

	^(ReadWriteStream with:
	  '
	type="multipart/alternative";
	boundary="=====================_86070711==_.REL"') reset! !

!ContentTypeHeader class methodsFor: 'test examples' stamp: 'SLB 9/27/2007 20:25'!
exampleWithCharacterSetParameter1
	"Example Content-Type header"
	"ContentTypeHeader new parseFromStream: ContentTypeHeader exampleWithCharacterSetParameter1"

	^(ReadWriteStream with:
	  'Content-Type: text/html; charset=ISO-8859-1') reset! !

!ContentTypeHeader class methodsFor: 'test examples' stamp: 'SLB 9/27/2007 20:25'!
exampleWithNameParameter1
	"Example Content-Type header"
	"ContentTypeHeader new parseFromStream: ContentTypeHeader exampleWithNameParameter1"

	^(ReadWriteStream with:
	  ' application/octet-stream; 
		name="smime.p7s"') reset! !

!ContentTypeHeader class methodsFor: 'test examples' stamp: 'SLB 9/27/2007 20:26'!
exampleWithOtherParameters1
	"Example Content-Type header"
	"ContentTypeHeader new parseFromStream: ContentTypeHeader exampleWithOtherParameters1"

	^(ReadWriteStream with:
	  ' image/x-myfancyimagetype;depth=2;alpha=ignore ') reset! !


!ContentTypeHeader class methodsFor: 'default headers' stamp: 'SLB 10/20/2007 17:35'!
default
	"Return the default instance, text/plain"
	"ContentTypeHeader default"

	| me |
	me := self new.
	me parameters add: (HeaderParameter name: 'charset:' value: 'us-ascii').
	me name: 'content-type:'.
	me value: 'text/plain'.
	me comments add: '(GENERATED DEFAULT)'.
	^me! !


EmailHeader subclass: #MalformedHeader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EmailParser-Model'!
!MalformedHeader commentStamp: 'SLB 3/9/2008 20:54' prior: 0!
MalformedHeader models an element in the correct position to be a header, i.e., prior to the blank line that is supposed to signal the end of the headers, that, nonetheless, has no legal header name, i.e., a string ended by a colon.
!


!MalformedHeader methodsFor: 'parsing' stamp: 'SLB 3/9/2008 19:04'!
parseFromStream: aReadWriteStream withFirstLine: aString
    "Warning: The stream may contain only the text for one header or may be much larger.
	Remove commas and doublequotes so it doesn't screw up the comma-separated-value table."

	| safeErrorString currentMessage |
	safeErrorString := aString reject: [:each | each = $, or: [each = $"]].
	self getOrigStringFrom: aReadWriteStream withFirstLine: aString.
	currentMessage := RFC2822MIMEMessage CurrentMessage.
	currentMessage
		recordParseErrorLocation: 'InputStream position: ', aReadWriteStream position printString
		type: 'Illegal header. Original string:  ', safeErrorString.
	currentMessage folder saveProblematicMessageStream: aReadWriteStream.
	self name: 'malformed:'.
	self value: aString! !


!MalformedHeader methodsFor: 'printing' stamp: 'SLB 12/13/2008 13:47'!
printXMLOn: aStream enclosingIndentLevel: tabs
	"Print my contents in the mail-account.xsd format at the indicated indent level."

	aStream crtab: tabs; basicNextPutAll: '<Header>'.
	aStream crtab: tabs + 1; basicNextPutAll: '<Name>';
			basicNextPutAll: 'malformed:';
		     basicNextPutAll: '</Name>'.
	aStream crtab: tabs + 1; basicNextPutAll: '<Value>'. 
	self printHeaderValueOn: aStream. 
	aStream basicNextPutAll: '</Value>'.
	aStream crtab: tabs; basicNextPutAll: '</Header>'! !


!MalformedHeader methodsFor: 'queries' stamp: 'SLB 11/24/2007 16:04'!
isMalformed
	"Yes I am"

	^true! !


Object subclass: #MboxFileReader
	instanceVariableNames: 'fullPath mboxFileStream hash eolConvention eolChar'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EmailParser-Model'!
!MboxFileReader commentStamp: 'SLB 8/22/2008 21:32' prior: 0!
The MboxFileReader class is responsible for extracting messages, one at a time, from a file in mbox format in which RFC822(ish) messages are concatenated.  Each message is introduced by and separated from the prior message by a "From " line.  These are lines that begin with $F $r $o $m $space.  The 'From ' line always looks like 'From envsender date moreinfo'.  envsender must be one word, without spaces or tabs; it is usually the envelope sender of the message.  Date is the delivery date of the message.  It always contains exactly 24 characters in asctime format: e.g.,  Sun Nov  6 08:49:37 1994.   Time is always supposed to be GMT.  However this parser does not depend upon anything in the From line other than the leading 'From '.

Any From  line marks the beginning of a message.  The mbox reader ***should not attempt*** to take advantage of the fact that every 'From ' line (past the beginning of the file) is supposed to be preceded by a blank line, nor that every message is supposed to be followed by exactly one blank line.  Those conventions are commonly ignored by various mail servers.

Once the reader finds a message, it extracts a (possibly corrupted) envelope sender and delivery date out of the From  line.  It then reads until the next From line or end of file, whichever comes first.  It strips off the final blank line and deletes one leading $> from >From  lines, >>From lines, >>>From lines and so on.  This process is called 'unquoting' the From lines.

The result of the above transformations is more or less an RFC 822 message.  What that means is that it is very much in the style of RFC822 although it may (usually will) contain messages with MIME extensions that did not exist at the time the RFC822 standard was defined.

EOL Issues: Note that an mbox file is a concatenation of RFC2822 messages.  If the file is created
by an email tool, one can reliably assume that every line in the file is terminated by the
same EOL sequence.  Different standards disagree about exactly what that sequence should be.
Some require CrLf and others explicitly reject CrLf in favor of just the *nix Lf convention.
Nonetheless, within one mbox file created by email client software, they will be the same.  The danger is that some well-meaning person may concatenate two or more mbox files together
(using an editor) and, in that case,
it is possible, although not very likely, that one chunk of messages uses one convention
and second chunck might use a different EOL convention.  So the MboxFileReader detects
what EOL is in use for each message.  However, it is not legitimate for there to be more
than one EOL convention within one message.  That does not mean that such a thing could
never occur.  There are many not very well crafted email clients in the world.
When incorporating a forwarded message, one of these could fail to ensure that
the EOL convention in the forwarded message is the same as the native message.  If the
MboxFileReader detects such a problem, it will signal an error.

Note: other format mail files such as .pst or .nsf will require different readers.  At present
the Folder class just assumes an mbox input file.

InstVar structure:

 fullPath 			iOf String -- the full path of the messages.mbox file beginning at the
					root of whatever machine is doing the parsing, e.g., C: on a Windows box
					or / on a *nix box.

 mboxFileStream 	iOf StandardFileStream -- the stream containing the 'messages.mbox' file

 messageDigest		hOf MessageDigest -- computes and returnd the hash/checksum.  For initial
					trials of the parser it will use SHA1.  If better security is desired, SHA512
					can be used.

 eolConvention		iOf String -- The one or two characters used to terminate a line in the
					particular mail.  Note: the RFC2822 spec asserts that EOL should be CRLF.
					Incontrast, RFC4155 (http://www.rfc-editor.org/rfc/rfc4155.txt) requires
					that email EOLs be a single LF and */must not/* use CRLF.  Despite these
					'standards', Mac software might use CR for compatibility with their OS.
					Since the hash is exquisitely sensitive to every single character those who
					later wish to compare the saved hash with a hash of their own copy of a
					message must know which EOL characters were involved in the hash
					computation.  If their copy uses, say, LF and the archived copy uses
					CRLF, the hashes won't match unless they transform their copy to use CRLF.

hash				hOf Hash -- the instance of Hash used to compute the checksum (hash) of
					each message.  The hash is computed over the characters in the message starting
					with the first char of the first header (i.e., ignoring the 'From ' line which
					is not actually a part of the message, it is created by the email transmission
					software en route), through the EOL at the end of the last non-empty line
					of the message (because various transmission protocols may, or may not, add
					extra blank lines between messages).!


!MboxFileReader methodsFor: 'accessing' stamp: 'SLB 7/8/2007 16:26'!
directory
	"<^hOf FileDirectory>"
	"Return the directory containing the file to be parsed."

	^FileDirectory directoryEntryFor: self fullPath ! !

!MboxFileReader methodsFor: 'accessing' stamp: 'SLB 7/8/2007 15:26'!
directoryPath
	"<^iOf String>"
	"Return the path to the directory containing the file to be parsed."

	^self fullPath copyUpToLast: FileDirectory pathNameDelimiter! !

!MboxFileReader methodsFor: 'accessing' stamp: 'SLB 9/17/2007 12:03'!
eolChar
	"<^iOf Character>"
	"Answer the character that signals an end of line."

	eolChar isNil  "Note: it is nilled whenever the eolConvention is reset."
		ifTrue:
			[(eolConvention == #cr or: [eolConvention == #crlf])
				ifTrue: [eolChar := Character cr]
				ifFalse: [eolChar := Character lf]].
	^eolChar! !

!MboxFileReader methodsFor: 'accessing' stamp: 'SLB 9/17/2007 12:05'!
eolConvention
	"<^iOf Symbol>"
	"Getter for the End of Line convention used by each message in the file to be parsed.
	 Acceptable symbols are: #cr, #lf, and #crlf.  The eol convention MUST be the same
	 throughout any given email message.  All legitimate email systems will guarantee that.
	 It is re-determined for each message because an mbox file could be cobbled together
	 (via an editor) from different mbox files that come from different machines with different
	 EOL conventions.  While this is unlikely, it cannot be ruled out epecially with submissions
	 of old email from pretty sketchy sources.  And it doesn't cost much time to do the detection
	 for each message."

	^eolConvention! !

!MboxFileReader methodsFor: 'accessing' stamp: 'SLB 8/28/2008 23:44'!
eolConvention: aSymbol
	"Setter for the eolConvention of the file to be parsed.  See comment in the getter
	 for more detail.  Acceptable symbols are: #cr, #lf, and #crlf"

	eolConvention := aSymbol.
	eolChar := nil 	"to force the eolChar to be recomputed"! !

!MboxFileReader methodsFor: 'accessing' stamp: 'SLB 7/8/2007 12:52'!
fullPath
	"<^iOf String>"
	"Getter for the full path of the file to be parsed."

	^fullPath! !

!MboxFileReader methodsFor: 'accessing' stamp: 'SLB 7/8/2007 12:53'!
fullPath: aString
	"Setter for the full path of the file to be parsed."

	fullPath := aString! !

!MboxFileReader methodsFor: 'accessing' stamp: 'SLB 8/30/2007 23:44'!
maxInMemory
	"<^hOf Integer>"
	"Maximum size beyond which the message is converted to a temp file.
	 This is also the size that triggers attachments to be saved in external files
	 rather than included in the XML stream.  The number is maintained in a
	 class variable of the Message class."

	^RFC2822MIMEMessage MaxBodyPartLength! !

!MboxFileReader methodsFor: 'accessing' stamp: 'SLB 9/11/2007 10:17'!
mboxFileStream
	"<^ hOf StandardFileStream>"
	"getter"

	^mboxFileStream! !

!MboxFileReader methodsFor: 'accessing' stamp: 'SLB 9/11/2007 10:17'!
mboxFileStream: aStandardFileStream
	"setter"

	mboxFileStream := aStandardFileStream! !

!MboxFileReader methodsFor: 'accessing' stamp: 'SLB 1/24/2008 21:46'!
myHash
	"<^hOf Hash>"
	"Getter Answer the hashing instance."

	^hash! !

!MboxFileReader methodsFor: 'accessing' stamp: 'SLB 8/28/2008 23:45'!
myHash: aHash
	"Setter"

	hash := aHash! !


!MboxFileReader methodsFor: 'file management' stamp: 'SLB 5/18/2007 21:10'!
atEnd
	"Return true if the mboxFileStream is at its end, else false."

	^self mboxFileStream atEnd! !

!MboxFileReader methodsFor: 'file management' stamp: 'SLB 8/24/2007 15:01'!
close
	"Close the fileStream"

	^self mboxFileStream close! !


!MboxFileReader methodsFor: 'queries' stamp: 'SLB 12/2/2007 13:20'!
firstEOLCharacter
	"<^iOf Character>"
	"Answer the End-Of-Line character, cr or lf.  If it is crlf, anser cr."

	^self eolConvention == #lf 
		ifTrue: [Character lf]
		ifFalse: [Character cr]! !

!MboxFileReader methodsFor: 'queries' stamp: 'SLB 1/18/2008 14:43'!
isLegalChar: char
	"Answer whether this char is legal in a UTF-8 XML file."

	| intValue |
	intValue := char asInteger.
	(intValue >= 32 and: [intValue <= 127])
		ifTrue: [^true].
	(intValue = 9 "tab" or: [intValue = 10 "Lf" or: [intValue = 13 "Cr"]])
		ifTrue: [^true].
	^false! !


!MboxFileReader methodsFor: 'message reading' stamp: 'SLB 9/16/2007 15:40'!
convertToCr: aString
	"<aString: iOfString, ^iOf String>"
	"Ensure that aString is a line terminated by Cr."

	self eolConvention == #crlf
		ifTrue: [^aString copyFrom: 1 to: aString size - 1].
	self eolConvention == #lf
		ifTrue: [aString at: aString size put: Character cr].
	^aString! !

!MboxFileReader methodsFor: 'message reading' stamp: 'SLB 8/28/2008 21:30'!
firstFromLine
	"Answer the first line of the mbox file which must be the 'From ' line that marks the
	 beginning of the first message.
	 Note: MboxFileReader>>rawFirstLine guarantees that the eol is a single Cr char no
	 matter which actual EOL convention is being used."

	| nextLine |
	nextLine := self rawFirstLine.
	(nextLine notNil and: [nextLine beginsWith: 'From '])
		ifFalse:
			[EmailMalformedMailException signal: 'first line of mbox file not a ''From '' line'].
	^nextLine copyFrom: 1 to: nextLine size -1	  "Remove eol"! !

!MboxFileReader methodsFor: 'message reading' stamp: 'SLB 8/28/2008 22:16'!
nextFromLine
	"Skip to and return the 'From ' line that marks the beginning of the next message.
	 Note: self wasLastLineOfThisMessage is what makes sure that the next line if we aren't at the
	 the EOL is a 'From ' line."

	| nextLine peekChar |
	self skipBlankLines.
	nextLine := WriteStream on: (String new: 70).
	[(peekChar := self mboxFileStream peek) = Character cr or: [peekChar = Character lf]]
		whileFalse:
			[nextLine nextPut: self mboxFileStream next].
	self mboxFileStream next.  "skip first eol char"
	self mboxFileStream peek = Character lf
		ifTrue: [self mboxFileStream next].  "skip the lf in a crlf eol"
	^nextLine contents! !

!MboxFileReader methodsFor: 'message reading' stamp: 'SLB 11/26/2008 21:45'!
nextMessage
	"<^hOf MessageStream>"
	"Extract the text for the next RFC2822 email message from the mbox FileStream, feed
	 it to a MessageStream to be input to the parser.  Before this method is called, the
	 mboxFileStream must be at the start of the line after its introductory 'From ' line.
	 Before each line is passed to the messageStream, it must be added to the hash (i.e.,
	 message digest), then any '>From ' quoting must be removed, and the EOL must be
	 forced to the Macintosh Cr convention."

	| messageStream nextLine |
	self myHash: (Hash messageDigest: SHAOne new eolType: self eolConvention).
	messageStream := MessageStream on: (ReadWriteStream with: '')
									in: self directoryPath.
	[(nextLine := self rawNextMessageLine) isNil] 	"nil signals this message is at its end"
		whileFalse:
			[messageStream nextPutAll: (self convertToCr: (self unquote: nextLine))].
	messageStream resetToBeginning.
	^messageStream
! !

!MboxFileReader methodsFor: 'message reading' stamp: 'SLB 2/20/2008 10:56'!
rawFirstLine
	"<(^iOf String | nil)>"
	"Answer the first line in the current mbox message, or nil if at end. Terminate the line with
	 the internal Cr EOL character.  However, eat whatever EOL char(s) actually terminated the line."

	| newStream char eolMarker |
	self mboxFileStream atEnd ifTrue: [^nil].
	newStream _ WriteStream on: (String new: 100).
	self eolConvention == #lf 
		ifTrue: [eolMarker := Character lf]
		ifFalse: [eolMarker := Character cr].
	[(char := self mboxFileStream next) isNil
		or: [char = eolMarker]]
			whileFalse: [newStream nextPut: char].
	newStream nextPut: Character cr.    "add the internal (Mac) EOL character to terminate the line"
	self eolConvention == #crlf
		ifTrue:
			[char := self mboxFileStream next. 	"Eat the lf at the end of the line in the mbox file."
			 char = Character lf		  "it better be an Lf"
				ifFalse: [EmailMalformedMailException signal: 'improper EOL in CrLf file']].
	^ newStream contents! !

!MboxFileReader methodsFor: 'message reading' stamp: 'SLB 8/28/2008 22:16'!
rawNextMessageLine
	"<(^iOf String | nil)>"
	"Answer the next line in the current mbox message (including its EOL), or nil if end of message.
	 End of message is signalled by End-Of-File or a 'From ' line.  Note: if the reader is crossing a
	 boundary from a cr to an lf convention region or vice versa (extremely rare), the new eol
	 may be ignored.  So the next line must be terminated at max line length which is 998.  It is also
	 possible for a malformed message to have a line longer than 998.  If both rare events happen
	 in the same region of mbox text, this code will fail.  Such is life.  SLB 9/16/2007 20:51."
	"And, it turns out that perhaps 1% of messages have illegal UTF-8 chars that must be 'escaped'.
	 In that case must compute hash on orig string and return the escaped string.  This code is
	 way too complex because it is doing too many things. It needs refactoring.  SLB 12/1/2007 14:41
	 Note: self wasLastLineOfThisMessage is what makes sure that the next line if we aren't at the
	 the EOL is a 'From ' line."

	| rawStream char eolMarker escapedStream hasIllegalUTF |
	(self mboxFileStream atEnd or: [self wasLastLineOfThisMessage]) ifTrue: [^nil].
	hasIllegalUTF := false.
	rawStream := WriteStream on: (String new: 80).
	eolMarker := self firstEOLCharacter.
	[self mboxFileStream atEnd or: [(char := self mboxFileStream next) = eolMarker]]
		whileFalse:
			[rawStream nextPut: char.	"always append to rawStream"
			 ((self isLegalChar: char) not or: [hasIllegalUTF])
				ifTrue:
					[hasIllegalUTF
						ifFalse:		"encountered the first illegal char in this line"
							[hasIllegalUTF := true.
							 escapedStream := WriteStream on: String new.	
							 escapedStream nextPutAll: (rawStream contents
														copyFrom: 1 to: rawStream size - 1)].
					 (self isLegalChar: char)  "append char, or its escaped representation"
						ifFalse:
							[escapedStream 	nextPutAll: '&#x00';
											nextPut: (char asInteger // 16) asHexDigit;
											nextPut: (char asInteger \\ 16) asHexDigit;
											nextPut: $; ]
						ifTrue:
							[escapedStream nextPut: char]]].
	rawStream nextPut: eolMarker.    "add the first or only EOL character"
	hasIllegalUTF ifTrue: [escapedStream nextPut: eolMarker].
	(self eolConvention == #crlf and: [rawStream size <999])
		ifTrue:
			[char := self mboxFileStream next.  "this should be the lf in the crlf"
			 char = Character lf		  "If it isn't something is very wrong."
				ifTrue:
					[rawStream nextPut: char.
					 hasIllegalUTF ifTrue: [escapedStream nextPut: char]]
				ifFalse: [EmailMalformedMailException signal: 'improper EOL in CrLf file']].
	self myHash update: rawStream contents.   "always update hash with the raw stream"	
	^ hasIllegalUTF		"return the escaped stream if there have been any illegal chars"
		ifTrue: [escapedStream contents]
		ifFalse: [rawStream contents]! !

!MboxFileReader methodsFor: 'message reading' stamp: 'SLB 7/25/2007 11:49'!
resetToBeginning: messageStream
	"Ensure messageStream is positioned at its start."

	messageStream isExternalStream
		ifTrue:
			[messageStream close.
			 messageStream openReadOnly]
		ifFalse:
			[messageStream reset]! !

!MboxFileReader methodsFor: 'message reading' stamp: 'SLB 9/17/2007 20:23'!
skipBlankLines
	"<^self>"
	"Skip any blank lines.  Don't worry about repositioning the FileStream.  This first-cut
	 version simply ignores consecutive cr and/or lf chars.  If it turns out that real emails
	 contain legitimate crs or lfs, i.e., they play some role other than in eols, this will have
	 to be more careful.  Note: peek returns nil if stream atEnd, so loop will terminate atEnd."

	| peekChar |
	[(peekChar := self mboxFileStream peek) = Character cr or: [peekChar = Character lf]]
		whileTrue: [self mboxFileStream next]! !

!MboxFileReader methodsFor: 'message reading' stamp: 'SLB 9/16/2007 14:55'!
unquote: aString
	"<aString: iOfString, ^iOf String>"
	"Remove one $> from in front of any line that begins with some number
	 of $> followed by 'From '.  Note that the algorithm is designed to rule out quoting
	 as quickly as possible.  The last two tests are in the order given because
	 findString:startingAt: uses a primitive.  Otherwise it would, in general, be much
	 slower than seeing if the leading chars are all > since most lines don't contain a
	 'From ' at all, hence would have to be traversed all the way to the end of line."

	| index |
	aString size < 6
		ifTrue: [^aString].  			"String too short to be a quoted From line"
	(aString at: 1) = $>
		ifFalse: [^aString].			"String doesn't begin with >"
	(index := aString findString: 'From ' startingAt: 2) > 0
		ifFalse: [^aString].			"String doesn't contain 'From '"
	2 to: index - 1 do: [ :i |
			(aString at: i) = $>
				ifFalse: [^aString]].	"String not preceeded by only > chars"
	^aString copyFrom: 2 to: aString size! !

!MboxFileReader methodsFor: 'message reading' stamp: 'SLB 12/1/2007 22:22'!
wasLastLineOfThisMessage
	"<(true | false)>"
	"Answer true if the next line is empty and there are only empty lines between it and
	 the EOF or the next 'From ' line indicating start of next message.  That is, if all chars
	 between here and start of next message are either cr or lf.  NEED TO PROFILE PARSING TO
	 SEE IF THIS AS A PERFORMANCE HOG"

	| peekChar savedPosition | 
	peekChar := self mboxFileStream peek.
	((peekChar ~= self eolChar) and: [peekChar ~= $F])
		ifTrue: [^false].
	savedPosition := self mboxFileStream position.
	self skipBlankLines.
	self mboxFileStream atEnd
		ifFalse:
			[(self mboxFileStream next: 5) = 'From '
				ifTrue:
					[self mboxFileStream position: savedPosition.
					 ^true]].
	self mboxFileStream position: savedPosition.
	^ false! !


!MboxFileReader methodsFor: 'EOL Detection' stamp: 'SLB 8/28/2008 23:42'!
detectLineEndConvention
	"<^ iOf Symbol>"
	"Answer the line end convention used in the mboxFileStream. The result is one of
	 {#cr, #lf or #crlf}.  Note: since email is line oriented and the max legal RFC2822 line length
	 is 998 (including the EOL), it only has to look at most at the first line.  In practice, the first
	 line is less than 78 chars.  If it can't find an eol in the first 998 characters, something is
	 drastically wrong -- signal error."

	| convention nextCharacter charactersRead originalPosition lf cr |
	charactersRead := 0.
	lf := Character lf.
	cr := Character cr.
	originalPosition := mboxFileStream position.
	[mboxFileStream atEnd not and: [charactersRead < 1000 "max look ahead count"]]
		whileTrue: 
			[nextCharacter := mboxFileStream next.
			nextCharacter = lf
				ifTrue: 
					[mboxFileStream position: originalPosition.
					^ #lf].
			nextCharacter = cr
				ifTrue: 
					[mboxFileStream peek = lf
						ifTrue: [convention := #crlf]
						ifFalse: [convention := #cr].
					mboxFileStream position: originalPosition.
					^ convention].
			charactersRead := charactersRead + 1].
	mboxFileStream position: originalPosition.
	EmailMalformedMailException signal: 'No EOL in first 1000 characters of message'! !

!MboxFileReader methodsFor: 'EOL Detection' stamp: 'SLB 9/16/2007 21:14'!
eolStringFor: eolSymbol
	"<^ iOf String>"
	"Answer the one or two character string for the given eol Symbol -- one of #crlf, #cr, or #lf"

	| eolString |
	eolSymbol == #crlf
		ifTrue: [eolString := String with: Character cr with: Character lf].
	eolSymbol == #cr
		ifTrue: [eolString := String with: Character cr].
	eolSymbol == #lf
		ifTrue: [eolString := String with: Character lf].
	^eolString! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

MboxFileReader class
	instanceVariableNames: ''!

!MboxFileReader class methodsFor: 'instance creation' stamp: 'SLB 8/29/2008 21:49'!
onFullPath: fullPath
	"<fullPath: iOf String>"
	"Answer a new MboxFileReader set to read the mbox file given by the full path string.
	 NOTE: deliberately uses StandardFileStream rather than CrLfFileStream because it must
	 hash each message in its original form (after 'From ' quoting is removed and extra
	 terminating blank lines removed."

	| mboxReader |
	mboxReader := self new.
	mboxReader fullPath: fullPath.
	mboxReader mboxFileStream: (StandardFileStream readOnlyFileNamed: fullPath).
	mboxReader eolConvention: mboxReader detectLineEndConvention.
	^mboxReader! !


Object subclass: #MessageStream
	instanceVariableNames: 'inputStream directoryPath'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EmailParser-Model'!
!MessageStream commentStamp: 'SLB 8/28/2008 23:47' prior: 0!
This class is responsible for managing the Stream of text representing one email message.  It converts
the in-memory stream to a FileStream if it's size exceeds the max allowed\,

Before passing lines of text to this class, any aspects of that text that are extraneous to RFC2822 message syntax are removed.  For example, the 'From ' quoting in an mbox format is removed.  

InstVer structure:
  inputStream		hOf ReadWriteStream -- the stream that contains the message text.
					It begins as a ReadWriteStream and is converted to a FileStream if the 
					message text is too long.

 directoryPath		iOf String -- the full path of the directory in which the mbox file lives,
					hence the directory in which a temp FileStream is created if needed!


!MessageStream methodsFor: 'accessing' stamp: 'SLB 9/6/2007 23:38'!
directoryPath
	"<ihOf String>"
	"Getter"

	^directoryPath! !

!MessageStream methodsFor: 'accessing' stamp: 'SLB 9/6/2007 23:40'!
directoryPath: aString
	"Set the directoryPath of the input stream for this MessageStream"

	directoryPath := aString! !

!MessageStream methodsFor: 'accessing' stamp: 'SLB 9/6/2007 21:51'!
inputStream
	"<^hOf ReadWriteStream>"
	"Getter"

	^inputStream! !

!MessageStream methodsFor: 'accessing' stamp: 'SLB 9/6/2007 21:50'!
inputStream: aReadWriteStream
	"Set the input stream for this MessageStream"

	inputStream := aReadWriteStream! !


!MessageStream methodsFor: 'stream management' stamp: 'SLB 9/6/2007 23:17'!
atEnd
	"pass on to the inputStream"

	^self inputStream atEnd! !

!MessageStream methodsFor: 'stream management' stamp: 'SLB 9/7/2007 09:30'!
close
	"pass on to the inputStream"

	^self inputStream close! !

!MessageStream methodsFor: 'stream management' stamp: 'SLB 1/15/2008 20:35'!
contents
	"pass on to the inputStream"

	^self inputStream contents! !

!MessageStream methodsFor: 'stream management' stamp: 'SLB 1/15/2008 20:36'!
contentsOfEntireFile
	"pass on to the inputStream"

	^self inputStream contentsOfEntireFile! !

!MessageStream methodsFor: 'stream management' stamp: 'SLB 2/5/2008 22:59'!
convertToTempFile
	"Convert my inputStream a randomly named StandardFileStream containing the contents
	 of the messageStream.
	 Position the file stream at its end so that it is ready to have more written to it."

	| fileName tempFileStream |
	fileName := self directoryPath, FileDirectory pathNameDelimiter asString, 'tempMessageFile', 99999 atRandom printString, '.txt'.
	tempFileStream := (StandardFileStream forceNewFileNamed: fileName).	
	tempFileStream nextPutAll: self inputStream contents.
	tempFileStream close.
	self inputStream: (StandardFileStream oldFileNamed: fileName) setToEnd! !

!MessageStream methodsFor: 'stream management' stamp: 'SLB 8/28/2008 23:51'!
cr
	"Pass this message through to the inputStream.
	This ensures that the parser sees CRs as the EOL.  Therefore, we need to have
	hashed the line with the original EOL before appekding this cr."

	^self inputStream cr! !

!MessageStream methodsFor: 'stream management' stamp: 'SLB 9/7/2007 09:45'!
directory
	"Answer the inputStream's directory (in the case where it is an external temp FileStream"

	^self inputStream directory! !

!MessageStream methodsFor: 'stream management' stamp: 'SLB 9/6/2007 23:27'!
isExternalStream
	"pass on to the inputStream"

	^self inputStream isExternalStream! !

!MessageStream methodsFor: 'stream management' stamp: 'SLB 9/11/2007 22:34'!
maxInMemory
	"<^hOf Integer>"
	"Maximum size beyond which the message is converted to a temp file.
	 This is also the size that triggers attachments to be saved in external files
	 rather than included in the XML stream.  The number is maintained in a
	 class variable of the Message class."

	^RFC2822MIMEMessage MaxBodyPartLength! !

!MessageStream methodsFor: 'stream management' stamp: 'SLB 9/19/2007 20:18'!
name
	"pass on to the inputStream"

	^self inputStream name! !

!MessageStream methodsFor: 'stream management' stamp: 'SLB 9/12/2007 11:12'!
nextLine
	"Pass on to the inputStream.  Note, the inputStream, whether an in-memory
	 ReadWriteStream or a StandardFileStream uses the Cr EOL convention which
	 is assumed by all internal Smalltalk line-oriented code."

	^self inputStream nextLine! !

!MessageStream methodsFor: 'stream management' stamp: 'SLB 12/16/2008 20:17'!
nextPut: aCharacter
	"Pass it through to the inputStream"

	^self inputStream basicNextPut: aCharacter! !

!MessageStream methodsFor: 'stream management' stamp: 'SLB 12/13/2008 14:51'!
nextPutAll: aString
	"Pass it through to the inputStream.
	 NOTE: here is where we discover if the body part is too large and, if so
	 convert the output stream into a temporary FileStream."

	(self inputStream size < self maxInMemory
		or: [self inputStream isExternalStream])
			ifFalse: [self convertToTempFile].
	^self inputStream basicNextPutAll: aString! !

!MessageStream methodsFor: 'stream management' stamp: 'SLB 9/6/2007 23:24'!
peek
	"Answer the inputStream's peek"

	^self inputStream peek! !

!MessageStream methodsFor: 'stream management' stamp: 'SLB 11/23/2007 23:23'!
position
	"Answer the inputStream's position"

	^self inputStream position! !

!MessageStream methodsFor: 'stream management' stamp: 'SLB 9/6/2007 22:21'!
resetToBeginning
	"Ensure inputStream is positioned at its start."

	self inputStream isExternalStream
		ifTrue:
			[self inputStream close.
			 self inputStream openReadOnly]
		ifFalse:
			[self inputStream reset]! !

!MessageStream methodsFor: 'stream management' stamp: 'SLB 9/6/2007 22:00'!
size
	"Answer the inputStream's size"

	^self inputStream size! !

!MessageStream methodsFor: 'stream management' stamp: 'SLB 9/17/2007 20:30'!
upTo: aCharacter
	"Pass it through to the inputStream"

	^self inputStream upTo: aCharacter! !

!MessageStream methodsFor: 'stream management' stamp: 'SLB 8/28/2008 23:53'!
upToAll: aString
	"Pass it through to the inputStream"

	^self inputStream upToAll: aString! !

!MessageStream methodsFor: 'stream management' stamp: 'SLB 9/6/2007 23:26'!
upToEnd
	"pass on to the inputStream"

	^self inputStream upToEnd! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

MessageStream class
	instanceVariableNames: ''!

!MessageStream class methodsFor: 'instance creation' stamp: 'SLB 9/7/2007 09:24'!
on: aReadWriteStream in: directoryPath
	"Answer a new instance of me with the given inputStream and directoryPath installed."

	| me |
	me := self new.
	me inputStream: aReadWriteStream.
	me directoryPath: directoryPath.
	^me! !


EmailHeader subclass: #MsgHeader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EmailParser-Model'!
!MsgHeader commentStamp: 'SLB 8/29/2008 22:15' prior: 0!
Models a RFC2822 message header which may be "folded" onto multiple lines by starting each line but the first with whitespace..

The process of unfolding is accomplished by simply removing any EOL that is immediately followed by Whitespace (space or tab).  Each header field should be treated in its unfolded form for further syntactic and semantic evaluation.

Everything you wanted to know about email headers (but were afraid to ask -- with reason) can be found at.  http://cr.yp.to/immhf/header.html 

An Internet mail message is a sequence of lines, starting with header lines. The headers are terminated by an empty line (or by the end of the message). The header is not terminated by an invisible line, i.e., a line consisting entirely of spaces and tabs. Users occasionally create invisible lines, usually for aesthetic reasons.  An example:

        Received:
                from [censored] by pinerolo.piw.it
                        with smtp
                        
                (Linux Smail3.1.28.1 #5)
                id m0uqMBq-000BzwC; Tue, 13 Aug 96 17:19 GMT+0100

There are three popular strategies for detecting the end of the header:

    * Stop reading the header at the first empty line. Unfortunately, there are a few broken gateways that corrupt messages, inserting a space into every empty line.
    * Stop reading the header at the first invisible line, whether or not it is empty. This handles the broken gateways mentioned above. However, it violates RFC822; for example, it completely misinterprets the valid Received line shown above. Eudora reportedly does this, so I recommend that writers avoid generating invisible lines. RFC822bis prohibits invisible lines.
    * Stop reading the header at the first line that cannot possibly be a header line: (1) an empty line or (2) a line that does not start with a space, a tab, or a field name followed by a colon. I recommend this strategy; it correctly handles 822-compliant messages as well as practically all of the broken messages. !


!MsgHeader methodsFor: 'printing' stamp: 'SLB 9/25/2007 14:52'!
printOn: aStream
	"Basic printing of header."

	super printOn: aStream.
	aStream cr.
	self hasBeenPrinted printOn: aStream! !

!MsgHeader methodsFor: 'printing' stamp: 'SLB 12/13/2008 13:47'!
printXMLOn: aStream enclosingIndentLevel: tabs
	"Print my contents in the mail-account.xsd format at the indicated indent level."

	aStream crtab: tabs; basicNextPutAll: '<Header>'.
	aStream crtab: tabs + 1; basicNextPutAll: '<Name>'.
	self printHeaderNameOn: aStream.
	aStream basicNextPutAll: '</Name>'.
	aStream crtab: tabs + 1; basicNextPutAll: '<Value>'. 
	self printHeaderValueOn: aStream. 
	aStream basicNextPutAll: '</Value>'.
	self comments isEmpty
		ifFalse:
			[aStream crtab: tabs + 1; basicNextPutAll: '<Comments>'.
			 comments do: [ :aComment |
				aStream basicNextPutAll: (self quotedValueOf: aComment)].
	aStream basicNextPutAll: '</Comments>'].
	aStream crtab: tabs; basicNextPutAll: '</Header>'.
	self hasBeenPrinted: true.! !


!MsgHeader methodsFor: 'parsing' stamp: 'SLB 8/29/2008 22:21'!
parseFromCleanedHeaderStream: aReadWriteStream
	"Parse a message header such as the Received: header of the form
	 Received: from gatekeeper2.pbs.com (gatekeeper2.pbs.com [194.191.169.74])
		by mail.uncw.edu (8.11.68.11.6) with SMTP id f7UEVC812079
		for <blough@mail.uncw.edu>; Thu, 30 Aug 2001 10:31:13 -0400 (EDT)
	 Headers have a primary value terminated with a colon, one or more parameters separated
	 by semicolons, and one or more comments in parentheses.  A cleaned header is unfolded,
	 comments removed, and all whitespace removed except within quoted strings.
	 Note: Subject: headers may have parentheses within them so do not extract comments
	 from Subject headers."


	self name: (aReadWriteStream upTo: $:) asLowercase, ':'.
	[aReadWriteStream atEnd not and: [aReadWriteStream peek isWhitespace]]	"skip whitespace"
		whileTrue: [aReadWriteStream next].
	self value: (aReadWriteStream upTo: Character cr).! !

!MsgHeader methodsFor: 'parsing' stamp: 'SLB 2/14/2008 21:53'!
parseFromStream: aReadWriteStream withFirstLine: aString
    "Warning: The stream may contain only the text for one header or may be much larger.
	If it is a Subject: header, don't try to remove comments.
	Note: MsgHeaders don't parse parameters hence don't need whitespace removed."

	self getOrigStringFrom: aReadWriteStream withFirstLine: aString.
	^(aString size > 8 and: [(aString copyFrom: 1 to: 8) asLowercase beginsWith: 'subject:'])
		ifTrue: [self parseFromCleanedHeaderStream: self unfoldedStream]
     	ifFalse: [self parseFromCleanedHeaderStream: (self extractCommentsFrom: self unfoldedStream)]! !


!MsgHeader methodsFor: 'queries' stamp: 'SLB 4/24/2007 13:23'!
isFolded
	"Answer true if the header extends beyond one line."

	| testStream |
	testStream := (ReadStream on: self origString).
	testStream nextLine.
	^testStream atEnd not and:
		[testStream peek isWhitespace]
! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

MsgHeader class
	instanceVariableNames: ''!

!MsgHeader class methodsFor: 'instance creation' stamp: 'SLB 4/23/2007 09:45'!
parseFromStream: aReadWriteStream
	"Answer a new instance of me parsed from the given stream"

	^self new parseFromStream: aReadWriteStream! !


!MsgHeader class methodsFor: 'testing' stamp: 'SLB 4/23/2007 17:07'!
exampleStreamForParsing1
	"Answer a stream containing a simple example header"

	^(ReadWriteStream with: 'X-Account-Key: account2') reset! !

!MsgHeader class methodsFor: 'testing' stamp: 'SLB 8/29/2008 22:08'!
exampleStreamForParsing2
	"Answer a stream containing a complex multi-line example header"

	^(ReadWriteStream with: 'Received: from d01av01.pok.imon.com (d01av01.pok.imon.com [9.56.224.215])
	by d01relay02.pok.imon.com (8.13.6/8.13.6/NCO v8.1.1) with ESMTP id kAEDc9rF265656
	for <jacksprat@mindspring.com>; Tue, 14 Nov 2006 08:38:09 -0500') reset! !

!MsgHeader class methodsFor: 'testing' stamp: 'SLB 8/29/2008 22:07'!
exampleStreamForParsing3
	"Answer a stream containing a complicated multi-line example header"

	^(ReadWriteStream with: 'X-MIMETrack: S/MIME Sign by Notes Client on Joe Blow/London/GE(Release 7.0 HF277|June
 21, 2006) at 11/14/2006 08:37:47 AM,
	Serialize by Notes Client on Joe Blow/London/GE(Release 7.0 HF277|June
 21, 2006) at 11/14/2006 08:37:47 AM,
	Serialize complete at 11/14/2006 08:37:47 AM,
	Itemize by Notes Client on Joe Blow/London/GE(Release 7.0 HF277|June 21, 2006) at
 11/14/2006 08:37:47 AM,
	S/MIME Sign failed at 11/14/2006 08:37:47 AM: Senders'' signing certificate
 is expired,
	S/MIME Sign by Notes Client on Joe Blow/London/GE(Release 7.0 HF277|June
 21, 2006) at 11/14/2006 08:38:03 AM,
	S/MIME Sign complete at 11/14/2006 08:38:03 AM,
	MIME-CD by Notes Server on ABC/01/M/GE(Release 7.0.2 IGS702HF1|November
 11, 2006) at 11/14/2006 08:38:07,
	MIME-CD complete at 11/14/2006 08:38:07,
	Serialize by Router on ABC/01/M/GE(Release 7.0.2 IGS702HF1|November
 11, 2006) at 11/14/2006 08:38:08') reset! !


!MsgHeader class methodsFor: 'defaults' stamp: 'SLB 8/29/2008 22:02'!
defaultMissingMessageIDHeader
	"This is used primarily if a message or Child is missing a message-id header.
	 This should use a true handle-server ID but, lacking a handle server, we use
	 a random integer.  Note that the schema has an element <Supplied> that marks
	 whether the message ID was a legitimate part of the message or was generated
	 by the parser.  "

	| me | 
	me := self new.
	me name: 'message-id'.
	me value: RFC2822MIMEMessage randomID printString, ' Supplied=true'.
	me origString: ''.
	^me! !


MsgHeader subclass: #MsgIdHeader
	instanceVariableNames: 'isGenerated'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EmailParser-Model'!

!MsgIdHeader methodsFor: 'printing' stamp: 'SLB 12/13/2008 13:46'!
printXMLOn: aStream enclosingIndentLevel: tabs
	"Print my contents in the mail-account.xsd format at the indicated indent level.
	 If the ID was generated by the parser, add the 'Supplied' attribute."

	aStream crtab: tabs.
	self isGenerated
		ifTrue: [aStream basicNextPutAll: '<MessageId Supplied="true">']
		ifFalse: [aStream basicNextPutAll: '<MessageId>'].
	self printHeaderValueOn: aStream.
	aStream basicNextPutAll: '</MessageId>'! !


!MsgIdHeader methodsFor: 'accessing' stamp: 'SLB 10/30/2007 14:14'!
isGenerated
	"<^(true | false)>"
	"getter with default false"

	isGenerated isNil	
		ifTrue: [isGenerated := false].
	^isGenerated! !

!MsgIdHeader methodsFor: 'accessing' stamp: 'SLB 10/30/2007 14:15'!
isGenerated: aBoolean
	"<aBooleand: (true | false), ^self>"
	"getter with default false"

	isGenerated := aBoolean! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

MsgIdHeader class
	instanceVariableNames: ''!

!MsgIdHeader class methodsFor: 'instance creation' stamp: 'SLB 11/14/2007 13:05'!
locallyGeneratedMessageID: idString
	"Create a self generated instance with name messageId and the given value"

	| me |
	me := self new.
	me isGenerated: true.
	me name: 'messageId'.
	me value: idString.
	^me! !


Object subclass: #MultiPartBody
	instanceVariableNames: 'multipartBoundary parentMultipartBoundary preamble bodyParts epilogue mimeHeaders'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EmailParser-Model'!
!MultiPartBody commentStamp: 'SLB 8/29/2008 22:58' prior: 0!
MultiPartBodies are packages for multiple BodyParts  Their content-type header must contain a "boundary" parameter that is used to delimit the beginning and end of its parts.  The boundary parameter is a string that does not appear in any of the bodies of the message.  Each occurrence of the boundary string begins with '--' followed by the unique string.  The last occurrence, that
signals the end of the sequence of BodyParts is supposed to also be follwed by '--'.  However, some email clients are lax about adding the '--' suffix.  And MultiPartBodies can be nested arbitrarily deeply.  ANY outer boundary string terminates ALL unterminated inner BodyParts.  At present (SLB 10/10/2007 17:23) this termination of inner bodies by outer boundaries works only for one level of nesting.  While deeper nesting is very rare (I haven't encountered any cases in parsing some 100Mb of email  Fixing that takes some refactoring that there isn't time for right now..  

A MultiPartBody MUST have at least one BodyPart.

The "mixed" subtype of "multipart" is intended for use when the body parts are independent and need to be bundled in a particular order. Any "multipart" subtypes that an implementation does not recognize must be treated as being of subtype "mixed".
5.1.4. Alternative Subtype

The "multipart/alternative" type is syntactically identical to "multipart/mixed", but the semantics are different. Each of the body parts is treated as an "alternative" version of the same information. 

A "digest" subtype of the "multipart" Content-Type is syntactically identical to "multipart/mixed", but the semantics are different. In a digest, the default Content-Type value for a body part is changed from "text/plain" to "message/rfc822". This is done to allow a more readable digest format that is largely compatible (except for the quoting convention) with RFC 934. Though it is possible to specify a Content-Type value for a body part in a digest which is other than "message/rfc822", such as a "text/plain" part containing a description of the material in the digest, actually doing so is undesireble. The "multipart/digest" Content-Type is intended to be used to send collections of messages. If a "text/plain" part is needed, it should be included as a seperate part of a "multipart/mixed" message!


!MultiPartBody methodsFor: 'parsing' stamp: 'SLB 6/27/2007 17:33'!
parseBodyPartsFrom: aStream
	"Parse one or more bodyParts, each beginning with --myBoundaryString.
	 The last body part should (according to RFC2822) end with --myBoundaryString-- .
	 However, some email clients take liberties with the spec and end the last body part
	 with the parent's ending string (i.e., parent boundary with '--' appended.
	 Parsing gets here after the beginning --myBoundaryString of each body part has been found."

	| nextLine nextBodyPart endingBoundaryString parentEndingBoundaryString |
	nextLine := ''.
	endingBoundaryString := self multipartBoundary, '--'.
	self parentMultipartBoundary notNil
		ifTrue: [parentEndingBoundaryString := self parentMultipartBoundary, '--']
		ifFalse: [parentEndingBoundaryString := ''].  "which is not included in any nonempty string"
	[(nextLine includesSubString: endingBoundaryString)
		or: [nextLine includesSubString:  parentEndingBoundaryString]]
		whileFalse:
			[self bodyParts add: (nextBodyPart := BodyPartType new).
			 nextLine := nextBodyPart parseFromStream: aStream
						 			  multipartBoundary: self multipartBoundary].
	^nextLine! !

!MultiPartBody methodsFor: 'parsing' stamp: 'SLB 8/29/2008 23:11'!
parseEpilogueFrom: aStream
	"The epilogue includes any lines appearing after the terminating boundary line.
	 But, must determine whether the terminating boundary line is signal for next bodyPart
	 or end of the multipart message and return the correct boundary line."

	| restOfLine |
	self isRecursivelyEmbedded
		ifTrue:
			[self epilogue: (aStream upToAll: '--', self parentMultipartBoundary).
			 restOfLine := aStream upTo: Character cr.  "eat '--' or final newline if present."
			 ^(restOfLine beginsWith: '--')
				ifTrue: ['--', self parentMultipartBoundary, '--']
				ifFalse: ['--', self parentMultipartBoundary]]
		ifFalse: [self epilogue: aStream upToEnd].! !

!MultiPartBody methodsFor: 'parsing' stamp: 'SLB 6/27/2007 20:13'!
parseFromStream: aStream
	"Parse myself from aStream."

	| nextLine |
	self parsePreambleFrom: aStream.
	nextLine := self parseBodyPartsFrom: aStream.
	aStream atEnd
		ifFalse: [nextLine := self parseEpilogueFrom: aStream].
	^nextLine
	! !

!MultiPartBody methodsFor: 'parsing' stamp: 'SLB 6/4/2007 20:44'!
parsePreambleFrom: aStream
	"The preamble includes any lines appearing before the first boundary line."

	| nextLine |
	[(nextLine := aStream nextLine) beginsWith: '--', self multipartBoundary]
		whileFalse: [self preamble: self preamble, nextLine, (String with: Character cr)].! !


!MultiPartBody methodsFor: 'queries' stamp: 'SLB 10/15/2007 20:45'!
epilogueNotEmpty
	"<^ (true | false) >"
	"Answer true iff the epilogue contains non-whitepace"

	self epilogue isNil
		ifTrue: [^false].
	self epilogue do: [:aChar |
		aChar isSeparator not
			ifTrue: [^true]].
	^false! !

!MultiPartBody methodsFor: 'queries' stamp: 'SLB 5/23/2007 22:16'!
isRecursivelyEmbedded
	"Answer true if so."

	^self parentMultipartBoundary notNil! !

!MultiPartBody methodsFor: 'queries' stamp: 'SLB 10/15/2007 20:45'!
preambleNotEmpty
	"<^ (true | false) >"
	"Answer true iff the preamble contains non-whitepace"

	self preamble isNil
		ifTrue: [^false].
	self preamble do: [:aChar |
		aChar isSeparator not
			ifTrue: [^true]].
	^false! !


!MultiPartBody methodsFor: 'accessing' stamp: 'SLB 5/6/2007 08:32'!
bodyParts
	"<^ hOf: OrderedCollection {:of BodyPart}>"
	"Getter with laissez faire initialization."

	bodyParts isNil
		ifTrue: [bodyParts := OrderedCollection new].
	^bodyParts! !

!MultiPartBody methodsFor: 'accessing' stamp: 'SLB 5/6/2007 08:19'!
epilogue
	"<^ iOf String>"
	"Getter"

	epilogue isNil
		ifTrue: [epilogue := ''].
	^epilogue! !

!MultiPartBody methodsFor: 'accessing' stamp: 'SLB 5/6/2007 08:16'!
epilogue: aString
	"Setter"

	epilogue := aString! !

!MultiPartBody methodsFor: 'accessing' stamp: 'SLB 10/19/2007 16:43'!
mimeHeaders
	"<^ (hOf OrderedCollection {of: hOf MIMEHeader})>"
	"laissez faire getter"

	mimeHeaders isNil
		ifTrue: [mimeHeaders := OrderedCollection new].
	^mimeHeaders! !

!MultiPartBody methodsFor: 'accessing' stamp: 'SLB 10/19/2007 11:51'!
mimeHeaders: aCollection
	"<aCollection: (hOf OrderedCollection {of: hOf MsgHeader} | nil)>"
	"setter"

	mimeHeaders := aCollection! !

!MultiPartBody methodsFor: 'accessing' stamp: 'SLB 5/5/2007 22:21'!
multipartBoundary
	"<^ iOf String>"
	"Getter"

	^multipartBoundary! !

!MultiPartBody methodsFor: 'accessing' stamp: 'SLB 5/5/2007 22:21'!
multipartBoundary: aString
	"Setter"

	multipartBoundary := aString! !

!MultiPartBody methodsFor: 'accessing' stamp: 'SLB 5/23/2007 22:09'!
parentMultipartBoundary
	"<^ iOf String>"
	"Getter"

	^parentMultipartBoundary! !

!MultiPartBody methodsFor: 'accessing' stamp: 'SLB 5/23/2007 22:09'!
parentMultipartBoundary: aString
	"Setter"

	parentMultipartBoundary := aString! !

!MultiPartBody methodsFor: 'accessing' stamp: 'SLB 5/6/2007 08:18'!
preamble
	"<^ iOf String>"
	"Getter"

	preamble isNil
		ifTrue: [preamble := ''].
	^preamble! !

!MultiPartBody methodsFor: 'accessing' stamp: 'SLB 6/4/2007 20:57'!
preamble: aString
	"Setter"

	preamble := aString.! !

!MultiPartBody methodsFor: 'accessing' stamp: 'SLB 11/15/2007 15:46'!
resetHeaders
	"Primarily for debugging.  Reset the 'hasBeenPrinted value of all headers
	 so next attempt to print them will work."

	self bodyParts do: [:aBodyPart | aBodyPart resetHeaders]! !


!MultiPartBody methodsFor: 'printing' stamp: 'SLB 5/10/2007 19:19'!
printOn: aStream
	"Print myself"

	aStream nextPutAll: 'MultiPartBody, boundary string: '.
	multipartBoundary printOn: aStream.
	self preamble notNil
		ifTrue: [aStream cr; nextPutAll: 'preamble -- '.
				preamble printOn: aStream].
	self bodyParts isEmpty
		ifFalse: [self bodyParts do: [ :aBodyPart |
					aStream cr.
					aBodyPart printOn: aStream]].
	self epilogue notNil
		ifTrue: [aStream cr; nextPutAll: 'epilogue -- '.
				epilogue printOn: aStream].
	aStream cr! !

!MultiPartBody methodsFor: 'printing' stamp: 'SLB 11/15/2007 19:29'!
printRecursiveMultiBodyXMLOn: aStream enclosingIndentLevel: tabs
	"Print my contents in the mail-account.xsd format at the indicated indent level.  This is used
	 in the context of a recursive multipart body, i.e., the calling routing in BodyPart has
	 already printed the <MultiBody> tag and the <ContentType> info."

	self preambleNotEmpty
		ifTrue: [self printXMLPreambleOn: aStream enclosingIndentLevel: tabs + 1].
	self bodyParts do: [ :aBodyPart |
		aBodyPart printXMLOn: aStream enclosingIndentLevel: tabs + 2].
	self epilogueNotEmpty
		ifTrue: [self printXMLEpilogueOn: aStream enclosingIndentLevel: tabs + 1]! !

!MultiPartBody methodsFor: 'printing' stamp: 'SLB 12/15/2008 19:12'!
printXMLEpilogueOn: aStream enclosingIndentLevel: tabs
	"Print my preamble in the mail-account.xsd format at the indicated indent level."

	aStream crtab: tabs; basicNextPutAll: '<Epilogue> '.
	((self epilogue indexOf: $<) > 0 or: [(self epilogue indexOf: $&) > 0])
		ifTrue: [aStream basicNextPutAll: '<!![CDATA['; basicNextPutAll: self preamble withoutTrailingBlanks;
				basicNextPutAll: ']]>']
		ifFalse: [aStream basicNextPutAll: self preamble withoutTrailingBlanks].
	aStream basicNextPutAll: ' </Epilogue>'! !

!MultiPartBody methodsFor: 'printing' stamp: 'SLB 12/13/2008 13:46'!
printXMLOn: aStream enclosingIndentLevel: tabs
	"Print my contents in the mail-account.xsd format at the indicated indent level
	 in the order specified by the schema."

	aStream crtab: tabs; basicNextPutAll: '<MultiBody>'.
	#('content-type:' 'content-transfer-encoding:' 'content-id:' 'content-description:' 'content-disposition:') do: [ :headerType | 
		mimeHeaders at: headerType 
			ifPresent: [ :aHeader | aHeader printXMLOn: aStream enclosingIndentLevel: tabs + 1.
				 aHeader hasBeenPrinted: true]].
	self mimeHeaders valuesDo: [ :aHeader |		  "print remaining unprinted MIME headers"
		aHeader hasBeenPrinted
			ifFalse:
				[aHeader printXMLOn: aStream enclosingIndentLevel: tabs + 1.
				 aHeader hasBeenPrinted: true]].
	self preambleNotEmpty
		ifTrue: [self printXMLPreambleOn: aStream enclosingIndentLevel: tabs + 1].
	self bodyParts do: [ :aBodyPart |
		aBodyPart printXMLOn: aStream enclosingIndentLevel: tabs + 2].
	self epilogueNotEmpty
		ifTrue: [self printXMLEpilogueOn: aStream enclosingIndentLevel: tabs + 1].
	aStream crtab: tabs; basicNextPutAll: '</MultiBody>'.! !

!MultiPartBody methodsFor: 'printing' stamp: 'SLB 11/21/2007 16:11'!
printXMLPreambleOn: aStream enclosingIndentLevel: tabs
	"Print my preamble in the mail-account.xsd format at the indicated indent level."

	aStream crtab: tabs; nextPutAll: '<Preamble>'.
	((self preamble indexOf: $<) > 0 or: [(self preamble indexOf: $&) > 0])
		ifTrue: [aStream nextPutAll: '<!![CDATA['; nextPutAll: self preamble withoutTrailingBlanks;
				nextPutAll: ']]>']
		ifFalse: [aStream nextPutAll: self preamble withoutTrailingBlanks].
	aStream nextPutAll: '</Preamble>'! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

MultiPartBody class
	instanceVariableNames: ''!

!MultiPartBody class methodsFor: 'instance creation' stamp: 'SLB 5/23/2007 21:11'!
multipartBoundary: aString
	"Answer a new instance with the given multipartBoundary string"

	| me |
	me := self new.
	me multipartBoundary: aString.
	^me! !


MIMEHeader subclass: #OtherMIMEHeader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EmailParser-Model'!
!OtherMIMEHeader commentStamp: 'SLB 8/30/2008 10:15' prior: 0!
For modeling MIME headers not specifically modeled by their own class.
An exhaustive list of registered headers, some of which are obscure MIME headers,
can be found at:  http://www.ninebynine.org/IETF/Messaging/draft-klyne-hdrreg-mail-00.html

Examples of obscure MIME headers include: Content-Class (currently a Microsoft-only header in which they store their 'Meta-tag data for content tagging), Content-Features, Content-Language, Content-Alternative, Content-Base, Content-Location, etc.  Some are 'Provisional', e.g., Content-Class.  Other provisional types include:  Content-Alias, Content-Length, Content-Conversion, !


!OtherMIMEHeader methodsFor: 'printing' stamp: 'SLB 12/13/2008 13:45'!
printXMLOn: aStream enclosingIndentLevel: tabs
	"Print my contents in the mail-account.xsd header format at the indicated indent level.
<Name> and <Value>  Shouldn't we have params and comments for this???"

	aStream crtab: tabs; basicNextPutAll: '<OtherMimeHeader>'.
	aStream crtab: tabs + 1; basicNextPutAll: '<Name>';
			nextPutAll: self name;
		     basicNextPutAll: '</Name>'.
	aStream crtab: tabs + 1; basicNextPutAll: '<Value>'. 
	self printHeaderValueOn: aStream. 
	aStream basicNextPutAll: '</Value>'.
	aStream crtab: tabs; basicNextPutAll: '</OtherMimeHeader>'! !


Object subclass: #ParseError
	instanceVariableNames: 'errorType errorLocation'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EmailParser-Model'!
!ParseError commentStamp: 'SLB 11/23/2007 11:07' prior: 0!
Information that is put out in the 'Incomplete' tag in the XML schema about an inability to fully parse an email message.  Note, the inability may be due to erroneous email input or to a flaw in the parser.

Structure:
 errorType		iOf String -- some descriptive information about the error, including a sample of the input
 errorLocation	iOf String -- some indication of where the error occurred
!


!ParseError methodsFor: 'printing' stamp: 'SLB 3/9/2008 19:10'!
printCommaDelimitedOn: aStream
	"Print first 100 chars of my contents in the comma delimited spreadsheet form.
	 Remove double quotes and or commas first since they screw up the spreadsheet."

	| outputString |
	outputString := self errorLocation, ' ', self errorType.
	outputString size > 100
		ifTrue: [outputString := outputString copyFrom: 1 to: 100].
	((outputString indexOf: $") > 0 or: [(outputString indexOf: $,) > 0])
		ifTrue: [aStream nextPutAll: (outputString
					reject: [ :char | char = $" or: [char = $,]])]
		ifFalse: [aStream nextPutAll: outputString].! !

!ParseError methodsFor: 'printing' stamp: 'SLB 3/2/2008 20:51'!
printOn: aStream
	"put readable rendering of me on aStream"

	aStream nextPutAll: 'ParseError - errorType: ', errorType printString,
						',  errorLocation: ', errorLocation printString.! !

!ParseError methodsFor: 'printing' stamp: 'SLB 12/13/2008 13:45'!
printXMLOn: aStream enclosingIndentLevel: tabs
	"Print my contents in the mail-account.xsd format at the indicated indent level
	 in the order specified by the multi-part-body schema."

	aStream crtab: tabs; basicNextPutAll: '<Incomplete>'.
	aStream crtab: tabs + 1; basicNextPutAll: '<ErrorType>'.
	((self errorType indexOf: $<) > 0 or: [(self errorType indexOf: $&) > 0])
		ifTrue: [aStream basicNextPutAll: '<!![CDATA[', self errorType, ']]>']
		ifFalse: [aStream basicNextPutAll: self errorType].
	aStream basicNextPutAll: '</ErrorType>'.
	self errorLocation isNil
		ifFalse:
			[aStream crtab: tabs + 1; basicNextPutAll: '<ErrorLocation>', self errorLocation, '</ErrorLocation>'].
	aStream crtab: tabs; basicNextPutAll: '</Incomplete>'.! !


!ParseError methodsFor: 'accessing' stamp: 'SLB 11/23/2007 11:08'!
errorLocation
	"A string giving information about where the parse error occurred."

	^errorLocation! !

!ParseError methodsFor: 'accessing' stamp: 'SLB 11/23/2007 11:08'!
errorLocation: aString
	"A string giving information about where the parse error occurred."

	errorLocation := aString! !

!ParseError methodsFor: 'accessing' stamp: 'SLB 11/23/2007 11:03'!
errorType
	"A string giving information about the parse error."

	^errorType! !

!ParseError methodsFor: 'accessing' stamp: 'SLB 11/23/2007 11:07'!
errorType: aString
	"A string giving information about the parse error."

	errorType := aString! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

ParseError class
	instanceVariableNames: ''!

!ParseError class methodsFor: 'instance creation' stamp: 'SLB 2/5/2008 22:26'!
errorLocation: locString errorType: typeString
	"Answer a new one of me with the given data"

	| me |
	me := self new.
	me errorLocation: locString.
	me errorType: typeString.
	^me! !


Object subclass: #RFC2822MIMEMessage
	instanceVariableNames: 'relativePath localID messageID hash messageHeaders mimeHeaders body w3cDate parseErrors folder isPrinting'
	classVariableNames: 'CurrentMessage MaxBodyPartLength'
	poolDictionaries: ''
	category: 'EmailParser-Model'!
!RFC2822MIMEMessage commentStamp: 'SLB 12/19/2008 17:42' prior: 0!
Models an email message in a form that best fits with the Mail-Account.xsd schema
"message-type" element.

A message may contain a large attachment which is to be parsed into a SingleBody.
Since Smalltalk cannot deal with arbitrarily large streams in memory (e.g., 100 megabyte video
attachments), there is a provision in BodyPartType for detecting such attachments and
storing them into external files.  The RFC2822MIMEMessage has a class variable, MaxBodyPartLength,
that determines where the file size cutoff is.  It defaults to 25K.

messageHeaders - an OrderedCollection of all the messageHeaders (in the order
		they appear in the message).
mimeHeaders -- a Dictionary of mimeHeaders keyed by header name, with the
		appropriate subclass of MIMEHeader as the value.
relativePath (anyURI, min 1) -- URI component that when added to the BasePath
		for the Account gives the URI where the mbox that contains the original
		email message is located.
Local-ID (Long) -- will eventually become a handle.
Message-ID (MsgIdHeader), exactly 1)
hash (aHash) holds the SHA1 hash information from hashing the original message
		string. a SingleBody, else a MultiPartBody that may recursively hold other
		multi-part or single bodies.
w3cDate (hOf W3CDateTime) -- the date/time in W3C format when the email was sent.
		Note, this is when the sender pushed the send-button, not when the email
		server actually transmitted the message.
parseErrors -- an OrderedCollection of ParseError objects
folder -- the Folder object that contains this message
isPrinting -- boolean, default false, set to true when the XML output for the message
		is begun.  Its purpose is to distinguish a fatal error during parsing from
		one during printing.  This is a work in progress. At this point a fatal printing
		error just causes a halt since it isn't clear how to proceed.  For the XML
		output to be valid a whole bunch of closing tags that follow all the schema
		rules would have to be generated.  No fatal errors during printing have
		occurred, so leave it as is for now.

!


!RFC2822MIMEMessage methodsFor: 'accessing' stamp: 'SLB 8/30/2008 17:20'!
body
	"<^ (hOf SingleBody | hOf MultiPartBody)."

	^body! !

!RFC2822MIMEMessage methodsFor: 'accessing' stamp: 'SLB 8/30/2008 17:19'!
body: singleOrMultipartBody
	"setter"

	body := singleOrMultipartBody! !

!RFC2822MIMEMessage methodsFor: 'accessing' stamp: 'SLB 11/16/2007 17:55'!
contentType
	"<^hOf ContentTypeType>"
	"Answer the ContentTypeHeader if any, else nil"

	^self mimeHeaders at: 'content-type:' ifAbsent: [nil]! !

!RFC2822MIMEMessage methodsFor: 'accessing' stamp: 'SLB 3/1/2008 20:40'!
folder
	"getter"

	^folder! !

!RFC2822MIMEMessage methodsFor: 'accessing' stamp: 'SLB 3/1/2008 20:48'!
folder: myFolder
	"setter"
	"<myFolder hOf Folder>"

	folder := myFolder! !

!RFC2822MIMEMessage methodsFor: 'accessing' stamp: 'SLB 3/5/2008 12:22'!
isPrinting
	"getter, default false."

	isPrinting isNil
		ifTrue: [isPrinting := false].
	^isPrinting! !

!RFC2822MIMEMessage methodsFor: 'accessing' stamp: 'SLB 3/5/2008 12:22'!
isPrinting: aBoolean
	"setter"

	isPrinting := aBoolean! !

!RFC2822MIMEMessage methodsFor: 'accessing' stamp: 'SLB 5/7/2007 14:30'!
localID
	"<^ hOf Integer>"
	^localID! !

!RFC2822MIMEMessage methodsFor: 'accessing' stamp: 'SLB 9/16/2007 15:08'!
localID: anInteger
	"setter"

	localID := anInteger! !

!RFC2822MIMEMessage methodsFor: 'accessing' stamp: 'SLB 10/20/2007 16:22'!
messageHeaders
	"<^ hOf OrderedCollection of: {hOf MsgHeader}>"
	"Laissez faire getter"

	messageHeaders isNil
		ifTrue: [messageHeaders := OrderedCollection new].
	^messageHeaders! !

!RFC2822MIMEMessage methodsFor: 'accessing' stamp: 'SLB 1/12/2008 14:57'!
messageID
	"<^ iOf MsgIdHeader>"
	"getter"

	^messageID! !

!RFC2822MIMEMessage methodsFor: 'accessing' stamp: 'SLB 10/30/2007 15:12'!
messageID: aMsgIdHeader
	"setter"

	messageID := aMsgIdHeader! !

!RFC2822MIMEMessage methodsFor: 'accessing' stamp: 'SLB 11/16/2007 19:35'!
mimeHeaders
	"<^ hOf Dictionary {name: iOf String, value: hOf MIMIHeader}>"
	"Laissez faire getter"

	mimeHeaders isNil
		ifTrue: [mimeHeaders := Dictionary new].
	^mimeHeaders! !

!RFC2822MIMEMessage methodsFor: 'accessing' stamp: 'SLB 1/24/2008 21:43'!
myHash
	"<^hOf Hash>"
	"Avoid naming this method 'hash' because that is an important system message."

	^hash! !

!RFC2822MIMEMessage methodsFor: 'accessing' stamp: 'SLB 1/24/2008 21:44'!
myHash: aHash
	"setter, can't name it just 'hash' because that's an important system message."

	hash := aHash! !

!RFC2822MIMEMessage methodsFor: 'accessing' stamp: 'SLB 2/7/2008 22:01'!
parseErrors
	"A collection of parse error objects."

	parseErrors isNil
		ifTrue: [parseErrors := OrderedCollection new].
	^parseErrors! !

!RFC2822MIMEMessage methodsFor: 'accessing' stamp: 'SLB 8/15/2007 15:17'!
relativePath
	"<^ iOf String>"
	"Getter -- returns a string that is the legal URI of the parsed XML output file for this message.
	 relative to the input file.  As this parser currently works (SLB 8/15/2007 15:16) this should be
	 simply '.' since the XML file is placed in the same directory in which its input is found."

	^relativePath! !

!RFC2822MIMEMessage methodsFor: 'accessing' stamp: 'SLB 8/27/2007 13:47'!
relativePath: aString
	"Setter -- aString is the legal URI of the folder path for the mbox file of this message
	 relative to the directory that contains the Account.    Hence top level folders will have
	 a path of './account directory/toplevelfoldername',  The next level will have
	 './account directory/toplevelfoldername/secondlevelfoldername' and so forth."

	relativePath := aString! !

!RFC2822MIMEMessage methodsFor: 'accessing' stamp: 'SLB 2/19/2008 19:16'!
w3cDate
	"Laissez faire getter"
	"<^ hOf W3CDateTime>"

	w3cDate isNil
		ifTrue: [w3cDate := W3CDateTime new].
	^w3cDate! !

!RFC2822MIMEMessage methodsFor: 'accessing' stamp: 'SLB 2/19/2008 19:19'!
w3cDate: aW3CDateTime
	"setter"

	w3cDate := aW3CDateTime! !


!RFC2822MIMEMessage methodsFor: 'printing' stamp: 'SLB 12/15/2008 19:13'!
printCommaDelimitedSummaryWithoutErrorsOn: aStream
	"Print my key header data in comma-delimited-valoue (csv) form on aStream."

	| headerDictionary |
	aStream cr.		"start new line"
	headerDictionary := Dictionary new.
	self messageHeaders do: [ :aHeader |
		headerDictionary at: aHeader name put: aHeader value].
	self printSummaryField: (headerDictionary at: 'from:' ifAbsent: ['Error, missing From: header'])
						 on: aStream.
	#('to:' 'date:' 'subject:') do: [ :aFieldName |
		aStream nextPut: $, .
		self printSummaryField: (headerDictionary at: aFieldName
										ifAbsent: ['--Missing Header--']) on: aStream].
	aStream nextPut: $, .
	self printSummaryField: self messageID value on: aStream.
	aStream basicNextPutAll: ',', self myHash asHex.! !

!RFC2822MIMEMessage methodsFor: 'printing' stamp: 'SLB 12/13/2008 13:39'!
printSummaryField: aString on: aStream
	"Print one field on aStream with proper escapes for commas and double quotes to satisfy
	 CSV (comma separated value) format."

	| safeString |
	aString size = 0
		ifTrue: [safeString := '--Empty Header--']
		ifFalse: [safeString := aString copy].
	((aString indexOf: $") > 0 or: [(safeString indexOf: $,) > 0])
		ifTrue:
			[safeString := self regularizeDoubleQuotesIn: safeString].
	aStream basicNextPutAll: safeString.! !

!RFC2822MIMEMessage methodsFor: 'printing' stamp: 'SLB 11/9/2007 20:50'!
printXMLForHeadersOn: aStream enclosingIndentLevel: tabs
	"Print my headers in the mail-account.xsd format at the indicated indent level.  Some use
	 individualized tags.  The rest just use the general <Header> tag.
	Note: For now (SLB 11/9/2007 20:47) only one header, date:, is printed both in a special and in a
	 generic form.  David Minor and I have agreed that special headers where the value is the same
	 do not need to be printed twice.  The Date: header is not the same.  It's a W3C DateTime in the
	 special version and a RFC822 DateTime in the generic version."

	| tags fields |
	tags := #('MimeVersion>' 'OrigDate>' 'From>' 'Sender>' 'To>' 'Cc>' 'Bcc>' 'InReplyTo>'
			'References>' 'Subject>' 'Comments>' 'Keywords>').
	fields := #('mime-version:' 'date:' 'from:' 'sender:' 'to:' 'cc:' 'bcc:' 'reply-to:' 
			'references:' 'subject:' 'comments:' 'keywords:').
	1 to: tags size do: [ :i |
		self messageHeaders do: [ :aHeader |
			(aHeader hasBeenPrinted not and: [aHeader name asLowercase = (fields at: i)])
				ifTrue:
					[aHeader hasBeenPrinted: true. "So special headers (except date:) won't be printed twice."
					 self printXMLForSpecialHeader: aHeader on: aStream withTag: (tags at: i)
							  enclosingIndentLevel: tabs]]].
	self messageHeaders do: [ :aHeader |		  "print remaining unprinted message headers"
		aHeader hasBeenPrinted
			ifFalse:
				[aHeader printXMLOn: aStream enclosingIndentLevel: tabs.
				 aHeader hasBeenPrinted: true]].

! !

!RFC2822MIMEMessage methodsFor: 'printing' stamp: 'SLB 12/16/2008 20:34'!
printXMLForSpecialHeader: aHeader on: aStream withTag: tagString enclosingIndentLevel: tabs
	"Print the 'special' header in the mail-account.xsd format at the indicated indent level."

	(tagString = 'OrigDate>' and: [w3cDate notNil])
		ifTrue: [^self].		 "to avoid printing two OrigDate headers"

	aStream crtab: tabs; nextPut: $<; nextPutAll: tagString.
	tagString = 'OrigDate>'
		ifTrue:  "Convert it to a W3C DateTime string before printing"
			[self w3cDate: W3CDateTime new.
			[self w3cDate parseFromRFC822String: aHeader value]
				on: EmailMalformedDateException
				do: [ :ex | self recordParseErrorLocation: 'printing W3CDate'
								type: ex description.
					self w3cDate: (self w3cDate guessFromRFC2822String: aHeader value)].
			self w3cDate printW3CDateTimeFormatOn: aStream.
			aHeader hasBeenPrinted: false "force the generic Date header to also be printed" ]
		ifFalse:
			[aHeader printHeaderValueOn: aStream].
	aStream basicNextPutAll: '</'; basicNextPutAll: tagString



! !

!RFC2822MIMEMessage methodsFor: 'printing' stamp: 'SLB 12/13/2008 13:44'!
printXMLOn: aStream enclosingIndentLevel: tabs
	"Print my contents in the mail-account.xsd format at the indicated indent level."

	self isPrinting: true.
	aStream crtab: tabs; basicNextPutAll: '<Message>'.
	aStream crtab: tabs + 1; basicNextPutAll: '<RelPath>'.
		aStream basicNextPutAll: self relativePath.
		aStream basicNextPutAll: '</RelPath>'.
	aStream crtab: tabs + 1; basicNextPutAll: '<LocalId>'.
		self localID printOn: aStream.
		aStream basicNextPutAll: '</LocalId>'.
	self messageID printXMLOn: aStream enclosingIndentLevel: tabs + 1.
	self printXMLForHeadersOn: aStream enclosingIndentLevel: tabs + 1.
	self body printXMLOn: aStream enclosingIndentLevel: tabs + 1.
	self hasNoErrors
		ifFalse:
			[self parseErrors do: [ :aParseError |
			 	aParseError printXMLOn: aStream enclosingIndentLevel: tabs + 1]].
	self myHash printXMLOn: aStream enclosingIndentLevel: tabs + 1.
	aStream crtab: tabs; basicNextPutAll: '</Message>'! !

!RFC2822MIMEMessage methodsFor: 'printing' stamp: 'SLB 1/28/2008 20:02'!
regularizeDoubleQuotesIn: aString
	"<aString iOf: String, ^iOf String>"
	"aString contains at least one double-quote or comma.  Wrap it in double quotes and
	 double any occurance of double-quotes."

	| tempInStream tempOutStream char |
	tempOutStream := WriteStream with: String new.
	tempOutStream nextPut: $".		"add opening doub-quote"
	tempInStream := ReadStream on: aString.
	[tempInStream atEnd]
		whileFalse:	"double any doub quotes."
			[char := tempInStream next.
			 char = $"
			 	ifTrue: [tempOutStream nextPut: $"].
			 tempOutStream nextPut: char].
	tempOutStream nextPut: $".		"add closing doub-quote"
	^tempOutStream contents! !

!RFC2822MIMEMessage methodsFor: 'printing' stamp: 'SLB 11/16/2007 17:58'!
resetHeaders
	"Primarily for debugging.  Reset the 'hasBeenPrinted value of all headers
	 so next attempt to print them will work."

	self messageHeaders do: [ :aHeader |
		aHeader hasBeenPrinted: false].
	self mimeHeaders valuesDo: [ :aHeader |
		aHeader hasBeenPrinted: false].
	self body resetHeaders! !


!RFC2822MIMEMessage methodsFor: 'queries' stamp: 'SLB 11/16/2007 17:52'!
contentTransferEncoding
	"return the Content-Transfer-Encoding header if present, Else nil"

	^self mimeHeaders at: 'content-transfer-encoding:'
			ifAbsent: [nil]! !

!RFC2822MIMEMessage methodsFor: 'queries' stamp: 'SLB 1/10/2008 14:07'!
fromHeader
	"return the From: header if present, Else nil"

	^self messageHeaders at: 'from:'
			ifAbsent: [nil]! !


!RFC2822MIMEMessage methodsFor: 'parsing' stamp: 'SLB 11/14/2007 13:06'!
createLocalMessageID
	"Fill in Message-ID with a local ID based upon my localID."
"Note, the Supplied=""true"" is supposed to be part of the MessageId tag...like so:
<MessageId  Supplied=""true"">727710803378239</MessageId>"

	self messageID: (MsgIdHeader locallyGeneratedMessageID: self localID printString)! !

!RFC2822MIMEMessage methodsFor: 'parsing' stamp: 'SLB 12/8/2008 10:07'!
getBodyFrom: aStream
	"Parse the body which is the remainder of the email."

	self contentType isNil
		ifTrue: [self setDefaultHeadersIfNeeded].
	self contentType isSimple
		ifTrue:
			[self body: (SingleBody mimeHeaders: self mimeHeaders).
			 self body parseFromStream: aStream]
		ifFalse: [self contentType isMultipart
			ifTrue:
				[self body: (MultiPartBody multipartBoundary: self contentType multiPartBoundary).
				 self body mimeHeaders: self mimeHeaders.
				 self body parseFromStream: aStream]
			ifFalse: [EmailUnrecognizedTypeException signal: 'Message Unhandled Content-Type: ', self contentType value]]  "Well, what kind of body is it?"! !

!RFC2822MIMEMessage methodsFor: 'parsing' stamp: 'SLB 11/23/2007 16:28'!
getHeadersFrom: messageStream
	"Parse all the headers.  The end of the headers is marked by an empty line.  Note that a legal message must have at least one header so we can grab the first one without checking for empty line since the mbox imput has been exported from an email system and hence is legal enough to have survived prior parsing by email servers and clients."

	| nextLine nextHeader |
	[messageStream atEnd or: [(nextLine := messageStream nextLine) size = 0]]
		whileFalse:
			[nextHeader := (EmailHeader headerSubclassFor: nextLine)
								parseFromStream: messageStream withFirstLine: nextLine.
			 (nextHeader name asLowercase = 'message-id:')
				ifTrue: [self messageID: nextHeader]
				ifFalse: [(nextHeader isMIME)
					ifTrue: [self mimeHeaders at: nextHeader name put: nextHeader]
					ifFalse: [self messageHeaders add: nextHeader]]]! !

!RFC2822MIMEMessage methodsFor: 'parsing' stamp: 'SLB 10/30/2007 15:33'!
parseFromStream: aReadWriteStream
	"The stream must contain the text for only one message.  If the original email was
	 in an mbox format, the sender of this message is expected to have undone any '>From '
	 quoting (see www.qmail.org/man/man5/mbox.html).  If the message has no Message0ID,
	 create one."

	self setLocalIDs.
	self getHeadersFrom: aReadWriteStream.
	self messageID isNil
		ifTrue: [self createLocalMessageID].
	self getBodyFrom: aReadWriteStream.! !

!RFC2822MIMEMessage methodsFor: 'parsing' stamp: 'SLB 11/16/2007 17:59'!
setDefaultHeadersIfNeeded
	"This is a Single/Simple body email.  Set Content-type/charset and Content-Transfer-Encoding
	 headers if absent."

	self contentType isNil
		ifTrue:	   "set to text/plain, us-ascii"
			[self mimeHeaders at: 'content-type:' put: ContentTypeHeader default]. 
	self contentTransferEncoding isNil
		ifTrue:
			[self mimeHeaders at: 'content-transfer-encoding:' put: ContentTransferEncodingHeader default].

! !

!RFC2822MIMEMessage methodsFor: 'parsing' stamp: 'SLB 6/26/2007 15:53'!
setLocalIDs
	"Fill in local-ID and folderID with random numbers.  This is for prototype only.
	 Something like a handle server should provide these IDs."

	self localID: RFC2822MIMEMessage randomID.! !


!RFC2822MIMEMessage methodsFor: 'error reporting' stamp: 'SLB 2/27/2008 21:34'!
addParseError: aParseError
	"Add aParseError to my collection of parse error objects."

	self parseErrors add: aParseError! !

!RFC2822MIMEMessage methodsFor: 'error reporting' stamp: 'SLB 3/5/2008 20:19'!
hasNoErrors
	"Answer true if no prior errors have been logged."

	^parseErrors isNil or: [self parseErrors isEmpty]! !

!RFC2822MIMEMessage methodsFor: 'error reporting' stamp: 'SLB 3/5/2008 12:24'!
inXMLOutputPhase
	"Did error occur while emitting XML?"

	^self isPrinting! !

!RFC2822MIMEMessage methodsFor: 'error reporting' stamp: 'SLB 3/5/2008 20:42'!
logFatalError: exception on: aStream fileName: badFileName
	"A fatal error has occured.  Print my key header data in tab-delimited form on aStream
	 followed by the error message.  Note if the parse didn't make it past the message headers
	 before failing, some of the header fields will be blank.  There are three cases
		1) fatal error only -- a) create BadMessage file, b) write error to log,  c) exit message
		   without any xml output.
		2) fatal *parsing* error after one or more header or multi-body parsing errors
		   (hence no xml has been emitted): a) ensure only one BadMessage file, b) backup in
		   log so only fatal error message prints (but print total # of errors); c) exit message
		   without any xml output
		3) fatal *xml output* error, hence some xml has alredy been emitted (should be very rare)
		   a) ensure only one BadMessage file; b) ensure fatal error message prints and print
		   total # errors; c) Somehow close all open xml tags so xml is valid."

	self printCommaDelimitedSummaryWithoutErrorsOn: aStream.
	self inXMLOutputPhase
		ifTrue: [self halt  "What to do?  Gotta create ending tags for all open message elements"].
	self hasNoErrors
		ifTrue:
			[aStream nextPutAll: ',1,'.
			 self printSummaryField:
				badFileName, ' -- Fatal error: ', exception description on: aStream]
		ifFalse:
			[aStream nextPut: $, ; nextPutAll: (self parseErrors size + 1) printString; nextPut: $, .
			 self printSummaryField:
				badFileName, ' -- Fatal error: ', exception description on: aStream].
	aStream flush! !

!RFC2822MIMEMessage methodsFor: 'error reporting' stamp: 'SLB 3/9/2008 20:25'!
printCommaDelimitedSummaryOn: aStream
	"Print my key header data in comma-delimited-valoue (csv) form on aStream."

	| headerDictionary |
	aStream cr.		"start new line"
	headerDictionary := Dictionary new.
	self messageHeaders do: [ :aHeader |
		headerDictionary at: aHeader name put: aHeader value].
	self printSummaryField: (headerDictionary at: 'from:' ifAbsent: ['Error, missing From: header'])
						 on: aStream.
	#('to:' 'date:' 'subject:') do: [ :aFieldName |
		aStream nextPut: $, .
		self printSummaryField: (headerDictionary at: aFieldName
										ifAbsent: ['--Missing Header--']) on: aStream].
	aStream nextPut: $, .
	self printSummaryField: self messageID value on: aStream.
	aStream nextPutAll: ',', self myHash asHex.
	self hasNoErrors ifFalse:
		[aStream nextPut: $, .
		 self printSummaryField: self parseErrors size printString on: aStream.
		 self folder badFileName notNil
			ifTrue:
				[aStream nextPutAll: ',' ,self folder badFileName, ' -- '.
				 self parseErrors first printCommaDelimitedOn: aStream]
			ifFalse:
				[aStream nextPut: $, .
				 self parseErrors first printCommaDelimitedOn: aStream]]! !

!RFC2822MIMEMessage methodsFor: 'error reporting' stamp: 'SLB 3/2/2008 16:47'!
recordParseErrorLocation: locString type: typeString
	"Create a new ParseError, fill it in, and add it to my collection of errors."

	self addParseError: (ParseError errorLocation: locString errorType: typeString)! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

RFC2822MIMEMessage class
	instanceVariableNames: ''!

!RFC2822MIMEMessage class methodsFor: 'random IDs' stamp: 'SLB 11/19/2007 16:23'!
randomID
	"Generate a random integer to be used as a mockup generator for local IDs.
	 Note: this is OK for the prototype, but must be replaced with a handle system
	 or other GUID generator for production."

	^ 2147483640 atRandom 	"the max number chosen to fit the range of XML type Int"! !


!RFC2822MIMEMessage class methodsFor: 'accessing' stamp: 'SLB 11/23/2007 15:57'!
CurrentMessage
	"Getter for class var that holds the currently parsing (or printing) method.  This class var
	 is a relatively clean way to make the current message model accessible to its tree of
	 component parts.  One use of this is to allow parse errors at any level in the tree to be
	 reported to the message object which tracks such things."

	^CurrentMessage! !

!RFC2822MIMEMessage class methodsFor: 'accessing' stamp: 'SLB 11/23/2007 15:57'!
CurrentMessage: anRFC2822MIMEMessage
	"Setter for class var that holds the currently parsing (or printing) method.  This class var
	 is a relatively clean way to make the current message model accessible to its tree of
	 component parts.  One use of this is to allow parse errors at any level in the tree to be
	 reported to the message object which tracks such things."

	CurrentMessage := anRFC2822MIMEMessage! !

!RFC2822MIMEMessage class methodsFor: 'accessing' stamp: 'SLB 8/13/2008 15:09'!
MaxBodyPartLength
	"Getter for class var that determines the largest string acceptable for a body part.
	 Larger strings are put out into the file system with reference to the file path/URL
	 saved.  Except for testing purposes, it is intended to be fairly large, say 25K or so,
	 to minimize the number of attachment files kept outboard of the XML parse output."
	"RFC2822MIMEMessage MaxBodyPartLength: 25000"

	 MaxBodyPartLength
		isNil ifTrue: [MaxBodyPartLength := 25000].    "default length"
	^MaxBodyPartLength! !

!RFC2822MIMEMessage class methodsFor: 'accessing' stamp: 'SLB 6/12/2007 10:43'!
MaxBodyPartLength: anInteger
	"Setter for class var that determines the largest string acceptable for a body part.
	 Larger strings are put out into the file system with reference to the file path/URL
	 saved.  Except for testing purposes, it is intended to be fairly large, say 100K or so,
	 to minimize the number of attachment files kept outboard of the XML parse output."

	MaxBodyPartLength := anInteger! !


!RFC2822MIMEMessage class methodsFor: 'instance creation' stamp: 'SLB 3/1/2008 20:43'!
folder: myFolder relativePath: folderPath
	"<nyFolder: hOf Folder, folderPath: iOf String, ^hOf self>"
	"Answer a new instance of me with the given folder and relative path.
	 This method should be the only standard way to create a new message object for parsing
	 because it also installs itself in the CurrentMessage class var."

	| me |
	me := self new.
	me folder: myFolder.
	me relativePath: folderPath.
	self CurrentMessage: me.
	^me! !


Object subclass: #SingleBody
	instanceVariableNames: 'mimeHeaders contents'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EmailParser-Model'!
!SingleBody commentStamp: 'SLB 8/31/2008 00:14' prior: 0!
A SingleBody represents one body part, its MIME headers and its content.

Structure:
 mimeHeaders	hOf OrderedCollection of: {hOf MIMIHeader} -- the mimeHeaders that characterize this body
 contents		iOf String -- the content string itself, wrapped in a CDATA section if necessary!


!SingleBody methodsFor: 'accessing' stamp: 'SLB 5/19/2007 21:13'!
contents
	"<^ (iOf String | nil)>"
	"getter"

	^contents! !

!SingleBody methodsFor: 'accessing' stamp: 'SLB 4/24/2007 21:42'!
contents: aString
	"setter"

	contents := aString! !

!SingleBody methodsFor: 'accessing' stamp: 'SLB 10/20/2007 21:13'!
mimeHeaders
	"<^ hOf OrderedCollection of: {hOf MIMIHeader}>"
	"Laissez faire getter"

	mimeHeaders isNil
		ifTrue: [mimeHeaders := OrderedCollection new].
	^mimeHeaders! !

!SingleBody methodsFor: 'accessing' stamp: 'SLB 11/16/2007 17:28'!
mimeHeaders: aDictionary
	"<aCollection: (hOf Dictionary {key: iOf String, value: hOf MsgHeader} | nil)>"
	"setter"

	mimeHeaders := aDictionary! !

!SingleBody methodsFor: 'accessing' stamp: 'SLB 12/5/2008 14:43'!
safeCDATAContents
	"<^ (iOf String | nil)>"
	"Answer a string that can be safely inserted into a CDATA body.
	 That is, a string that contains no occurrence of ']]'  If that string is,
	 present, replace it with escaped hex values of characters &#093;&#093; "

	^self contents copyReplaceAll: ']]' with: '&#093;&#093;' ! !


!SingleBody methodsFor: 'parsing' stamp: 'SLB 8/31/2008 00:15'!
parseFromStream: messageStream
	"The headers have already been parsed.  All the remaining text is the body
	 (see www.w3.org/Protocols/rfc1341/7_2_Multipart.html)."

	self contents: messageStream upToEnd.! !


!SingleBody methodsFor: 'printing' stamp: 'SLB 4/24/2007 21:44'!
printOn: aStream
	"print my contents"

	self contents printOn: aStream! !

!SingleBody methodsFor: 'printing' stamp: 'SLB 12/13/2008 13:44'!
printXMLOn: aStream enclosingIndentLevel: tabs
	"Print my contents in the mail-account.xsd format at the indicated indent level."

	| contentType |
	aStream crtab: tabs; basicNextPutAll: '<SingleBody>'.
	(contentType := self mimeHeaders at: 'content-type:' ifAbsent: [nil]) isNil
		ifTrue: [ContentTypeHeader default printXMLOn: aStream enclosingIndentLevel: tabs + 1]
		ifFalse:
			[contentType printXMLOn: aStream enclosingIndentLevel: tabs + 1.
			 contentType hasBeenPrinted: true].
	#('content-transfer-encoding:' 'content-id:' 'content-description:' 'content-disposition:')
		do: [ :headerType | 
			mimeHeaders at: headerType 
			ifPresent: [ :aHeader | aHeader printXMLOn: aStream enclosingIndentLevel: tabs + 1.
				 aHeader hasBeenPrinted: true]].
	self mimeHeaders valuesDo: [ :aMimeHeader | 	"print rest of mime headers"
		aMimeHeader hasBeenPrinted
			ifFalse: 
				[aMimeHeader printXMLOn: aStream enclosingIndentLevel: tabs + 1.
				 aMimeHeader hasBeenPrinted: true]].
	aStream crtab: tabs  + 1; basicNextPutAll: '<BodyContent>';
			 crtab: tabs + 2; basicNextPutAll: '<Content>'.		"begin int-body-content-type tags".
	self contents notNil ifTrue:
		[((self contents indexOf: $<) > 0 or: [(self contents indexOf: $&) > 0])
			ifTrue:
				[aStream basicNextPutAll: '<!![CDATA['; basicNextPutAll: self safeCDATAContents; basicNextPutAll: ']]>']
			ifFalse: [aStream basicNextPutAll: self contents]].
	aStream  crtab: tabs + 2; basicNextPutAll: '</Content>';	"end int-body-content-type tags"
			  crtab: tabs + 1; basicNextPutAll: '</BodyContent>'.
	aStream crtab: tabs; basicNextPutAll: '</SingleBody>'! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

SingleBody class
	instanceVariableNames: ''!

!SingleBody class methodsFor: 'instance creation' stamp: 'SLB 8/31/2008 00:11'!
mimeHeaders: aDictionaryOfHeaders
	"Create a new one of me with mime headers installed.  We know the
	 MIME headers when this object is created because parsing them is what
	 determines that this is a single body part"

	| me |
	me := self new.
	me mimeHeaders: aDictionaryOfHeaders.
	^me! !


Object subclass: #W3CDateTime
	instanceVariableNames: 'rfc822String dayOfMonth month year hour minute second timeZoneOffset'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EmailParser-Model'!
!W3CDateTime commentStamp: 'SLB 12/19/2008 11:26' prior: 0!
This class parses RFC822 DateTime strings and outputs W3C DateTime strings.

It depends upon the EmailParser-Misc changeset changes as of 13/18/2008 or later (EmailParser-Misc.5.cs)
   					--------------------------------------------------------
An RFC822 DateTime string is supposed to be of the form:
         date-time   =  [ day "," ] date time     ; dd mm yy
                                                     ;  hh:mm:ss zzz
         day         =  "Mon"  / "Tue" /  "Wed"  / "Thu"
                     /  "Fri"  / "Sat" /  "Sun"
         date        =  1*2DIGIT month 2DIGIT   ; day month year
                                                     ;  e.g. 20 Jun 82
         month       =  "Jan"  /  "Feb" /  "Mar"  /  "Apr"
                     /  "May"  /  "Jun" /  "Jul"  /  "Aug"
                     /  "Sep"  /  "Oct" /  "Nov"  /  "Dec"
         time        =  hour zone                    ; ANSI and Military
         hour        =  2DIGIT ":" 2DIGIT [":" 2DIGIT]
                                                    ; 00:00:00 - 23:59:59
         zone        =  "UT"  / "GMT"             ; Universal Time
                                                     ; North American : UT
                     /  "EST" / "EDT"               ;  Eastern:  - 5/ - 4
                     /  "CST" / "CDT"               ;  Central:  - 6/ - 5
                     /  "MST" / "MDT"              ;  Mountain: - 7/ - 6
                     /  "PST" / "PDT"               ;  Pacific:  - 8/ - 7
                     /  1ALPHA                    ; Military: Z = UT;
                                                     ;  A:-1; (J not used)
                                                     ;  M:-12; N:+1; Y:+12
                     / ( ("+" / "-") 4DIGIT )        ; Local differential
                                                     ;  hours+min. (HHMM)
Examples:
    Wed, 02 Oct 2002 08:00:00 EST
    Wed, 02 Oct 2002 13:00:00 GMT
    Wed, 02 Oct 2002 15:00:00 +0200

Note that there are lots of unusual semi-valid dates out there in the world.  Examples:
	1) The standard mandates day month year, yet we see  Fri, Sep 14 2001 18:39:29 +0000
	2) The "GMT" timezone is now obsolete, but is still in use, Wed, 02 Oct 2002 08:01:02 GMT
	3) Although poorly documented, numeric timezone offsets must not be larger than +/-1400.
	   yet one might encounter  13 May 2003 18:54:42 -1900  or  23 Oct 2003 17:05:02 +1500
	4) One digit minutes occur, e.g., Tue, 2 Apr 2002 20:2:57 -0800
	5) No day name, e.g., 07 Aug 2002 10:59:55 -0400 (note: day name is optional)
	6) One digit minute, e.g., Tue, 2 Apr 2002 20:2:57 -0800
	7) Month and day-of-month reversed, e.g., Fri, Sep 14 2001 18:39:29 +0000
	8) numeric timezone offsets > 14, e.g., Tue, 13 May 2003 18:54:42 -1900
	9) two digit years, e.g., Fri, 05 Dec 03 21:23:59 GMT
	10) improper capitalization, e.g., tue, 1 MAR 2005 19:00:11 -0500
	11) improper time separator, e.g., Fri, 28 Jan 2005 20.02.49
	12) improper date separator, e.g., 02-Feb-2005 17:21:26 EST
	13) improper date format, e.g., 1/31/2005 13:24
	14) non-ascii and non-english dayname, e.g., mie (accented, &#x00E9);, 4 may 2005 10:42:44
	15) Two conflicting timezones, e.g., Wed, 16 Feb 2005 11:52:54 -0500 GMT
	The parsing code attempts to handle most of these variations but cannot yet handle
	all of them.  If a parse fails, the resulting W3CDate is set to a default (see class method).
Note, bad daynames are handled by simply ignoring them since the dayname is optional and does not appear in a W3C DateTime.
Double timezones, e.g., Wed, 26 Jan 2005 10:07:26 -0500 GMT, where the first is a legal
numeric zone are handled by assuming the first timezone is correct and ignoring the
second one which in all cases seen so far is claimed to be GMT no matter what the numeric
timezone.  Non-standard separators, e.g., 02-Feb-2005, are handled by allowing the separators in Character>>isEmailDateTimeSeparator (an extension that is in the EmailParser-Misc changeset.

			-------------------------------------------------------------------------
A W3C DateTime consists of a sequences of characters of the form: '-'? yyyy '-' mm '-' dd 'T' hh ':' mm ':' ss ('.' s+)? (zzzzzz)?
    * '-'? yyyy is a four-or-more digit optionally negative-signed numeral that represents the year; if more than four digits, leading zeros are prohibited, and '0000' is prohibited (see the Note above  (§3.2.7); also note that a plus sign is not permitted);
    * the remaining '-'s are separators between parts of the date portion;
    * the first mm is a two-digit numeral that represents the month;
    * dd is a two-digit numeral that represents the day;
    * 'T' is a separator indicating that time-of-day follows;
    * hh is a two-digit numeral that represents the hour; '24' is permitted if the minutes and seconds represented are zero, and the dateTime value so represented is the first instant of the following day (the hour property of a dateTime object in the value space cannot have a value greater than 23);
    * ':' is a separator between parts of the time-of-day portion;
    * the second mm is a two-digit numeral that represents the minute;
    * ss is a two-integer-digit numeral that represents the whole seconds;
    * '.' s+ (if present) represents the fractional seconds;
    * zzzzzz (if present) represents the timezone. 
That is, YYYY-MM-DDThh:mm:ss.sTZD (eg 1997-07-16T19:20:30.45+01:00)
where TZD  = time zone designator (Z or +hh:mm or -hh:mm)
!


!W3CDateTime methodsFor: 'accessing' stamp: 'SLB 11/7/2007 20:46'!
dayOfMonth
	"Getter.  One or two digit integer <= 28, 29, 30, or 31 (depending on month)"

	^dayOfMonth! !

!W3CDateTime methodsFor: 'accessing' stamp: 'SLB 11/7/2007 20:45'!
dayOfMonth: anInteger
	"Setter.  One or two digit integer <= 28, 29, 30, or 31 (depending on month)"

	dayOfMonth := anInteger! !

!W3CDateTime methodsFor: 'accessing' stamp: 'SLB 11/7/2007 20:38'!
dayOfWeek
	"<^iOf String>"
	"Getter, returns one of Sun Mon Tue Wed Thu Fri Sat or nil"

	^dayOfWeek! !

!W3CDateTime methodsFor: 'accessing' stamp: 'SLB 11/7/2007 20:37'!
dayOfWeek: aString
	"Setter, one of Sun Mon Tue Wed Thu Fri Sat"

	dayOfWeek := aString! !

!W3CDateTime methodsFor: 'accessing' stamp: 'SLB 11/7/2007 21:06'!
hour
	"Getter."

	^hour! !

!W3CDateTime methodsFor: 'accessing' stamp: 'SLB 11/7/2007 21:08'!
hour: anInteger
	"Setter."

	hour := anInteger! !

!W3CDateTime methodsFor: 'accessing' stamp: 'SLB 11/7/2007 21:07'!
minute
	"Getter."

	^minute! !

!W3CDateTime methodsFor: 'accessing' stamp: 'SLB 11/7/2007 21:07'!
minute: anInteger
	"Setter."

	minute := anInteger! !

!W3CDateTime methodsFor: 'accessing' stamp: 'SLB 11/7/2007 20:53'!
month
	"Getter.  A positive integer <= 12"

	^month! !

!W3CDateTime methodsFor: 'accessing' stamp: 'SLB 2/20/2008 11:04'!
month: anInteger
	"setter.  Ensure it is positive integer <= 12"

	(anInteger > 0 and: [anInteger <= 12])
		ifTrue: [month := anInteger]
		ifFalse: [EmailMalformedDateException signal: 'illegal month number: ', anInteger printString]! !

!W3CDateTime methodsFor: 'accessing' stamp: 'SLB 11/7/2007 20:39'!
rfc822String
	"<^iOf String>"
	"Getter, returns the entire RFC822 DateTime string that was parsed."

	^rfc822String! !

!W3CDateTime methodsFor: 'accessing' stamp: 'SLB 11/7/2007 20:41'!
rfc822String: aString
	"Setter.  Example of such a string is:  Wed, 02 Oct 2002 08:00:00 EST"

	rfc822String := aString! !

!W3CDateTime methodsFor: 'accessing' stamp: 'SLB 11/25/2007 15:46'!
second
	"Getter.  If second is nil, return zero since second is optional in the Email time and
	 not optional in the W3C time."

	second isNil
		ifTrue: [second := 0].
	^second! !

!W3CDateTime methodsFor: 'accessing' stamp: 'SLB 11/7/2007 21:08'!
second: anInteger
	"Setter."

	second := anInteger! !

!W3CDateTime methodsFor: 'accessing' stamp: 'SLB 11/9/2007 13:15'!
timeZoneOffset
	"Getter.  time zoneOffset designator is a String consisting of 'Z' or '+hh:mm' or '-hh:mm'.
	 Z stands for 'Zulu', UT, or GMT, i.e., Genwitch mean time or Universal Time."

	^timeZoneOffset! !

!W3CDateTime methodsFor: 'accessing' stamp: 'SLB 11/8/2007 12:00'!
timeZoneOffset: aString
	"Setter.  time zoneOffset designator is Z or +hh:mm or -hh:mm.
	 Z stands for 'Zulu', UT, or GMT, i.e., Genwitch mean time or Universal Time."

	timeZoneOffset := aString! !

!W3CDateTime methodsFor: 'accessing' stamp: 'SLB 11/7/2007 21:05'!
year
	"Getter.  Should be 4 digit year."

	^year! !

!W3CDateTime methodsFor: 'accessing' stamp: 'SLB 11/7/2007 21:04'!
year: anInteger
	"setter.  Should be 4 digit year."

	year := anInteger! !


!W3CDateTime methodsFor: 'printing' stamp: 'SLB 11/9/2007 11:29'!
printOn: aStream
	"Print myself in a human-friendly way on the stream."

	aStream nextPutAll: 'rfc822String: '.
	rfc822String printOn: aStream.
	aStream cr.
	dayOfWeek printOn: aStream.
	aStream space.
	dayOfMonth printOn: aStream.
	aStream space.
	month printOn: aStream.
	aStream space.
	year printOn: aStream.
	aStream space.
	hour printOn: aStream.
	aStream space.
	minute printOn: aStream.
	aStream space.
	second printOn: aStream.
	aStream space.
	timeZoneOffset printOn: aStream.! !

!W3CDateTime methodsFor: 'printing' stamp: 'SLB 12/15/2008 19:15'!
printW3CDateTimeFormatOn: aStream
	"Print myself in W3C DateTime format on the stream.  That format
	 is, YYYY-MM-DDThh:mm:ss.sTZD (eg 1997-07-16T19:20:30.45+01:00)
	 where TZD  = time zone designator (Z or +hh:mm or -hh:mm)"

	self nonEmpty
		ifTrue:
			[aStream basicNextPutAll: self year printString, '-'.
			self month < 10
				ifTrue: [aStream nextPut: $0].
			aStream basicNextPutAll: self month printString, '-'.
			self dayOfMonth < 10
				ifTrue: [aStream nextPut: $0].
			aStream basicNextPutAll: self dayOfMonth printString, 'T'.
			self hour < 10
				ifTrue: [aStream nextPut: $0].
			aStream basicNextPutAll: self hour printString, ':'.
			self minute < 10
				ifTrue: [aStream nextPut: $0].
			aStream basicNextPutAll: self minute printString, ':'.
			self second < 10
				ifTrue: [aStream nextPut: $0].
			aStream basicNextPutAll: second printString, self timeZoneOffset]
		ifFalse:
			[aStream basicNextPutAll: '- empty w3c dateTime -'].! !


!W3CDateTime methodsFor: 'parsing' stamp: 'SLB 1/13/2008 21:28'!
getDayOfWeekFrom: dateStream
	"Get the day field of an RFC822 DateTime stream that is positioned just before it..
	 The whole string looks roughly like this:  'Wed, 02 Oct 2002 13:00:00 GMT', with variations
	 on the timeZone info."

	dateStream peek isLetter		"If not, there is no dayname at start of date."
		ifTrue:
			[self dayOfWeek: (dateStream next: 3).
			(#('Mon' 'Tue' 'Wed' 'Thu' 'Fri' 'Sat' 'Sun') indexOf: self dayOfWeek) > 0
				ifFalse:
					[EmailMalformedDateException
						signal: 'illegal day-of-week string: ',self dayOfWeek].
			dateStream skip: 1.  "Skip the comma or space following the day string"]
	! !

!W3CDateTime methodsFor: 'parsing' stamp: 'SLB 12/18/2008 11:03'!
getMonthFrom: dateStream
	"Get the month field of an RFC822 DateTime stream that is positioned just before it..
	 The whole string looks roughly like this:  'Wed, 02 Oct 2002 13:00:00 GMT', with
	 variations."

	| monthString monthNumber |
	monthString := (dateStream skipSeparators; next: 3) asLowercase.
	(monthNumber :=
		#('jan' 'feb' 'mar' 'apr' 'may' 'jun' 'jul' 'aug' 'sep' 'oct' 'nov' 'dec')
			indexOf: monthString) = 0
				ifTrue: [EmailMalformedDateException signal: 'illegal month string: ', monthString].
	self month: monthNumber.
	! !

!W3CDateTime methodsFor: 'parsing' stamp: 'SLB 12/3/2008 22:36'!
getTimeFrom: dateStream
	"Get the year field of an RFC822 DateTime stream that is positioned just before it.
	 The whole string looks roughly like this:  'Wed, 02 Oct 2002 13:00:00 GMT', with variations
	 on the timeZone info.  The year may occasionally be only two digits, e.g., 03 for 2003."

	| |
	[self hour: (dateStream upTo: $:) asInteger.
	self minute: (dateStream next asString) asInteger.   "on 20.02.49 this grabs rest of string"
	dateStream peek isDigit
		ifTrue: [self minute: (self minute * 10 + (dateStream next asString asInteger))].
	(dateStream peekFor: $:)  ifTrue: [self second: (dateStream next: 2) asInteger]]
		on: Error
		do: [ :ex |	"install default values"
			self hour: 0;
				minute: 0;
				second: 0].! !

!W3CDateTime methodsFor: 'parsing' stamp: 'SLB 12/18/2008 17:50'!
getTimeZoneOffsetFrom: dateStream
	"Get the time zone offset from the time zone field of an RFC822 DateTime stream that is 
	 positioned just before it..
	 The whole string looks roughly like this:  'Wed, 02 Oct 2002 13:00:00 -0500'.
	 The three legal variations on the timeZone info are:
	 1) a two or three letter timezone name, e.g., UT, GMT, EST, PDT.,
	 2) a one letter 'military time' designator, e.g., Z=0, A= -1, M= -12, (J not used), n= +1, Y= +12,
	 3) +/- a four digit offset in the form HHMM.  However, StylusStudio claims that timezones
	    like -19:00 or -20:00 are improper.  So if abs tz > 14, convert to opposite sign
	Note: AspMail mailer has quoted the timezone, e.g., ""GMT""
		  and several bad dates are due to a numeric followed by a GMT timezone."

	| tzString firstChar index |
	tzString := (dateStream skipSeparators; upToEnd) asUppercase copyWithout: $". "remove quoting"
	(tzString isNil or: [tzString isEmpty])
		ifTrue:
			[self timeZoneOffset: 'Z'.
			 ^self].
	firstChar := tzString at: 1.
	(firstChar = $+ or: [firstChar = $-])
		ifTrue:
			[(tzString size >= 5 and: [(tzString copyFrom: 2 to: 5) isAllDigits])
				ifTrue:
					[^self setLegalTimeZoneOffsetFirst: firstChar
								hr: (tzString copyFrom: 2 to: 3)
								min: (tzString copyFrom: 4 to: 5)]].
	index := #('UT' 'GMT' 'EST' 'EDT' 'CST' 'CDT' 'MST' 'MDT' 'PST' 'PDT'
				'Z' 'A' 'B' 'C' 'D' 'E' 'F' 'G' 'H' 'I' 'K' 'L' 'M' 'N' 'O' 'P'
				'Q' 'R' 'S' 'T' 'U' 'V' 'W' 'X' 'Y' 'US/EASTERN') indexOf: tzString.
	index = 0
		ifTrue:
			[self timeZoneOffset: 'Z'.  "set for default timezone"
			 EmailMalformedDateException signal: 'Warning - illegal time zone: ', tzString, ' Assuming GMT'].
	self timeZoneOffset:
		(#('Z' 'Z' '-05:00' '-04:00' '-06:00' '-05:00' '-07:00' '-06:00' '-08:00' '-07:00' 'Z' '-01:00' '-02:00' '-03:00' '-04:00' '-05:00' '-06:00' '-07:00' '-08:00' '-09:00' '-10:00' '-11:00' '-12:00' '+01:00' '+02:00' '+03:00' '+04:00' '+05:00' '+06:00' '+07:00' '+08:00' '+09:00' '+10:00' '+11:00' '+12:00' '-04:00')
		at: index).
	! !

!W3CDateTime methodsFor: 'parsing' stamp: 'SLB 8/31/2008 22:34'!
getYearFrom: dateStream
	"Get the year field of an RFC822 DateTime stream that is positioned just before it.
	 The whole string looks roughly like this:  'Wed, 02 Oct 2002 13:00:00 GMT', with variations
	 on the timeZone info.  The year may occasionally be only two digits, e.g., 03 for 2003."

	| yearNumber |
	yearNumber := (dateStream upTo: Character space) asInteger.
	yearNumber < 72	"email did not exist prior to 1972"
		ifTrue:
			[yearNumber := yearNumber + 2000]  "assume 21st century"
		ifFalse:
			[yearNumber < 100
				ifTrue: [yearNumber := yearNumber + 1900]].  "assume 20th century"
	self year: yearNumber.
	! !

!W3CDateTime methodsFor: 'parsing' stamp: 'SLB 12/3/2008 21:56'!
guessFromRFC2822String: dateTimeString
	"<dateTimeString: iOf String, ^hOf W3CDateTime>"
	"One or more fields of the RFC822 DateTime string are bogus.  Attempt to guess
	 what they might be and install the guess into my instVars. Install '01' as the
	 value of any guesses that fail completely.
	 The string should look roughly like this:  'Wed, 02 Oct 2002 13:00:00 GMT'
	 In addition to the canonical format above, other anomalous
	 formats, legal or not, do occur in real email:
		1) No day name, e.g., 07 Aug 2002 10:59:55 -0400 (note: day name is optional)
		2) One digit minute, e.g., Tue, 2 Apr 2002 20:2:57 -0800
		3) Month and day-of-month reversed, e.g., Fri, Sep 14 2001 18:39:29 +0000
		4) numeric timezone offsets > 14, e.g., Tue, 13 May 2003 18:54:42 -1900
		5) two digit years, e.g., Fri, 05 Dec 03 21:23:59 GMT
		6) improper capitalization, e.g., tue, 1 MAR 2005 19:00:11 -0500
		7) improper time separator, e.g., Fri, 28 Jan 2005 20.02.49
		8) improper date separator, e.g., 02-Feb-2005 17:21:26 EST
		9) improper date format, e.g., 1/31/2005 13:24
	    10) non-ascii and non-english dayname, e.g., mi&#x00E9;, 4 may 2005 10:42:44
		   (which is a Spanish or French day name abreviation with accented e)
	    11) Two conflicting timezones, e.g., Wed, 16 Feb 2005 11:52:54 -0500 GMT
	This code attempts to handle some of them and set the date to a default if it
	runs into an error it can't handle."

"W3CDateTime exampleRFC822TestBadDateTime1 inspect." "example of  #3 above"

	| tokens |
	self rfc822String: dateTimeString.
	tokens := self legalTokensFrom: (self rfc822String findTokens: $ ). "to be used to remove illegal fields.  Not used yet"
	Transcript cr; show: 'Illegal date string: ', dateTimeString.
	^W3CDateTime default

	
! !

!W3CDateTime methodsFor: 'parsing' stamp: 'SLB 11/23/2008 22:47'!
isPlausibleDateToken: token
	"For use in guessing something like the correct date and time.
	 Answer true if token could belong to a date.  NOTE: this is a kludge designed
	to handle the one example of problem date strings we have seen so far.
	 As new examples surface, this needs to get smarter.
	 Examples of the string that got us here this time is:
	'mi&#x00E9;, 4 may 2005 10:42:44', i.e., some sort of garbage prepended to
	 an otherwise sorta legal date. "

	| daynames monthnames possibleTimeTokens |
	token isAllDigits
		ifTrue: [^true].
	daynames := #('mon' 'tue' 'wed' 'thu' 'fri' 'sat' 'sun' 'mon,' 'tue,' 'wed,' 'thu,' 'fri,' 'sat,' 'sun,').
	(daynames includes: token asLowercase)
		ifTrue: [^true].
	monthnames := #('jan' 'feb' 'mar' 'apr' 'may' 'jun' 'jul' 'aug' 'sep' 'oct' 'nov' 'dec' 'jan,' 'feb,' 'mar,' 'apr,' 'may,' 'jun,' 'jul,' 'aug,' 'sep,' 'oct,' 'nov,' 'dec,').
	(monthnames includes: token asLowercase)
		ifTrue: [^true].
	possibleTimeTokens := token findTokens: $:.
	(possibleTimeTokens size = 3
		and: [(possibleTimeTokens at: 1) isAllDigits]
		and: [(possibleTimeTokens at: 2) isAllDigits]
		and: [(possibleTimeTokens at: 3) isAllDigits])
			ifTrue: [^true].
	^false "NOTE not dealt with timezone yet"
	
! !

!W3CDateTime methodsFor: 'parsing' stamp: 'SLB 11/23/2008 22:37'!
legalTokensFrom: collectionOfDateTokens
	"For use in guessing something like the correct date and time.
	 Thus we already know that the tokens have problems or we wouldn't be here..
	 Collect and return tokens that might be days, months or times or timezones.
	 Eliminate those that don't look like they could be parts of date-times.
	 Examples of strings that get us here include:
	'mi&#x00E9;, 4 may 2005 10:42:44'"
	"W3CDateTime new legalTokensFrom: ('mi&#x00E9;, 4 may 2005 10:42:44' findTokens: $ )"

	^collectionOfDateTokens select: [ :token |
		self isPlausibleDateToken: token]
	
! !

!W3CDateTime methodsFor: 'parsing' stamp: 'SLB 12/18/2008 20:12'!
parseFromRFC822String: dateTimeString
	"Parse out the various fields of an RFC822 DateTime string and install in my instVars.
	 The string looks roughly like this:  'Wed, 02 Oct 2002 13:00:00 GMT', with variations
	 on the timeZone info.  In addition to the canonical format above, other anomalous
	 formats, legal or not, do occur in real email:
		1) No day name, e.g., 07 Aug 2002 10:59:55 -0400 (note: day name is optional)
		2) One digit minute, e.g., Tue, 2 Apr 2002 20:2:57 -0800
		3) Month and day-of-month reversed, e.g., Fri, Sep 14 2001 18:39:29 +0000
		4) numeric timezone offsets > 14, e.g., Tue, 13 May 2003 18:54:42 -1900
		5) two digit years, e.g., Fri, 05 Dec 03 21:23:59 GMT
		6) improper capitalization, e.g., tue, 1 MAR 2005 19:00:11 -0500
		7) improper timezone, e.g., Wed, 13 Feb 2000 18:09:08 US/EASTERN
		8) improper date separator, e.g., 02-Feb-2005 17:21:26 EST
		9) improper time separator, e.g., Fri, 28 Jan 2005 20.02.49
	    10) improper date format, e.g., 1/31/2005 13:24
	    11) non-ascii and non-english dayname, e.g., mi&#x00E9;, 4 may 2005 10:42:44
		   (which is a Spanish day name abreviation with accented e)
	    12) Two conflicting timezones, e.g., Wed, 16 Feb 2005 11:52:54 -0500 GMT
	This code attempts to handle all variations, but as of 12/18/2008, 9-10 are not done."

"W3CDateTime exampleRFC822TestDateTime12 inspect." "example of  #3 above"

	| dateStream |
	self rfc822String: dateTimeString.
	dateStream := ReadStream on: dateTimeString withoutTrailingBlanks.
	self skipDayOfWeekFrom: dateStream.  "day of week, if present, not used in W3CDate"
	dateStream peek isLetter
		ifTrue:		"It is Month/day/year form"
			[self getMonthFrom: dateStream.
			 dateStream skipEmailDateTimeSeparators.
			 self dayOfMonth: (dateStream next: 2) asInteger.]
		ifFalse: 		"It is day/month/year form"
			[self dayOfMonth: (dateStream next: 2) asInteger.
			 dateStream skipEmailDateTimeSeparators.
			 self getMonthFrom: dateStream].
	dateStream skipEmailDateTimeSeparators.
	self getYearFrom: dateStream.
	self getTimeFrom: dateStream.
	dateStream skipSeparators.
	self getTimeZoneOffsetFrom: dateStream

	! !

!W3CDateTime methodsFor: 'parsing' stamp: 'SLB 3/11/2008 09:11'!
setLegalTimeZoneOffsetFirst: firstChar hr: hrString min: minString
	"Setter.  time zoneOffset designator is +hh:mm or -hh:mm.
	 Ensure that hh is <= 14.  E.g., convert -1900 to +500"
	"W3CDateTime exampleRFC822TestDateTime14 inspect."
	"W3CDateTime exampleRFC822TestDateTime15 inspect."

	| legalString hours |
	hours := hrString asInteger.
	hours > 14
		ifTrue:		"convert to opposite sign modulo 24"
			[firstChar = $-		"swap sign character"
				ifTrue: [legalString := '+0', (24 - hours) printString, ':', minString]
			 	ifFalse: [legalString := '-0', (24 - hours) printString, ':', minString]]
		ifFalse:
			[legalString := firstChar asString, hrString, ':', minString].
	self timeZoneOffset: legalString! !

!W3CDateTime methodsFor: 'parsing' stamp: 'SLB 12/17/2008 22:38'!
skipDayOfWeekFrom: dateStream
	"Get past the day field of an RFC822 DateTime stream, if present.
	 dateStream should be positioned just before it.. The whole string looks
	 roughly like this:  'Wed, 02 Oct 2002 13:00:00 GMT', with variations
	 on the timeZone info.  Day of week is optional."

	dateStream skipSeparators.		"Just in case there is a blank before dayname"
	dateStream peek isLetter		"If not, there is no dayname at start of date."
		ifTrue:
			[(dateStream upTo: $,).		"Skip the dayname itself"
			 dateStream skip: 1].		  	"Skip the comma or space following the day string"
	dateStream skipSeparators.
	


"DAY OF WEEK NOT USED, SO OPTIMIZE FOR IGNORING BAD ONES
	self dayOfWeek: (dateStream next: 3) asLowercase.
			(#('mon' 'tue' 'wed' 'thu' 'fri' 'sat' 'sun') indexOf: self dayOfWeek) > 0
				ifFalse:
					[EmailMalformedDateException
						signal: 'illegal day-of-week string: ',self dayOfWeek].
			dateStream skip: 1.]"! !


!W3CDateTime methodsFor: 'queries' stamp: 'SLB 2/19/2008 20:32'!
nonEmpty
	"Answer whether my parse has begun"

	^self rfc822String notNil! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

W3CDateTime class
	instanceVariableNames: ''!

!W3CDateTime class methodsFor: 'testing' stamp: 'SLB 11/22/2008 21:20'!
exampleRFC822TestBadDateTime1
	"Parse example.  Note: Z is military notation for UT, i.e.,g Grenwitch time."
	"W3CDateTime exampleRFC822TestBadDateTime1 inspect."

	| me |
	me := self new.
	me parseFromRFC822String: 'mi&#x00E9;, 4 may 2005 10:42:44'.
	^me! !

!W3CDateTime class methodsFor: 'testing' stamp: 'SLB 1/13/2008 12:55'!
exampleRFC822TestDateTime1
	"Parse example"
	"W3CDateTime exampleRFC822TestDateTime1 inspect."

	| me |
	me := self new.
	me parseFromRFC822String: 'Wed, 02 Oct 2002 08:01:02 EST'.
	^me! !

!W3CDateTime class methodsFor: 'testing' stamp: 'SLB 1/13/2008 21:49'!
exampleRFC822TestDateTime10
	"Parse example with missing day name"
	"W3CDateTime exampleRFC822TestDateTime10 inspect."

	| me |
	me := self new.
	me parseFromRFC822String: '07 Aug 2002 10:59:55 -0400'.
	^me! !

!W3CDateTime class methodsFor: 'testing' stamp: 'SLB 1/14/2008 21:16'!
exampleRFC822TestDateTime11
	"Parse example with obsolete timezone"
	"W3CDateTime exampleRFC822TestDateTime11 inspect."

	| me |
	me := self new.
	me parseFromRFC822String: 'Wed, 02 Oct 2002 08:01:02 GMT'.
	^me! !

!W3CDateTime class methodsFor: 'testing' stamp: 'SLB 1/19/2008 21:32'!
exampleRFC822TestDateTime12
	"Parse example with Month before day."
	"W3CDateTime exampleRFC822TestDateTime12 inspect."

	| me |
	me := self new.
	me parseFromRFC822String: 'Fri, Sep 14 2001 18:39:29 +0000'.
	^me! !

!W3CDateTime class methodsFor: 'testing' stamp: 'SLB 3/10/2008 18:46'!
exampleRFC822TestDateTime13
	"Parse example with two digit year."
	"W3CDateTime exampleRFC822TestDateTime13 inspect."

	| me |
	me := self new.
	me parseFromRFC822String: 'Fri, 05 Dec 03 21:23:59 GMT
'.
	^me! !

!W3CDateTime class methodsFor: 'testing' stamp: 'SLB 3/11/2008 09:06'!
exampleRFC822TestDateTime14
	"Parse example with timezone magnitude greater than 14."
	"W3CDateTime exampleRFC822TestDateTime14 inspect."

	| me |
	me := self new.
	me parseFromRFC822String: 'Tue, 13 May 2003 18:54:42 -1900'.
	^me! !

!W3CDateTime class methodsFor: 'testing' stamp: 'SLB 3/11/2008 09:11'!
exampleRFC822TestDateTime15
	"Parse example with timezone magnitude greater than 14."
	"W3CDateTime exampleRFC822TestDateTime15 inspect."

	| me |
	me := self new.
	me parseFromRFC822String: 'Tue, 13 May 2003 18:54:42 +1500'.
	^me! !

!W3CDateTime class methodsFor: 'testing' stamp: 'SLB 3/11/2008 09:39'!
exampleRFC822TestDateTime16
	"Parse example with two digit year."
	"W3CDateTime exampleRFC822TestDateTime16 inspect."

	| me |
	me := self new.
	me parseFromRFC822String: 'Tue, 13 May 03 18:54:42 -0500'.
	^me! !

!W3CDateTime class methodsFor: 'testing' stamp: 'SLB 3/11/2008 09:49'!
exampleRFC822TestDateTime17
	"Parse example with two digit year."
	"W3CDateTime exampleRFC822TestDateTime17 inspect."

	| me |
	me := self new.
	me parseFromRFC822String: 'Tue, 13 May 98 18:54:42 -0500'.
	^me! !

!W3CDateTime class methodsFor: 'testing' stamp: 'SLB 11/7/2007 23:03'!
exampleRFC822TestDateTime2
	"Parse example"
	"W3CDateTime exampleRFC822TestDateTime2 inspect."

	| me |
	me := self new.
	me parseFromRFC822String: 'Wed, 02 Oct 2002 13:01:02 GMT'.
	^me! !

!W3CDateTime class methodsFor: 'testing' stamp: 'SLB 11/7/2007 23:03'!
exampleRFC822TestDateTime3
	"Parse example.  Note: P is military notation for +3, i.e., three hours East of Grenwitch."
	"W3CDateTime exampleRFC822TestDateTime3 inspect."

	| me |
	me := self new.
	me parseFromRFC822String: 'Wed, 02 Oct 2002 16:01:02 P'.
	^me! !

!W3CDateTime class methodsFor: 'testing' stamp: 'SLB 11/7/2007 23:04'!
exampleRFC822TestDateTime4
	"Parse example.  Note: -0500 timezone means five hours West of Grenwitch, i.e.,
	 EST or CDT "
	"W3CDateTime exampleRFC822TestDateTime4 inspect."

	| me |
	me := self new.
	me parseFromRFC822String: 'Tue, 13 Feb 2007 09:58:02 -0500'.
	^me! !

!W3CDateTime class methodsFor: 'testing' stamp: 'SLB 11/7/2007 22:13'!
exampleRFC822TestDateTime5
	"Parse example.  Note: Z is military notation for UT, i.e.,g Grenwitch time."
	"W3CDateTime exampleRFC822TestDateTime5 inspect."

	| me |
	me := self new.
	me parseFromRFC822String: 'Tue, 13 Feb 2007 09:58:00 Z'.
	^me! !

!W3CDateTime class methodsFor: 'testing' stamp: 'SLB 11/7/2007 22:13'!
exampleRFC822TestDateTime6
	"Parse example.  Note: E is military notation for -0500 i.e. five hours West of Grenwitch,
	 equivalent to EST or CDT "
	"W3CDateTime exampleRFC822TestDateTime6 inspect."

	| me |
	me := self new.
	me parseFromRFC822String: 'Tue, 13 Feb 2007 09:58:00 E'.
	^me! !

!W3CDateTime class methodsFor: 'testing' stamp: 'SLB 11/25/2007 15:29'!
exampleRFC822TestDateTime7
	"Parse example.  Note, seconds are optional"
	"W3CDateTime exampleRFC822TestDateTime7 inspect."

	| me |
	me := self new.
	me parseFromRFC822String: 'Tue, 18 Sep 2001 06:52 -0700'.
	^me! !

!W3CDateTime class methodsFor: 'testing' stamp: 'SLB 11/25/2007 18:25'!
exampleRFC822TestDateTime8
	"Parse example.  Note, seconds are optional"
	"W3CDateTime exampleRFC822TestDateTime8 inspect."

	| me |
	me := self new.
	me parseFromRFC822String: 'Tue, 18 Sep 2001  6:58 -0500'.
	^me! !

!W3CDateTime class methodsFor: 'testing' stamp: 'SLB 1/13/2008 21:46'!
exampleRFC822TestDateTime9
	"Parse example.  Note, seconds are optional"
	"W3CDateTime exampleRFC822TestDateTime9 inspect."

	| me |
	me := self new.
	me parseFromRFC822String: 'Tue, 2 Apr 2002 20:2:57 -0800'.
	^me! !


!W3CDateTime class methodsFor: 'instance creation' stamp: 'SLB 12/2/2008 20:51'!
default
	"Answer an instance of me with default values."
	"W3CDateTime default inspect."

	| me |
	me := self new.
	me parseFromRFC822String: 'Tue, 1 Jan 1980 00:00:00 -0400'.
	^me! !


MsgHeader subclass: #XHeader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EmailParser-Model'!
!XHeader commentStamp: 'SLB 11/21/2007 09:37' prior: 0!
X-headers are determined and generated by different email client software for their own purposes.  Some X-headers disobey some rules used by other headers.  For example, X-UIDL headers used as unique IDs by POP3 servers have apparently random strings of characters as their IDs.  And the string may contain an open Parens, otherwise a signal for the beginning of a header comment.  Because of such oddities and the fact that the contents of X-headers are strictly for consumption by their own email client software, i.e., not for any human consumption. This class short circuits any parsing of the value string.!


!XHeader methodsFor: 'parsing' stamp: 'SLB 11/21/2007 10:22'!
parseFromStream: aReadWriteStream withFirstLine: aString
    "Warning: The stream may contain only the text for one header or may be much larger.
	Note: X-Headers don't parse comments hence don't need comments removed."

    self getOrigStringFrom: aReadWriteStream withFirstLine: aString.
    ^self parseFromCleanedHeaderStream: self unfoldedStream! !
